/*******************************************************************************
 * Copyright (c) 2004, 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.core.internal.content.FileSpec;

import dwt.dwthelper.utils;

import dwtx.core.runtime.content.IContentType;

/**
 * Provides a uniform representation for file specifications, such 
 * as file names, file extensions and regular expressions.
 */
class FileSpec {
    final static int BASIC_TYPE = IContentType.FILE_EXTENSION_SPEC | IContentType.FILE_NAME_SPEC;
    private String text;
    private int type;

    public FileSpec(String text, int type) {
        this.text = text;
        this.type = type;
    }

    public String getText() {
        return text;
    }

    public int getType() {
        return type;
    }

    public static int getBasicType(int type) {
        return BASIC_TYPE & type;
    }

    public bool equals(Object other) {
        if (!(other instanceof FileSpec))
            return false;
        FileSpec otherFileSpec = (FileSpec) other;
        return equals(text, otherFileSpec.getType(), false);
    }

    public bool equals(final String text, final int otherType, final bool strict) {
        return ((!strict && getBasicType(type) is getBasicType(otherType)) || type is otherType) && this.text.equalsIgnoreCase(text);
    }

    public int hashCode() {
        return text.hashCode();
    }

    public static String getMappingKeyFor(String fileSpecText) {
        return fileSpecText.toLowerCase();
    }

    public String toString() {
        return getText();
    }
}
