/*******************************************************************************
 * Copyright (c) 2004, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.core.internal.content.TextContentDescriber;

import dwt.dwthelper.utils;

import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;

import dwtx.core.runtime.QualifiedName;
import dwtx.core.runtime.content.IContentDescription;
import dwtx.core.runtime.content.ITextContentDescriber;

/**
 * This class provides internal basis for text-based content describers. 
 * 
 * <p>
 * Note: do not add protected/public members to this class if you don't intend to 
 * make them public API.
 * </p>  
 * 
 * @see dwtx.core.runtime.content.XMLRootElementContentDescriber2
 * @since 3.0
 */
public class TextContentDescriber implements ITextContentDescriber {

    private final static QualifiedName[] SUPPORTED_OPTIONS = {IContentDescription.BYTE_ORDER_MARK};

    /*
     *  (non-Javadoc)
     * @see dwtx.core.runtime.content.ITextContentDescriber#describe(java.io.Reader, dwtx.core.runtime.content.IContentDescription)
     */
    public int describe(Reader contents, IContentDescription description) throws IOException {
        // we want to be pretty loose on detecting the text content type  
        return INDETERMINATE;
    }

    /*
     *  (non-Javadoc)
     * @see dwtx.core.runtime.content.IContentDescriber#describe(java.io.InputStream, dwtx.core.runtime.content.IContentDescription)
     */
    public int describe(InputStream contents, IContentDescription description) throws IOException {
        if (description is null || !description.isRequested(IContentDescription.BYTE_ORDER_MARK))
            return INDETERMINATE;
        byte[] bom = getByteOrderMark(contents);
        if (bom !is null)
            description.setProperty(IContentDescription.BYTE_ORDER_MARK, bom);
        // we want to be pretty loose on detecting the text content type        
        return INDETERMINATE;
    }

    /*
     *  (non-Javadoc)
     * @see dwtx.core.runtime.content.IContentDescriber#getSupportedOptions()
     */
    public QualifiedName[] getSupportedOptions() {
        return SUPPORTED_OPTIONS;
    }

    byte[] getByteOrderMark(InputStream input) throws IOException {
        int first = input.read();
        if (first is 0xEF) {
            //look for the UTF-8 Byte Order Mark (BOM)
            int second = input.read();
            int third = input.read();
            if (second is 0xBB && third is 0xBF)
                return IContentDescription.BOM_UTF_8;
        } else if (first is 0xFE) {
            //look for the UTF-16 BOM
            if (input.read() is 0xFF)
                return IContentDescription.BOM_UTF_16BE;
        } else if (first is 0xFF) {
            if (input.read() is 0xFE)
                return IContentDescription.BOM_UTF_16LE;
        }
        return null;
    }
}
