/*******************************************************************************
 * Copyright (c) 2007 Tom Schindl and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Tom Schindl - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 ******************************************************************************/

module dwtx.core.internal.databinding.Activator;

import dwt.dwthelper.utils;

import java.util.ArrayList;

import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.util.tracker.ServiceTracker;

import dwtx.core.databinding.util.ILogger;
import dwtx.core.databinding.util.Policy;
import dwtx.core.runtime.CoreException;
import dwtx.core.runtime.IStatus;
import dwtx.osgi.framework.log.FrameworkLog;
import dwtx.osgi.framework.log.FrameworkLogEntry;

/**
 * @since 3.3
 * 
 */
public class Activator implements BundleActivator {
    /**
     * The plug-in ID
     */
    public static final String PLUGIN_ID = "dwtx.core.databinding"; //$NON-NLS-1$

    private volatile static ServiceTracker _frameworkLogTracker;

    /**
     * The constructor
     */
    public Activator() {
    }

    public void start(BundleContext context) throws Exception {
        _frameworkLogTracker = new ServiceTracker(context, FrameworkLog.class.getName(), null);
        _frameworkLogTracker.open();

        Policy.setLog(new ILogger() {

            public void log(IStatus status) {
                ServiceTracker frameworkLogTracker = _frameworkLogTracker;
                FrameworkLog log = frameworkLogTracker is null ? null : (FrameworkLog) frameworkLogTracker.getService();
                if (log !is null) {
                    log.log(createLogEntry(status));
                } else {
                    // fall back to System.err
                    System.err.println(status.getPlugin() + " - " + status.getCode() + " - " + status.getMessage());  //$NON-NLS-1$//$NON-NLS-2$
                    if( status.getException() !is null ) {
                        status.getException().printStackTrace(System.err);
                    }
                }
            }

        });
    }
    
    // Code copied from PlatformLogWriter.getLog(). Why is logging an IStatus so
    // hard?
    FrameworkLogEntry createLogEntry(IStatus status) {
        Throwable t = status.getException();
        ArrayList childlist = new ArrayList();

        int stackCode = t instanceof CoreException ? 1 : 0;
        // ensure a substatus inside a CoreException is properly logged 
        if (stackCode is 1) {
            IStatus coreStatus = ((CoreException) t).getStatus();
            if (coreStatus !is null) {
                childlist.add(createLogEntry(coreStatus));
            }
        }

        if (status.isMultiStatus()) {
            IStatus[] children = status.getChildren();
            for (int i = 0; i < children.length; i++) {
                childlist.add(createLogEntry(children[i]));
            }
        }

        FrameworkLogEntry[] children = (FrameworkLogEntry[]) (childlist.size() is 0 ? null : childlist.toArray(new FrameworkLogEntry[childlist.size()]));

        return new FrameworkLogEntry(status.getPlugin(), status.getSeverity(), status.getCode(), status.getMessage(), stackCode, t, children);
    }

    
    public void stop(BundleContext context) throws Exception {
        if (_frameworkLogTracker !is null) {
            _frameworkLogTracker.close();
            _frameworkLogTracker = null;
        }
    }

}
