/*******************************************************************************
 * Copyright (c) 2007 Matt Carter and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Matt Carter - initial API and implementation
 *     Tom Schindl<tom.schindl@bestsolution.at> - bugfix for 217940
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 ******************************************************************************/

module dwtx.core.internal.databinding.validation.StringToCharacterValidator;

import dwt.dwthelper.utils;

import dwtx.core.databinding.validation.IValidator;
import dwtx.core.databinding.validation.ValidationStatus;
import dwtx.core.internal.databinding.BindingMessages;
import dwtx.core.internal.databinding.conversion.StringToCharacterConverter;
import dwtx.core.runtime.IStatus;
import dwtx.core.runtime.Status;

/**
 * Validates a String to Character conversion.
 */
public class StringToCharacterValidator implements IValidator {

    private final StringToCharacterConverter converter;

    /**
     * @param converter
     */
    public StringToCharacterValidator(StringToCharacterConverter converter) {
        this.converter = converter;
    }

    /*
     * (non-Javadoc)
     *
     * @see dwtx.core.databinding.validation.IValidator#validate(java.lang.Object)
     */
    public IStatus validate(Object value) {
        try {
            converter.convert(value);
        } catch (IllegalArgumentException e) {
            // The StringToCharacterConverter throws an IllegalArgumentException
            // if it cannot convert.
            return ValidationStatus.error(BindingMessages
                    .getString(BindingMessages.VALIDATE_CHARACTER_HELP));
        }
        return Status.OK_STATUS;
    }

}
