/*******************************************************************************
 * Copyright (c) 2000, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.core.internal.expressions.CompositeExpression;

import dwt.dwthelper.utils;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import dwtx.core.expressions.EvaluationResult;
import dwtx.core.expressions.Expression;
import dwtx.core.expressions.ExpressionInfo;
import dwtx.core.expressions.IEvaluationContext;
import dwtx.core.runtime.CoreException;

public abstract class CompositeExpression extends Expression {

    private static final Expression[] EMPTY_ARRAY = new Expression[0];

    /**
     * The seed for the hash code for all composite expressions.
     */
    private static final int HASH_INITIAL= CompositeExpression.class.getName().hashCode();
    
    protected List fExpressions;
    
    public void add(Expression expression) {
        if (fExpressions is null)
            fExpressions= new ArrayList(2);
        fExpressions.add(expression);
    }
    
    public Expression[] getChildren() {
        if (fExpressions is null)
            return EMPTY_ARRAY;
        return (Expression[])fExpressions.toArray(new Expression[fExpressions.size()]);
    }
    
    protected EvaluationResult evaluateAnd(IEvaluationContext scope) throws CoreException {
        if (fExpressions is null)
            return EvaluationResult.TRUE;
        EvaluationResult result= EvaluationResult.TRUE;
        for (Iterator iter= fExpressions.iterator(); iter.hasNext();) {
            Expression expression= (Expression)iter.next();
            result= result.and(expression.evaluate(scope));
            // keep iterating even if we have a not loaded found. It can be
            // that we find a false which will result in a better result.
            if (result is EvaluationResult.FALSE)
                return result;
        }
        return result;
    }
    
    protected EvaluationResult evaluateOr(IEvaluationContext scope) throws CoreException {
        if (fExpressions is null)
            return EvaluationResult.TRUE;
        EvaluationResult result= EvaluationResult.FALSE;
        for (Iterator iter= fExpressions.iterator(); iter.hasNext();) {
            Expression expression= (Expression)iter.next();
            result= result.or(expression.evaluate(scope));
            if (result is EvaluationResult.TRUE)
                return result;
        }
        return result;
    }
    
    public void collectExpressionInfo(ExpressionInfo info) {
        if (fExpressions is null)
            return;
        for (Iterator iter= fExpressions.iterator(); iter.hasNext();) {
            Expression expression= (Expression)iter.next();
            expression.collectExpressionInfo(info);
        }
    }

    protected int computeHashCode() {
        return HASH_INITIAL * HASH_FACTOR + hashCode(fExpressions);
    }
}
