/*******************************************************************************
 * Copyright (c) 2000, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.core.internal.expressions.PropertyTesterDescriptor;

import dwt.dwthelper.utils;

import org.osgi.framework.Bundle;

import dwtx.core.expressions.IPropertyTester;
import dwtx.core.runtime.Assert;
import dwtx.core.runtime.CoreException;
import dwtx.core.runtime.IConfigurationElement;
import dwtx.core.runtime.IStatus;
import dwtx.core.runtime.Platform;
import dwtx.core.runtime.Status;

public class PropertyTesterDescriptor implements IPropertyTester {
    
    private IConfigurationElement fConfigElement;
    private String fNamespace;
    private String fProperties;
    
    private static final String PROPERTIES= "properties"; //$NON-NLS-1$
    private static final String NAMESPACE= "namespace"; //$NON-NLS-1$
    private static final String CLASS= "class";  //$NON-NLS-1$
    
    public PropertyTesterDescriptor(IConfigurationElement element) throws CoreException {
        fConfigElement= element;
        fNamespace= fConfigElement.getAttribute(NAMESPACE);
        if (fNamespace is null) {
            throw new CoreException(new Status(IStatus.ERROR, ExpressionPlugin.getPluginId(),
                IStatus.ERROR, 
                ExpressionMessages.PropertyTesterDescriptor_no_namespace, 
                null));
        }
        StringBuffer buffer= new StringBuffer(","); //$NON-NLS-1$
        String properties= element.getAttribute(PROPERTIES);
        if (properties is null) {
            throw new CoreException(new Status(IStatus.ERROR, ExpressionPlugin.getPluginId(),
                IStatus.ERROR, 
                ExpressionMessages.PropertyTesterDescritpri_no_properties, 
                null));
        }
        for (int i= 0; i < properties.length(); i++) {
            char ch= properties.charAt(i);
            if (!Character.isWhitespace(ch))
                buffer.append(ch);
        }
        buffer.append(',');
        fProperties= buffer.toString();
    }
    
    public PropertyTesterDescriptor(IConfigurationElement element, String namespace, String properties) {
        fConfigElement= element;
        fNamespace= namespace;
        fProperties= properties;
    }
    
    public String getProperties() {
        return fProperties;
    }
    
    public String getNamespace() {
        return fNamespace;
    }
    
    public IConfigurationElement getConfigurationElement() {
        return fConfigElement;
    }
    
    public bool handles(String namespace, String property) {
        return fNamespace.equals(namespace) && fProperties.indexOf("," + property + ",") !is -1;  //$NON-NLS-1$//$NON-NLS-2$
    }
    
    public bool isInstantiated() {
        return false;
    }
    
    public bool isDeclaringPluginActive() {
        Bundle fBundle= Platform.getBundle(fConfigElement.getContributor().getName());
        return fBundle.getState() is Bundle.ACTIVE;     
    }
    
    public IPropertyTester instantiate() throws CoreException {
        return (IPropertyTester)fConfigElement.createExecutableExtension(CLASS);
    }
    
    public bool test(Object receiver, String method, Object[] args, Object expectedValue) {
        Assert.isTrue(false, "Method should never be called"); //$NON-NLS-1$
        return false;
    }
}
