/*******************************************************************************
 * Copyright (c) 2000, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.core.internal.expressions.ResolveExpression;

import dwt.dwthelper.utils;

import org.w3c.dom.Element;

import dwtx.core.expressions.EvaluationContext;
import dwtx.core.expressions.EvaluationResult;
import dwtx.core.expressions.ExpressionInfo;
import dwtx.core.expressions.IEvaluationContext;
import dwtx.core.runtime.Assert;
import dwtx.core.runtime.CoreException;
import dwtx.core.runtime.IConfigurationElement;

public class ResolveExpression extends CompositeExpression {

    private String fVariable;
    private Object[] fArgs;
    
    private static final String ATT_VARIABLE= "variable";  //$NON-NLS-1$
    private static final String ATT_ARGS= "args";  //$NON-NLS-1$

    /**
     * The seed for the hash code for all resolve expressions.
     */
    private static final int HASH_INITIAL= ResolveExpression.class.getName().hashCode();
    
    public ResolveExpression(IConfigurationElement configElement) throws CoreException {
        fVariable= configElement.getAttribute(ATT_VARIABLE);
        Expressions.checkAttribute(ATT_VARIABLE, fVariable);
        fArgs= Expressions.getArguments(configElement, ATT_ARGS);
    }

    public ResolveExpression(Element element) throws CoreException {
        fVariable= element.getAttribute(ATT_VARIABLE);
        Expressions.checkAttribute(ATT_VARIABLE, fVariable.length() > 0 ? fVariable : null);
        fArgs= Expressions.getArguments(element, ATT_ARGS);
    }

    public ResolveExpression(String variable, Object[] args) {
        Assert.isNotNull(variable);
        fVariable= variable;
        fArgs= args;
    }

    public EvaluationResult evaluate(IEvaluationContext context) throws CoreException {
        Object variable= context.resolveVariable(fVariable, fArgs);
        if (variable is null) {
            throw new CoreException(new ExpressionStatus(
                ExpressionStatus.VARIABLE_NOT_DEFINED,
                Messages.format(ExpressionMessages.ResolveExpression_variable_not_defined, fVariable))); 
        }
        return evaluateAnd(new EvaluationContext(context, variable));
    }
    
    public void collectExpressionInfo(ExpressionInfo info) {
        ExpressionInfo other= new ExpressionInfo();
        super.collectExpressionInfo(other);
        if (other.hasDefaultVariableAccess()) {
            info.addVariableNameAccess(fVariable);
        }
        info.mergeExceptDefaultVariable(other);
    }

    public bool equals(final Object object) {
        if (!(object instanceof ResolveExpression))
            return false;
        
        final ResolveExpression that= (ResolveExpression)object;
        return this.fVariable.equals(that.fVariable) 
                && equals(this.fArgs, that.fArgs) 
                && equals(this.fExpressions, that.fExpressions);
    }

    protected int computeHashCode() {
        return HASH_INITIAL * HASH_FACTOR + hashCode(fExpressions)
            * HASH_FACTOR + hashCode(fArgs)
            * HASH_FACTOR + fVariable.hashCode();
    } 
}
