/*******************************************************************************
 * Copyright (c) 2000, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.core.internal.expressions.SystemTestExpression;

import dwt.dwthelper.utils;

import org.w3c.dom.Element;

import dwtx.core.expressions.EvaluationResult;
import dwtx.core.expressions.Expression;
import dwtx.core.expressions.ExpressionInfo;
import dwtx.core.expressions.IEvaluationContext;
import dwtx.core.runtime.Assert;
import dwtx.core.runtime.CoreException;
import dwtx.core.runtime.IConfigurationElement;

public class SystemTestExpression extends Expression {

    private String fProperty;
    private String fExpectedValue;
    
    private static final String ATT_PROPERTY= "property"; //$NON-NLS-1$
    
    /**
     * The seed for the hash code for all system test expressions.
     */
    private static final int HASH_INITIAL= SystemTestExpression.class.getName().hashCode();
    
    public SystemTestExpression(IConfigurationElement element) throws CoreException {
        fProperty= element.getAttribute(ATT_PROPERTY);
        Expressions.checkAttribute(ATT_PROPERTY, fProperty);
        fExpectedValue= element.getAttribute(ATT_VALUE);
        Expressions.checkAttribute(ATT_VALUE, fExpectedValue);
    }

    public SystemTestExpression(Element element) throws CoreException {
        fProperty= element.getAttribute(ATT_PROPERTY);
        Expressions.checkAttribute(ATT_PROPERTY, fProperty.length() > 0 ? fProperty : null);
        fExpectedValue= element.getAttribute(ATT_VALUE);
        Expressions.checkAttribute(ATT_VALUE, fExpectedValue.length() > 0 ? fExpectedValue : null);
    }

    public SystemTestExpression(String property, String expectedValue) {
        Assert.isNotNull(property);
        Assert.isNotNull(expectedValue);
        fProperty= property;
        fExpectedValue= expectedValue;
    }
    
    public EvaluationResult evaluate(IEvaluationContext context) throws CoreException {
        String str= System.getProperty(fProperty);
        if (str is null) 
            return EvaluationResult.FALSE;
        return EvaluationResult.valueOf(str.equals(fExpectedValue));
    }

    public void collectExpressionInfo(ExpressionInfo info) {
        info.markSystemPropertyAccessed();
    }

    public bool equals(final Object object) {
        if (!(object instanceof SystemTestExpression))
            return false;
        
        final SystemTestExpression that= (SystemTestExpression)object;
        return this.fProperty.equals(that.fProperty)
                && this.fExpectedValue.equals(that.fExpectedValue);
    }

    protected int computeHashCode() {
        return HASH_INITIAL * HASH_FACTOR + fExpectedValue.hashCode()
            * HASH_FACTOR + fProperty.hashCode();
    }
    
    // ---- Debugging ---------------------------------------------------
    
    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return "<systemTest property=\"" + fProperty +  //$NON-NLS-1$
          "\" value=\"" + fExpectedValue + "\""; //$NON-NLS-1$ //$NON-NLS-2$
    }
}
