/*******************************************************************************
 * Copyright (c) 2004, 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.core.internal.preferences.AbstractScope;

import dwt.dwthelper.utils;

import dwtx.core.runtime.IPath;
import dwtx.core.runtime.preferences.IEclipsePreferences;
import dwtx.core.runtime.preferences.IScopeContext;

/**
 * Abstract super-class for scope context object contributed
 * by the Platform.
 * 
 * @since 3.0
 */
public abstract class AbstractScope implements IScopeContext {

    /*
     * @see dwtx.core.runtime.preferences.IScopeContext#getName()
     */
    public abstract String getName();

    /*
     * Default path hierarchy for nodes is /<scope>/<qualifier>.
     * 
     * @see dwtx.core.runtime.preferences.IScopeContext#getNode(java.lang.String)
     */
    public IEclipsePreferences getNode(String qualifier) {
        if (qualifier is null)
            throw new IllegalArgumentException();
        return (IEclipsePreferences) PreferencesService.getDefault().getRootNode().node(getName()).node(qualifier);
    }

    /*
     * @see dwtx.core.runtime.preferences.IScopeContext#getLocation()
     */
    public abstract IPath getLocation();

    /* (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public bool equals(Object obj) {
        if (this is obj)
            return true;
        if (!(obj instanceof IScopeContext))
            return false;
        IScopeContext other = (IScopeContext) obj;
        if (!getName().equals(other.getName()))
            return false;
        IPath location = getLocation();
        return location is null ? other.getLocation() is null : location.equals(other.getLocation());
    }

    /* (non-Javadoc)
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return getName().hashCode();
    }
}
