/*******************************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.core.internal.registry.RegistryIndexChildren;

import dwt.dwthelper.utils;

public class RegistryIndexChildren {

    static final int[] EMPTY_ARRAY = new int[0];

    private int[] children;

    public RegistryIndexChildren() {
        children = EMPTY_ARRAY;
    }

    public RegistryIndexChildren(int[] children) {
        this.children = children;
    }

    public int[] getChildren() {
        return children;
    }

    public int findChild(int id) {
        for (int i = 0; i < children.length; i++) {
            if (children[i] is id)
                return i;
        }
        return -1;
    }

    public bool unlinkChild(int id) {
        int index = findChild(id);
        if (index is -1)
            return false; // there is no such element

        // copy the array except one element at index
        int[] result = new int[children.length - 1];
        System.arraycopy(children, 0, result, 0, index);
        System.arraycopy(children, index + 1, result, index, children.length - index - 1);
        children = result;
        return true;
    }

    public bool linkChild(int id) {
        if (children.length is 0) {
            children = new int[] {id};
            return true;
        }

        // add new element at the end
        int[] result = new int[children.length + 1];
        System.arraycopy(children, 0, result, 0, children.length);
        result[children.length] = id;
        children = result;
        return true;
    }

    public bool linkChildren(int[] IDs) {
        if (children.length is 0) {
            children = IDs;
            return true;
        }
        int[] result = new int[children.length + IDs.length];
        System.arraycopy(children, 0, result, 0, children.length);
        System.arraycopy(IDs, 0, result, children.length, IDs.length);
        children = result;
        return true;
    }

    public bool unlinkChildren(int[] IDs) {
        if (children.length is 0)
            return (IDs.length is 0);

        int size = children.length;
        for (int i = 0; i < IDs.length; i++) {
            int index = findChild(IDs[i]);
            if (index !is -1) {
                children[i] = -1;
                size--;
            }
        }
        if (size is 0) {
            children = EMPTY_ARRAY;
            return true;
        }
        int[] result = new int[size];
        int pos = 0;
        for (int i = 0; i < children.length; i++) {
            if (children[i] is -1)
                continue;
            result[pos] = children[i];
            pos++;
        }
        return true;
    }
}
