/*******************************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.core.internal.registry.RegistryIndexElement;

import dwt.dwthelper.utils;

public class RegistryIndexElement implements KeyedElement {

    // The key on which indexing is done
    final protected String key;

    // Extension points matching the key
    private RegistryIndexChildren extensionPoints;

    // Extensions matching the key
    private RegistryIndexChildren extensions;

    public RegistryIndexElement(String key) {
        this.key = key;
    }
    
    public RegistryIndexElement(String key, int[] extensionPoints, int[] extensions) {
        this.key = key;
        this.extensionPoints = new RegistryIndexChildren(extensionPoints);
        this.extensions = new RegistryIndexChildren(extensions);
    }

    protected int[] getExtensions() {
        if (extensions is null)
            return RegistryIndexChildren.EMPTY_ARRAY;
        return extensions.getChildren();
    }

    protected int[] getExtensionPoints() {
        if (extensionPoints is null)
            return RegistryIndexChildren.EMPTY_ARRAY;
        return extensionPoints.getChildren();
    }

    public bool updateExtension(int id, bool add) {
        if (extensions is null)
            extensions = new RegistryIndexChildren();

        if (add)
            return extensions.linkChild(id);
        else
            return extensions.unlinkChild(id);
    }

    public bool updateExtensions(int[] IDs, bool add) {
        if (extensions is null)
            extensions = new RegistryIndexChildren();

        if (add)
            return extensions.linkChildren(IDs);
        else
            return extensions.unlinkChildren(IDs);
    }

    public bool updateExtensionPoint(int id, bool add) {
        if (extensionPoints is null)
            extensionPoints = new RegistryIndexChildren();

        if (add)
            return extensionPoints.linkChild(id);
        else
            return extensionPoints.unlinkChild(id);
    }

    public bool updateExtensionPoints(int[] IDs, bool add) {
        if (extensionPoints is null)
            extensionPoints = new RegistryIndexChildren();

        if (add)
            return extensionPoints.linkChildren(IDs);
        else
            return extensionPoints.unlinkChildren(IDs);
    }

    //Implements the KeyedElement interface
    public int getKeyHashCode() {
        return getKey().hashCode();
    }

    public Object getKey() {
        return key;
    }

    public bool compare(KeyedElement other) {
        return key.equals(((RegistryIndexElement) other).key);
    }
}
