/*******************************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.core.internal.registry.osgi.RegistryProviderOSGI;

import dwt.dwthelper.utils;

import org.osgi.framework.BundleContext;
import org.osgi.util.tracker.ServiceTracker;

import dwtx.core.internal.registry.RegistryMessages;
import dwtx.core.internal.runtime.RuntimeLog;
import dwtx.core.runtime.IExtensionRegistry;
import dwtx.core.runtime.IStatus;
import dwtx.core.runtime.Status;
import dwtx.core.runtime.spi.IRegistryProvider;

public final class RegistryProviderOSGI implements IRegistryProvider {

    private ServiceTracker registryTracker = null;

    /* (non-Javadoc)
     * @see dwtx.equinox.registry.IRegistryProvider#getRegistry()
     */
    public IExtensionRegistry getRegistry() {
        if (registryTracker is null) {
            BundleContext context = Activator.getContext();
            if (context is null) {
                RuntimeLog.log(new Status(IStatus.ERROR, RegistryMessages.OWNER_NAME, 0, RegistryMessages.bundle_not_activated, null));
                return null;
            }
            registryTracker = new ServiceTracker(context, IExtensionRegistry.class.getName(), null);
            registryTracker.open();
        }
        return (IExtensionRegistry) registryTracker.getService();
    }

    /**
     * Release OSGi tracker
     */
    public void release() {
        if (registryTracker !is null) {
            registryTracker.close();
            registryTracker = null;
        }
    }
}
