/*******************************************************************************
 * Copyright (c) 2003, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     Julian Chen - fix for bug #92572, jclRM
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.core.internal.runtime.PlatformActivator;

import dwt.dwthelper.utils;

import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceRegistration;

import dwtx.core.runtime.Plugin;
import dwtx.equinox.internal.app.CommandLineArgs;

/**
 * Activator for the Eclipse runtime.
 */
public class PlatformActivator extends Plugin implements BundleActivator {
    private static BundleContext context;
    private ServiceRegistration entryLocatorRegistration;

    public static BundleContext getContext() {
        return context;
    }

    public void start(BundleContext runtimeContext) throws Exception {
        PlatformActivator.context = runtimeContext;
        InternalPlatform.getDefault().start(runtimeContext);
        startAppContainer();
        InternalPlatform.getDefault().setRuntimeInstance(this);
        super.start(runtimeContext);
    }

    public void stop(BundleContext runtimeContext) {
        // unregister the EntryLocator to prevent the Framework from calling it
        unregisterEntryLocator();
        // Stop the platform orderly.       
        InternalPlatform.getDefault().stop(runtimeContext);
        InternalPlatform.getDefault().setRuntimeInstance(null);
    }

    private void startAppContainer() {
        // just using a class out of app admin to force it to lazy-start
        CommandLineArgs.getApplicationArgs();
    }

    private void unregisterEntryLocator() {
        if (entryLocatorRegistration !is null) {
            entryLocatorRegistration.unregister();
            entryLocatorRegistration = null;
        }
    }
}
