/*******************************************************************************
 * Copyright (c) 2004, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.core.internal.runtime.PlatformURLConfigConnection;

import dwt.dwthelper.utils;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.net.URL;
import java.net.UnknownServiceException;

import dwtx.core.internal.boot.PlatformURLConnection;
import dwtx.core.internal.boot.PlatformURLHandler;
import dwtx.osgi.service.datalocation.Location;
import dwtx.osgi.util.NLS;

public class PlatformURLConfigConnection extends PlatformURLConnection {
    private static final String FILE_PROTOCOL = "file"; //$NON-NLS-1$
    private static bool isRegistered = false;
    public static final String CONFIG = "config"; //$NON-NLS-1$

    private bool parentConfiguration = false;

    /*
     * Constructor for the class.
     */
    public PlatformURLConfigConnection(URL url) {
        super(url);
    }

    /* (non-Javadoc)
     * @see dwtx.equinox.internal.url.PlatformURLConnection#resolve()
     */
    protected URL resolve() throws IOException {
        String spec = url.getFile().trim();
        if (spec.startsWith("/")) //$NON-NLS-1$
            spec = spec.substring(1);
        if (!spec.startsWith(CONFIG))
            throw new IOException(NLS.bind(CommonMessages.url_badVariant, url.toString()));
        String path = spec.substring(CONFIG.length() + 1);
        // resolution takes parent configuration into account (if it exists)
        Activator activator = Activator.getDefault();
        if (activator is null)
            throw new IOException(CommonMessages.activator_not_available);
        Location localConfig = activator.getConfigurationLocation();
        Location parentConfig = localConfig.getParentLocation();
        // assume we will find the file locally
        URL localURL = new URL(localConfig.getURL(), path);
        if (!FILE_PROTOCOL.equals(localURL.getProtocol()) || parentConfig is null)
            // we only support cascaded file: URLs
            return localURL;
        File localFile = new File(localURL.getPath());
        if (localFile.exists())
            // file exists in local configuration
            return localURL;
        // try to find in the parent configuration
        URL parentURL = new URL(parentConfig.getURL(), path);
        if (FILE_PROTOCOL.equals(parentURL.getProtocol())) {
            // we only support cascaded file: URLs          
            File parentFile = new File(parentURL.getPath());
            if (parentFile.exists()) {
                // parent has the location
                parentConfiguration = true;
                return parentURL;
            }
        }
        return localURL;
    }

    public static void startup() {
        // register connection type for platform:/config handling
        if (isRegistered)
            return;
        PlatformURLHandler.register(CONFIG, PlatformURLConfigConnection.class);
        isRegistered = true;
    }

    /* (non-Javadoc)
     * @see java.net.URLConnection#getOutputStream()
     */
    public OutputStream getOutputStream() throws IOException {
        if (parentConfiguration || Activator.getDefault() is null || Activator.getDefault().getConfigurationLocation().isReadOnly())
            throw new UnknownServiceException(NLS.bind(CommonMessages.url_noOutput, url));
        //This is not optimal but connection is a private instance variable in the super-class.
        URL resolved = getResolvedURL();
        if (resolved !is null) {
            String fileString = resolved.getFile();
            if (fileString !is null) {
                File file = new File(fileString);
                String parent = file.getParent();
                if (parent !is null)
                    new File(parent).mkdirs();
                return new FileOutputStream(file);
            }
        }
        return null;
    }
}
