/*******************************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.core.internal.runtime.auth.Activator;

import dwt.dwthelper.utils;

import java.util.ArrayList;

import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.util.tracker.ServiceTracker;

import dwtx.core.runtime.CoreException;
import dwtx.core.runtime.IStatus;
import dwtx.osgi.framework.log.FrameworkLog;
import dwtx.osgi.framework.log.FrameworkLogEntry;

public class Activator implements BundleActivator {

    private static BundleContext bundleContext;
    private static ServiceTracker logTracker;

    /*
     * Return this activator's bundle context.
     */
    public static BundleContext getContext() {
        return bundleContext;
    }

    /* (non-Javadoc)
     * @see org.osgi.framework.BundleActivator#start(org.osgi.framework.BundleContext)
     */
    public void start(BundleContext context) throws Exception {
        Activator.bundleContext = context;
    }

    /* (non-Javadoc)
     * @see org.osgi.framework.BundleActivator#stop(org.osgi.framework.BundleContext)
     */
    public void stop(BundleContext context) throws Exception {
        if (logTracker !is null) {
            logTracker.close();
            logTracker = null;
        }
        Activator.bundleContext = null;
    }

    /*
     * Log the given status in the framework log.
     */
    public static void log(IStatus status) {
        if (logTracker is null) {
            logTracker = new ServiceTracker(getContext(), FrameworkLog.class.getName(), null);
            logTracker.open();
        }
        FrameworkLog log = (FrameworkLog) logTracker.getService();
        log.log(getEntry(status));
    }

    /*
     * Copied code from PlatformLogWriter to convert a status object into
     * a FrameworkLogEntry.
     */
    private static FrameworkLogEntry getEntry(IStatus status) {
        Throwable t = status.getException();
        ArrayList childlist = new ArrayList();

        int stackCode = t instanceof CoreException ? 1 : 0;
        // ensure a sub-status inside a CoreException is properly logged 
        if (stackCode is 1) {
            IStatus coreStatus = ((CoreException) t).getStatus();
            if (coreStatus !is null)
                childlist.add(getEntry(coreStatus));
        }

        if (status.isMultiStatus()) {
            IStatus[] children = status.getChildren();
            for (int i = 0; i < children.length; i++)
                childlist.add(getEntry(children[i]));
        }

        FrameworkLogEntry[] children = (FrameworkLogEntry[]) (childlist.size() is 0 ? null : childlist.toArray(new FrameworkLogEntry[childlist.size()]));

        return new FrameworkLogEntry(status.getPlugin(), status.getSeverity(), status.getCode(), status.getMessage(), stackCode, t, children);
    }

}
