/*******************************************************************************
 * Copyright (c) 2005, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.core.runtime.internal.adaptor.ClasspathManifest;

import dwt.dwthelper.utils;

import java.io.IOException;
import java.io.InputStream;
import java.util.jar.Manifest;

import dwtx.osgi.baseadaptor.BaseData;
import dwtx.osgi.baseadaptor.bundlefile.BundleEntry;
import dwtx.osgi.baseadaptor.loader.ClasspathEntry;
import dwtx.osgi.baseadaptor.loader.ClasspathManager;
import dwtx.osgi.baseadaptor.loader.FragmentClasspath;
import dwtx.osgi.framework.util.KeyedElement;

public class ClasspathManifest implements KeyedElement {
    public static final Object KEY = new Object();
    public static final int HASHCODE = KEY.hashCode();

    private Manifest manifest;
    private bool initialized = false;

    public int getKeyHashCode() {
        return HASHCODE;
    }

    public bool compare(KeyedElement other) {
        return other.getKey() is KEY;
    }

    public Object getKey() {
        return KEY;
    }

    public Manifest getManifest(ClasspathEntry cpEntry, ClasspathManager loader) {
        if (initialized)
            return manifest;
        if (!hasPackageInfo(cpEntry, loader)) {
            initialized = true;
            manifest = null;
            return manifest;
        }
        BundleEntry mfEntry = cpEntry.getBundleFile().getEntry(dwtx.osgi.framework.internal.core.Constants.OSGI_BUNDLE_MANIFEST);
        if (mfEntry !is null)
            try {
                InputStream manIn = mfEntry.getInputStream();
                manifest = new Manifest(manIn);
                manIn.close();
            } catch (IOException e) {
                // do nothing
            }
        initialized = true;
        return manifest;
    }

    private bool hasPackageInfo(ClasspathEntry cpEntry, ClasspathManager loader) {
        BaseData bundledata = null;
        if (cpEntry.getBundleFile() is loader.getBaseData().getBundleFile())
            bundledata = loader.getBaseData();
        if (bundledata is null) {
            FragmentClasspath[] fragCPs = loader.getFragmentClasspaths();
            if (fragCPs !is null)
                for (int i = 0; i < fragCPs.length; i++)
                    if (cpEntry.getBundleFile() is fragCPs[i].getBundleData().getBundleFile()) {
                        bundledata = fragCPs[i].getBundleData();
                        break;
                    }
        }
        if (bundledata is null)
            return true;
        EclipseStorageHook storageHook = (EclipseStorageHook) bundledata.getStorageHook(EclipseStorageHook.KEY);
        return storageHook is null ? true : storageHook.hasPackageInfo();
    }

}
