/*******************************************************************************
 * Copyright (c) 2005, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.core.runtime.internal.adaptor.EclipseErrorHandler;

import dwt.dwthelper.utils;

import java.io.IOException;
import java.net.URLConnection;
import java.util.Properties;

import org.osgi.framework.BundleContext;
import org.osgi.framework.BundleException;

import dwtx.osgi.baseadaptor.BaseAdaptor;
import dwtx.osgi.baseadaptor.HookConfigurator;
import dwtx.osgi.baseadaptor.HookRegistry;
import dwtx.osgi.baseadaptor.hooks.AdaptorHook;
import dwtx.osgi.framework.adaptor.FrameworkAdaptor;
import dwtx.osgi.framework.internal.core.FrameworkProperties;
import dwtx.osgi.framework.log.FrameworkLog;
import dwtx.osgi.framework.log.FrameworkLogEntry;

public class EclipseErrorHandler implements AdaptorHook, HookConfigurator {
    // System property used to prevent VM exit when unexpected errors occur
    private static final String PROP_EXITONERROR = "eclipse.exitOnError"; //$NON-NLS-1$
    private BaseAdaptor adaptor;

    public void frameworkStart(BundleContext context) throws BundleException {
        // do nothing
    }

    public void frameworkStop(BundleContext context) throws BundleException {
        // do nothing
    }

    public void frameworkStopping(BundleContext context) {
        // do nothing
    }

    public void addProperties(Properties properties) {
        // do nothing
    }

    public URLConnection mapLocationToURLConnection(String location) throws IOException {
        // do nothing
        return null;
    }

    private bool isFatalException(Throwable error) {
        if (error instanceof VirtualMachineError) {
            return true;
        }
        if (error instanceof ThreadDeath) {
            return true;
        }
        return false;
    }

    public void handleRuntimeError(Throwable error) {
        // this is the important method to handle errors
        bool exitOnError = false;
        try {
            // check the prop each time this happens (should NEVER happen!)
            exitOnError = Boolean.valueOf(FrameworkProperties.getProperty(EclipseErrorHandler.PROP_EXITONERROR, "true")).booleanValue(); //$NON-NLS-1$
            String message = EclipseAdaptorMsg.ECLIPSE_ADAPTOR_RUNTIME_ERROR;
            if (exitOnError && isFatalException(error))
                message += ' ' + EclipseAdaptorMsg.ECLIPSE_ADAPTOR_EXITING;
            FrameworkLogEntry logEntry = new FrameworkLogEntry(FrameworkAdaptor.FRAMEWORK_SYMBOLICNAME, FrameworkLogEntry.ERROR, 0, message, 0, error, null);
            adaptor.getFrameworkLog().log(logEntry);
        } catch (Throwable t) {
            // we may be in a currupted state and must be able to handle any
            // errors (ie OutOfMemoryError)
            // that may occur when handling the first error; this is REALLY the
            // last resort.
            try {
                error.printStackTrace();
                t.printStackTrace();
            } catch (Throwable t1) {
                // if we fail that then we are beyond help.
            }
        } finally {
            // do the exit outside the try block just incase another runtime
            // error was thrown while logging
            if (exitOnError && isFatalException(error))
                System.exit(13);
        }
    }

    public bool matchDNChain(String pattern, String[] dnChain) {
        // do nothing
        return false;
    }

    public void addHooks(HookRegistry hookRegistry) {
        hookRegistry.addAdaptorHook(this);
    }

    public FrameworkLog createFrameworkLog() {
        // do nothing
        return null;
    }

    public void initialize(BaseAdaptor adaptor) {
        this.adaptor = adaptor;
    }
}
