/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.help.internal.UAElementFactory;

import dwt.dwthelper.utils;

import java.lang.reflect.Constructor;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.w3c.dom.Element;

import dwtx.help.IAnchor;
import dwtx.help.ICommandLink;
import dwtx.help.IContentExtension;
import dwtx.help.IContext;
import dwtx.help.IInclude;
import dwtx.help.IIndex;
import dwtx.help.IIndexEntry;
import dwtx.help.ILink;
import dwtx.help.IToc;
import dwtx.help.ITopic;
import dwtx.help.IUAElement;
import dwtx.help.internal.context.Context;
import dwtx.help.internal.extension.ContentExtension;
import dwtx.help.internal.index.Index;
import dwtx.help.internal.index.IndexEntry;
import dwtx.help.internal.toc.Link;
import dwtx.help.internal.toc.Toc;

/*
 * Constructs typed model elements from DOM elements or interface
 * implementations.
 */
public class UAElementFactory {

    private static final Class[][] interfaceTable = new Class[][] {
        { ITopic.class, Topic.class },
        { IIndexEntry.class, IndexEntry.class },
        { IContext.class, Context.class },
        { IAnchor.class, Anchor.class },
        { IInclude.class, Include.class },
        { ILink.class, Link.class },
        { IToc.class, Toc.class },
        { ICommandLink.class, CommandLink.class },
        { IIndex.class, Index.class },
        { IContentExtension.class, ContentExtension.class },
    };

    private static final Map classByElementName;

    static {
        classByElementName = Collections.synchronizedMap(new HashMap());
        classByElementName.put(Anchor.NAME, Anchor.class);
        classByElementName.put(Include.NAME, Include.class);
        classByElementName.put(Toc.NAME, Toc.class);
        classByElementName.put(Topic.NAME, Topic.class);
        classByElementName.put(Index.NAME, Index.class);
        classByElementName.put(IndexEntry.NAME, IndexEntry.class);
        classByElementName.put(Context.NAME, Context.class);
        classByElementName.put(CommandLink.NAME, CommandLink.class);
        classByElementName.put(Link.NAME, Link.class);
        classByElementName.put(ContentExtension.NAME_CONTRIBUTION, ContentExtension.class);
        classByElementName.put(ContentExtension.NAME_CONTRIBUTION_LEGACY, ContentExtension.class);
        classByElementName.put(ContentExtension.NAME_REPLACEMENT, ContentExtension.class);
        classByElementName.put(ContentExtension.NAME_REPLACEMENT_LEGACY, ContentExtension.class);
    }
    
    public static UAElement newElement(Element element) {
        String name = element.getNodeName();
        Class clazz = (Class)classByElementName.get(name);
        if (clazz !is null) {
            try {
                Constructor constructor = clazz.getConstructor(new Class[] { Element.class });
                return (UAElement)constructor.newInstance(new Object[] { element });
            }
            catch (Exception e) {
                String msg = "Error creating document model element"; //$NON-NLS-1$
                HelpPlugin.logError(msg, e);
            }
        }
        return new UAElement(element);
    }

    public static UAElement newElement(IUAElement src) {
        for (int i=0;i<interfaceTable.length;++i) {
            Class interfaze = interfaceTable[i][0];
            Class clazz = interfaceTable[i][1];
            if (interfaze.isAssignableFrom(src.getClass())) {
                try {
                    Constructor constructor = clazz.getConstructor(new Class[] { interfaze });
                    return (UAElement)constructor.newInstance(new Object[] { src });
                }
                catch (Exception e) {
                    String msg = "Error creating document model element"; //$NON-NLS-1$
                    HelpPlugin.logError(msg, e);
                }
            }
        }
        return null;
    }
}
