/*******************************************************************************
 * Copyright (c) 2000, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     Phil Loats (IBM Corp.) - fix to use only foundation APIs
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.help.internal.context.Context;

import dwt.dwthelper.utils;

import java.util.ArrayList;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import dwtx.help.ICommandLink;
import dwtx.help.IContext;
import dwtx.help.IContext2;
import dwtx.help.IContext3;
import dwtx.help.IHelpResource;
import dwtx.help.ITopic;
import dwtx.help.internal.CommandLink;
import dwtx.help.internal.Topic;
import dwtx.help.internal.UAElement;

public class Context extends UAElement implements IContext3 {

    public static final String ATTRIBUTE_TITLE = "title"; //$NON-NLS-1$
    public static final String NAME = "context"; //$NON-NLS-1$
    public static final String ELEMENT_DESCRIPTION = "description"; //$NON-NLS-1$
    public static final String ATTRIBUTE_ID = "id"; //$NON-NLS-1$
    public static final String ATTRIBUTE_PLUGIN_ID = "pluginId"; //$NON-NLS-1$
    
    public Context(Element src) {
        super(src);
    }
    
    public Context(IContext src, String id) {
        super(NAME);
        setId(id);
        children = new ArrayList();
        mergeContext(src);
    }
    
    public void mergeContext(IContext src) {
        String text = src.getText();
        if (getText() is null || getText().length() is 0) {
            setText(text);
        }
        if (src instanceof IContext2 && getTitle() is null) {
            String title  = ((IContext2)src).getTitle();
            if (title !is null) {
                setAttribute(ATTRIBUTE_TITLE, title);
            }
        }
        if (src instanceof IContext3) {
            ICommandLink[] commands = ((IContext3)src).getRelatedCommands();
            for (int i=0;i<commands.length;++i) {
                appendChild(new CommandLink(commands[i]));
            }
        }
        IHelpResource[] topics = src.getRelatedTopics();
        for (int i=0;i<topics.length;++i) {
            if (topics[i] instanceof ITopic) {
                appendChild(new Topic((ITopic)topics[i]));
            }
            else {
                Topic topic = new Topic();
                topic.setHref(topics[i].getHref());
                topic.setLabel(topics[i].getLabel());
                appendChild(topic);
            }
        }
    }

    public String getCategory(IHelpResource topic) {
        return null;
    }
    
    public String getId() {
        return getAttribute(ATTRIBUTE_ID);
    }
    
    public ICommandLink[] getRelatedCommands() {
        return (ICommandLink[])getChildren(ICommandLink.class);
    }
    
    public IHelpResource[] getRelatedTopics() {
        return (IHelpResource[])getChildren(IHelpResource.class);
    }

    public String getStyledText() {
        return null;
    }
    
    public String getText() {
        Node node = getElement().getFirstChild();
        while (node !is null) {
            if (node.getNodeType() is Node.ELEMENT_NODE) {
                if (ELEMENT_DESCRIPTION.equals(node.getNodeName())) {
                    node.normalize();
                    Node text = node.getFirstChild();
                    if (text is null) {
                        return new String();
                    }
                    if (text.getNodeType() is Node.TEXT_NODE) {
                        return text.getNodeValue();
                    }
                }
            }
            node = node.getNextSibling();
        }
        return null; 
    }
    
    public String getTitle() {
        String title = getAttribute(ATTRIBUTE_TITLE);
        if (title is null || title.length() is 0) {
            return null;
        }
        return title;
    }

    public void setId(String id) {
        setAttribute(ATTRIBUTE_ID, id);
    }

    public void setText(String text) {
        Node node = getElement().getFirstChild();
        while (node !is null) {
            if (node.getNodeType() is Node.ELEMENT_NODE) {
                if (ELEMENT_DESCRIPTION.equals(node.getNodeName())) {
                    getElement().removeChild(node);
                    break;
                }
            }
            node = node.getNextSibling();
        }
        if (text !is null) {
            Document document = getElement().getOwnerDocument();
            Node description = getElement().appendChild(document.createElement(ELEMENT_DESCRIPTION));
            description.appendChild(document.createTextNode(text));
        }
    }
    
}
