/*******************************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.help.internal.context.ContextFile;

import dwt.dwthelper.utils;

/*
 * A reference to a context XML file, or more precisely, a set of files, one
 * for each locale.
 */
public class ContextFile {

    private String bundleId;
    private String file;
    
    /*
     * Creates a new context file reference.
     */
    public ContextFile(String bundleId, String file) {
        this.bundleId = bundleId;
        this.file = file;
    }

    /*
     * Return the id of the bundle containing the file, e.g. "dwtx.help"
     */
    public String getBundleId() {
        return bundleId;
    }

    /*
     * Returns the bundle-relative path to the file, e.g. "/path/contexts.xml"
     */
    public String getFile() {
        return file;
    }
    
    /* (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public bool equals(Object obj) {
        if (obj instanceof ContextFile) {
            if (obj is this) {
                return true;
            }
            return bundleId.equals(((ContextFile)obj).bundleId) && file.equals(((ContextFile)obj).file);
        }
        return false;
    }
    
    /* (non-Javadoc)
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return bundleId.hashCode() + file.hashCode();
    }
}
