/*******************************************************************************
 * Copyright (c) 2006, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.help.internal.extension.ContentExtensionFileParser;

import dwt.dwthelper.utils;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import javax.xml.parsers.ParserConfigurationException;

import org.osgi.framework.Bundle;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import dwtx.help.IUAElement;
import dwtx.help.internal.UAElement;
import dwtx.help.internal.dynamic.DocumentProcessor;
import dwtx.help.internal.dynamic.DocumentReader;
import dwtx.help.internal.dynamic.ProcessorHandler;
import dwtx.help.internal.dynamic.ValidationHandler;

/*
 * Parses content extension XML files into extension model elements.
 */
public class ContentExtensionFileParser extends DefaultHandler {

    private DocumentReader reader;
    private DocumentProcessor processor;
    private Map requiredAttributes;
    private Map deprecatedElements;
    
    /*
     * Parses the specified content extension XML file into model elements.
     */
    public ContentExtension[] parse(Bundle bundle, String path) throws IOException, SAXException, ParserConfigurationException {
        if (reader is null) {
            reader = new DocumentReader();
        }
        URL url = bundle.getEntry(path);
        if (url !is null) {
            InputStream in = url.openStream();
            UAElement extension = (UAElement)reader.read(in);
            if (processor is null) {
                processor = new DocumentProcessor(new ProcessorHandler[] {
                    new ValidationHandler(getRequiredAttributes(), getDeprecatedElements())
                });
            }
            processor.process(extension, '/' + bundle.getSymbolicName() + '/' + path);
            IUAElement[] children = extension.getChildren();
            ContentExtension[] result = new ContentExtension[children.length];
            System.arraycopy(children, 0, result, 0, children.length);
            return result;
        }
        else {
            throw new FileNotFoundException();
        }
    }

    private Map getRequiredAttributes() {
        if (requiredAttributes is null) {
            requiredAttributes = new HashMap();
            requiredAttributes.put(ContentExtension.NAME_CONTRIBUTION, new String[] { ContentExtension.ATTRIBUTE_CONTENT, ContentExtension.ATTRIBUTE_PATH });
            requiredAttributes.put(ContentExtension.NAME_CONTRIBUTION_LEGACY, new String[] { ContentExtension.ATTRIBUTE_CONTENT, ContentExtension.ATTRIBUTE_PATH });
            requiredAttributes.put(ContentExtension.NAME_REPLACEMENT, new String[] { ContentExtension.ATTRIBUTE_CONTENT, ContentExtension.ATTRIBUTE_PATH });
            requiredAttributes.put(ContentExtension.NAME_REPLACEMENT_LEGACY, new String[] { ContentExtension.ATTRIBUTE_CONTENT, ContentExtension.ATTRIBUTE_PATH });
        }
        return requiredAttributes;
    }

    private Map getDeprecatedElements() {
        if (deprecatedElements is null) {
            deprecatedElements = new HashMap();
            deprecatedElements.put(ContentExtension.NAME_CONTRIBUTION_LEGACY, ContentExtension.NAME_CONTRIBUTION);
            deprecatedElements.put(ContentExtension.NAME_REPLACEMENT_LEGACY, ContentExtension.NAME_REPLACEMENT);
        }
        return deprecatedElements;
    }
}
