/*******************************************************************************
 * Copyright (c) 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.help.internal.index.IndexDocumentReader;

import dwt.dwthelper.utils;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import dwtx.help.internal.dynamic.DocumentReader;

public class IndexDocumentReader extends DocumentReader {
    
    protected void prepareDocument(Document document) {
        prune(document.getDocumentElement());
    }

    private void prune(Node element) {
        NodeList nodes = element.getChildNodes();
        Node node = nodes.item(0); 
        while (node !is null) {
            short nodeType = node.getNodeType();
            if (nodeType is Node.TEXT_NODE || nodeType is Node.COMMENT_NODE) {
                Node nodeToDelete = node;
                node = node.getNextSibling();
                element.removeChild(nodeToDelete);
            } else  if (nodeType is Node.ELEMENT_NODE) {
                String kind = node.getNodeName();
                if ("topic".equalsIgnoreCase(kind)) { //$NON-NLS-1$
                    fixTopicAttributes((Element)node);
                }
                prune(node);
                node = node.getNextSibling();
            } else {
                node = node.getNextSibling();
            }
        }
    }

    private void fixTopicAttributes(Element topic) {
        String title = topic.getAttribute("name"); //$NON-NLS-1$
        if (title !is null && title.length() > 0) {
            topic.setAttribute("label", title);          //$NON-NLS-1$
            topic.removeAttribute("name"); //$NON-NLS-1$
        }
    }

}
