/*******************************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.help.internal.toc.DocumentFinder;

import dwt.dwthelper.utils;

import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import org.osgi.framework.Bundle;

import dwtx.core.runtime.FileLocator;
import dwtx.core.runtime.IPath;
import dwtx.core.runtime.Path;
import dwtx.core.runtime.Platform;
import dwtx.help.internal.HelpPlugin;
import dwtx.help.internal.util.ResourceLocator;

public class DocumentFinder {
    
    public static String[] collectExtraDocuments(TocFile tocFile) {
        String dir = HrefUtil.normalizeDirectoryHref(tocFile.getPluginId(), tocFile.getExtraDir());
        String locale = tocFile.getLocale();
        
        List result = new ArrayList();
        String pluginID = HrefUtil.getPluginIDFromHref(dir);
        if (pluginID is null) {
            return new String[0];
        }
        Bundle pluginDesc = Platform.getBundle(pluginID);
        if (pluginDesc is null || pluginDesc.getState() is Bundle.INSTALLED
                || pluginDesc.getState() is Bundle.UNINSTALLED)
            return new String[0];
        String directory = HrefUtil.getResourcePathFromHref(dir);
        if (directory is null) {
            // the root - all files in a zip should be indexed
            directory = ""; //$NON-NLS-1$
        }
        // Find doc.zip file
        IPath iPath = new Path("$nl$/doc.zip"); //$NON-NLS-1$
        Map override = new HashMap(1);
        override.put("$nl$", locale); //$NON-NLS-1$
        URL url = FileLocator.find(pluginDesc, iPath, override);
        if (url is null) {
            url = FileLocator.find(pluginDesc, new Path("doc.zip"), null); //$NON-NLS-1$
        }
        if (url !is null) {
            // collect topics from doc.zip file
            result.addAll(collectExtraDocumentsFromZip(pluginID, directory, url));
        }
        
        // Find topics in plugin
        Set paths = ResourceLocator.findTopicPaths(pluginDesc, directory,
                locale);
        for (Iterator it = paths.iterator(); it.hasNext();) {
            String href = "/" + pluginID + "/" + (String) it.next();  //$NON-NLS-1$//$NON-NLS-2$
            href = HrefUtil.normalizeDirectoryPath(href);
            result.add(href);
        }
        return (String[])result.toArray(new String[result.size()]);
    }
    
    private static List collectExtraDocumentsFromZip(String pluginID, String directory,
            URL url) {
        List result = new ArrayList();
        URL realZipURL;
        try {
            realZipURL = FileLocator.toFileURL(FileLocator.resolve(url));
            if (realZipURL.toExternalForm().startsWith("jar:")) { //$NON-NLS-1$
                // doc.zip not allowed in jarred plug-ins.
                return result;
            }
        } catch (IOException ioe) {
            HelpPlugin.logError("IOException occurred, when resolving URL " //$NON-NLS-1$
                    + url.toString() + ".", ioe); //$NON-NLS-1$
            return result;
        }
        ZipFile zipFile;
        try {
            zipFile = new ZipFile(realZipURL.getFile());
            result = createExtraTopicsFromZipFile(pluginID, zipFile, directory);
            zipFile.close();
        } catch (IOException ioe) {
            HelpPlugin.logError(
                    "IOException occurred, when accessing Zip file " //$NON-NLS-1$
                            + realZipURL.getFile()
                            + ".  File might not be locally available.", ioe); //$NON-NLS-1$
            return new ArrayList();
        }
        return result;
    }
    
    private static List createExtraTopicsFromZipFile(String pluginID, ZipFile zipFile,
            String directory) {
        String constantHrefSegment = "/" + pluginID + "/"; //$NON-NLS-1$ //$NON-NLS-2$
        List result = new ArrayList();
        for (Enumeration entriesEnum = zipFile.entries(); entriesEnum.hasMoreElements();) {
            ZipEntry zEntry = (ZipEntry) entriesEnum.nextElement();
            if (zEntry.isDirectory()) {
                continue;
            }
            String docName = zEntry.getName();
            int l = directory.length();
            if (l is 0 || docName.length() > l && docName.charAt(l) is '/'
                    && directory.equals(docName.substring(0, l))) {
                String href = constantHrefSegment + docName;
                result.add(href);
            }
        }
        return result;
    }
}
