/*******************************************************************************
 * Copyright (c) 2006, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.help.internal.toc.TocFileProvider;

import dwt.dwthelper.utils;

import java.util.ArrayList;
import java.util.List;

import dwtx.core.runtime.IConfigurationElement;
import dwtx.core.runtime.IExtensionRegistry;
import dwtx.core.runtime.InvalidRegistryObjectException;
import dwtx.core.runtime.Platform;
import dwtx.help.AbstractTocProvider;
import dwtx.help.ITocContribution;
import dwtx.help.internal.HelpPlugin;

/*
 * Provides toc data from toc XML files to the help system.
 */
public class TocFileProvider extends AbstractTocProvider {

    public static final String EXTENSION_POINT_ID_TOC = HelpPlugin.PLUGIN_ID + ".toc"; //$NON-NLS-1$
    public static final String ELEMENT_NAME_TOC = "toc"; //$NON-NLS-1$
    public static final String ATTRIBUTE_NAME_FILE = "file"; //$NON-NLS-1$
    public static final String ATTRIBUTE_NAME_PRIMARY = "primary"; //$NON-NLS-1$
    public static final String ATTRIBUTE_NAME_EXTRADIR = "extradir"; //$NON-NLS-1$
    public static final String ATTRIBUTE_NAME_CATEGORY = "category"; //$NON-NLS-1$

    /* (non-Javadoc)
     * @see dwtx.help.AbstractTocProvider#getTocContributions(java.lang.String)
     */
    public ITocContribution[] getTocContributions(String locale) {
        List contributions = new ArrayList();
        TocFile[] tocFiles = getTocFiles(locale);
        TocFileParser parser = new TocFileParser();
        for (int i=0;i<tocFiles.length;++i) {
            try {
                ITocContribution toc = parser.parse(tocFiles[i]);
                contributions.add(toc);
            }
            catch (Throwable t) {
                String msg = "Error reading help table of contents file /\"" + tocFiles[i].getPluginId() + '/' + tocFiles[i].getFile() + "\" (skipping file)"; //$NON-NLS-1$ //$NON-NLS-2$
                HelpPlugin.logError(msg, t);
            }
        }
        return (ITocContribution[])contributions.toArray(new ITocContribution[contributions.size()]);
    }

    /*
     * Returns all available TocFiles for the given locale.
     */
    protected TocFile[] getTocFiles(String locale) {
        List tocFiles = new ArrayList();
        IExtensionRegistry registry = Platform.getExtensionRegistry();
        IConfigurationElement[] elements = registry.getConfigurationElementsFor(EXTENSION_POINT_ID_TOC);
        for (int i=0;i<elements.length;++i) {
            IConfigurationElement elem = elements[i];
            String pluginId;
            try {
                pluginId = elem.getNamespaceIdentifier();
            }
            catch (InvalidRegistryObjectException e) {
                // no longer valid; skip it
                continue;
            }

            if (elem.getName().equals(ELEMENT_NAME_TOC)) {
                String file = elem.getAttribute(ATTRIBUTE_NAME_FILE);
                bool primary = "true".equalsIgnoreCase(elem.getAttribute(ATTRIBUTE_NAME_PRIMARY)); //$NON-NLS-1$
                String extradir = elem.getAttribute(ATTRIBUTE_NAME_EXTRADIR);
                String category = elem.getAttribute(ATTRIBUTE_NAME_CATEGORY);
                TocFile tocFile = new TocFile(pluginId, file, primary, locale, extradir, category);
                tocFiles.add(tocFile);
            }
        }
        return (TocFile[])tocFiles.toArray(new TocFile[tocFiles.size()]);
    }
}
