/*******************************************************************************
 * Copyright (c) 2006, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     Brad Reynolds - bug 164247
 *     Brad Reynolds - bug 164134
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.jface.databinding.viewers.ObservableMapLabelProvider;

import dwt.dwthelper.utils;

import java.util.Set;

import dwt.graphics.Image;
import dwtx.core.databinding.observable.map.IMapChangeListener;
import dwtx.core.databinding.observable.map.IObservableMap;
import dwtx.core.databinding.observable.map.MapChangeEvent;
import dwtx.jface.viewers.ILabelProvider;
import dwtx.jface.viewers.ITableLabelProvider;
import dwtx.jface.viewers.LabelProvider;
import dwtx.jface.viewers.LabelProviderChangedEvent;

/**
 * @since 1.1
 * 
 */
public class ObservableMapLabelProvider extends LabelProvider
        implements ILabelProvider, ITableLabelProvider {

    private final IObservableMap[] attributeMaps;

    private IMapChangeListener mapChangeListener = new IMapChangeListener() {
        public void handleMapChange(MapChangeEvent event) {
            Set affectedElements = event.diff.getChangedKeys();
            LabelProviderChangedEvent newEvent = new LabelProviderChangedEvent(
                    ObservableMapLabelProvider.this, affectedElements
                            .toArray());
            fireLabelProviderChanged(newEvent);
        }
    };

    /**
     * @param attributeMap
     */
    public ObservableMapLabelProvider(IObservableMap attributeMap) {
        this(new IObservableMap[] { attributeMap });
    }

    /**
     * @param attributeMaps
     */
    public ObservableMapLabelProvider(IObservableMap[] attributeMaps) {
        System.arraycopy(attributeMaps, 0, this.attributeMaps = attributeMaps, 0, attributeMaps.length);
        for (int i = 0; i < attributeMaps.length; i++) {
            attributeMaps[i].addMapChangeListener(mapChangeListener);
        }
    }

    public void dispose() {
        for (int i = 0; i < attributeMaps.length; i++) {
            attributeMaps[i].removeMapChangeListener(mapChangeListener);
        }
        super.dispose();
    }

    public Image getImage(Object element) {
        return null;
    }

    public String getText(Object element) {
        return getColumnText(element, 0);
    }

    public Image getColumnImage(Object element, int columnIndex) {
        return null;
    }

    public String getColumnText(Object element, int columnIndex) {
        if (columnIndex < attributeMaps.length) {
            Object result = attributeMaps[columnIndex].get(element);
            return result is null ? "" : result.toString(); //$NON-NLS-1$
        }
        return null;
    }

}
