/*******************************************************************************
 * Copyright (c) 2006 The Pampered Chef, Inc. and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     The Pampered Chef, Inc. - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 ******************************************************************************/

module dwtx.jface.internal.databinding.provisional.swt.AbstractSWTObservableValue;

import dwt.dwthelper.utils;


import dwt.events.DisposeEvent;
import dwt.events.DisposeListener;
import dwt.widgets.Widget;
import dwtx.core.databinding.observable.Realm;
import dwtx.core.databinding.observable.value.AbstractObservableValue;
import dwtx.jface.databinding.swt.ISWTObservableValue;
import dwtx.jface.databinding.swt.SWTObservables;

/**
 * NON-API - An abstract superclass for observable values that gurantees that the 
 * observable will be disposed when the control to which it is attached is
 * disposed.
 * 
 * @since 1.1
 */
public abstract class AbstractSWTObservableValue extends AbstractObservableValue implements ISWTObservableValue {

    private final Widget widget;

    /**
     * Standard constructor for an DWT ObservableValue.  Makes sure that
     * the observable gets disposed when the DWT widget is disposed.
     * 
     * @param widget
     */
    protected AbstractSWTObservableValue(Widget widget) {
        this(SWTObservables.getRealm(widget.getDisplay()), widget);
    }
    
    /**
     * Constructor that allows for the setting of the realm. Makes sure that the
     * observable gets disposed when the DWT widget is disposed.
     * 
     * @param realm
     * @param widget
     * @since 1.2
     */
    protected AbstractSWTObservableValue(Realm realm, Widget widget) {
        super(realm);
        this.widget = widget;
        widget.addDisposeListener(disposeListener);
    }
    
    private DisposeListener disposeListener = new DisposeListener() {
        public void widgetDisposed(DisposeEvent e) {
            AbstractSWTObservableValue.this.dispose();
        }
    };

    /**
     * @return Returns the widget.
     */
    public Widget getWidget() {
        return widget;
    }
}
