/*******************************************************************************
 * Copyright (c) 2005, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     Matthew Hall - bug 223123
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.jface.internal.databinding.provisional.viewers.ViewerLabelProvider;

import dwt.dwthelper.utils;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import dwt.graphics.Image;
import dwtx.core.databinding.util.Policy;
import dwtx.core.runtime.IStatus;
import dwtx.core.runtime.Status;
import dwtx.jface.viewers.ILabelProvider;
import dwtx.jface.viewers.ILabelProviderListener;
import dwtx.jface.viewers.IViewerLabelProvider;
import dwtx.jface.viewers.LabelProviderChangedEvent;
import dwtx.jface.viewers.ViewerLabel;

/**
 * NON-API - Generic viewer label provider.
 * @since 1.1
 *
 */
public class ViewerLabelProvider implements IViewerLabelProvider,
        ILabelProvider {

    private List listeners = new ArrayList();

    /**
     * Subclasses should override this method. They should not call the base
     * class implementation.
     */
    public void updateLabel(ViewerLabel label, Object element) {
        label.setText(element.toString());
    }

    protected final void fireChangeEvent(Collection changes) {
        final LabelProviderChangedEvent event = new LabelProviderChangedEvent(
                this, changes.toArray());
        ILabelProviderListener[] listenerArray = (ILabelProviderListener[]) listeners
                .toArray(new ILabelProviderListener[listeners.size()]);
        for (int i = 0; i < listenerArray.length; i++) {
            ILabelProviderListener listener = listenerArray[i];
            try {
                listener.labelProviderChanged(event);
            } catch (Exception e) {
                Policy.getLog().log(
                        new Status(IStatus.ERROR, Policy.JFACE_DATABINDING, e
                                .getLocalizedMessage(), e));
            }
        }
    }

    public final Image getImage(Object element) {
        ViewerLabel label = new ViewerLabel("", null); //$NON-NLS-1$
        updateLabel(label, element);
        return label.getImage();
    }

    public final String getText(Object element) {
        ViewerLabel label = new ViewerLabel("", null); //$NON-NLS-1$
        updateLabel(label, element);
        return label.getText();
    }

    public void addListener(ILabelProviderListener listener) {
        listeners.add(listener);
    }

    public void dispose() {
        listeners.clear();
    }

    public final bool isLabelProperty(Object element, String property) {
        return true;
    }

    public void removeListener(ILabelProviderListener listener) {
        listeners.remove(listener);
    }

}
