/*******************************************************************************
 * Copyright (c) 2005, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     Brad Reynolds - bug 164653
 *     Ashley Cambrell - bug 198904
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.jface.internal.databinding.swt.ButtonObservableValue;

import dwt.dwthelper.utils;


import dwt.DWT;
import dwt.widgets.Button;
import dwt.widgets.Event;
import dwt.widgets.Listener;
import dwtx.core.databinding.observable.Diffs;
import dwtx.core.databinding.observable.Realm;
import dwtx.jface.internal.databinding.provisional.swt.AbstractSWTObservableValue;

/**
 * @since 1.0
 * 
 */
public class ButtonObservableValue extends AbstractSWTObservableValue {

    private final Button button;

    private bool selectionValue;

    private Listener updateListener = new Listener() {
        public void handleEvent(Event event) {
            bool oldSelectionValue = selectionValue;
            selectionValue = button.getSelection();
                        
            notifyIfChanged(oldSelectionValue, selectionValue);
        }
    };

    /**
     * @param button
     */
    public ButtonObservableValue(Button button) {
        super(button);
        this.button = button;
        init();
    }
    
    /**
     * @param realm
     * @param button
     */
    public ButtonObservableValue(Realm realm, Button button) {
        super(realm, button);
        this.button = button;
        init();
    }
    
    private void init() {
        button.addListener(DWT.Selection, updateListener);
        button.addListener(DWT.DefaultSelection, updateListener);       
    }

    public void doSetValue(final Object value) {
        bool oldSelectionValue = selectionValue;
        selectionValue = value is null ? false : ((Boolean) value)
                .booleanValue();
        
        button.setSelection(selectionValue);
        notifyIfChanged(oldSelectionValue, selectionValue);
    }

    public Object doGetValue() {
        return button.getSelection() ? Boolean.TRUE : Boolean.FALSE;
    }

    public Object getValueType() {
        return Boolean.TYPE;
    }
    
    /*
     * (non-Javadoc)
     *
     * @see dwtx.core.databinding.observable.value.AbstractObservableValue#dispose()
     */
    public synchronized void dispose() {
        super.dispose();

        if (!button.isDisposed()) {
            button.removeListener(DWT.Selection, updateListener);
            button.removeListener(DWT.DefaultSelection, updateListener);
        }
    }

    /**
     * Notifies consumers with a value change event only if a change occurred.
     * 
     * @param oldValue
     * @param newValue
     */
    private void notifyIfChanged(bool oldValue, bool newValue) {
        if (oldValue !is newValue) {
            fireValueChange(Diffs.createValueDiff(oldValue ? Boolean.TRUE : Boolean.FALSE,
                    newValue ? Boolean.TRUE : Boolean.FALSE));
        }       
    }
}
