/*******************************************************************************
 * Copyright (c) 2005, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     Brad Reynolds - bug 164653
 *     Ashley Cambrell - bug 198904
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.jface.internal.databinding.swt.CComboSingleSelectionObservableValue;

import dwt.dwthelper.utils;


import dwt.custom.CCombo;
import dwt.events.SelectionEvent;
import dwt.events.SelectionListener;
import dwtx.core.databinding.observable.Realm;

/**
 * @since 1.0
 *
 */
public class CComboSingleSelectionObservableValue extends
        SingleSelectionObservableValue {

    private SelectionListener selectionListener;

    /**
     * @param combo
     */
    public CComboSingleSelectionObservableValue(CCombo combo) {
        super(combo);
    }
    
    /**
     * @param realm
     * @param combo
     */
    public CComboSingleSelectionObservableValue(Realm realm, CCombo combo) {
        super(realm, combo);
    }

    private CCombo getCCombo() {
        return (CCombo) getWidget();
    }

    protected void doAddSelectionListener(final Runnable runnable) {
        selectionListener = new SelectionListener() {
            public void widgetDefaultSelected(SelectionEvent e) {
                runnable.run();
            }

            public void widgetSelected(SelectionEvent e) {
                runnable.run();
            }
        };
        getCCombo().addSelectionListener(selectionListener);
    }

    protected int doGetSelectionIndex() {
        return getCCombo().getSelectionIndex();
    }

    protected void doSetSelectionIndex(int index) {
        getCCombo().setText(getCCombo().getItem(index));
    }

    /*
     * (non-Javadoc)
     *
     * @see dwtx.core.databinding.observable.value.AbstractObservableValue#dispose()
     */
    public synchronized void dispose() {
        super.dispose();
        if (selectionListener !is null && !getCCombo().isDisposed()) {
            getCCombo().removeSelectionListener(selectionListener);
        }

    }
}
