/*******************************************************************************
 * Copyright (c) 2003, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.osgi.framework.internal.core.ExportedPackageImpl;

import dwt.dwthelper.utils;

import java.util.ArrayList;

import org.osgi.framework.Bundle;
import org.osgi.framework.Constants;
import org.osgi.framework.Version;
import org.osgi.service.packageadmin.ExportedPackage;

import dwtx.osgi.service.resolver.BundleDescription;
import dwtx.osgi.service.resolver.ExportPackageDescription;

public class ExportedPackageImpl implements ExportedPackage {

    private final String specVersion;
    private final ExportPackageDescription exportedPackage;
    private final BundleLoaderProxy supplier;

    public ExportedPackageImpl(ExportPackageDescription exportedPackage, BundleLoaderProxy supplier) {
        this.exportedPackage = exportedPackage;
        this.supplier = supplier;
        Version version = exportedPackage.getVersion();
        if (version !is null)
            this.specVersion = version.toString();
        else
            this.specVersion = null;
    }

    public String getName() {
        return exportedPackage.getName();
    }

    public org.osgi.framework.Bundle getExportingBundle() {
        if (supplier.isStale())
            return null;
        return supplier.getBundleHost();
    }

    /*
     * get the bundle without checking if it is stale
     */
    AbstractBundle getBundle() {
        return supplier.getBundleHost();
    }

    public Bundle[] getImportingBundles() {
        if (supplier.isStale())
            return null;
        AbstractBundle bundle = (AbstractBundle) getExportingBundle();
        if (bundle is null)
            return null;
        AbstractBundle[] bundles = bundle.framework.getAllBundles();
        ArrayList importers = new ArrayList(10);
        PackageSource supplierSource = supplier.createPackageSource(exportedPackage, false);
        for (int i = 0; i < bundles.length; i++) {
            if (!(bundles[i] instanceof BundleHost))
                continue;
            BundleLoader loader = ((BundleHost) bundles[i]).getBundleLoader();
            if (loader is null || loader.getBundle() is supplier.getBundle())
                continue; // do not include include the exporter of the package
            PackageSource importerSource = loader.getPackageSource(getName());
            if (supplierSource !is null && supplierSource.hasCommonSource(importerSource))
                importers.add(bundles[i]);
        }
        return (Bundle[]) importers.toArray(new Bundle[importers.size()]);
    }

    public String getSpecificationVersion() {
        return specVersion;
    }

    public Version getVersion() {
        return exportedPackage.getVersion();
    }

    public bool isRemovalPending() {
        BundleDescription exporter = exportedPackage.getExporter();
        if (exporter !is null)
            return exporter.isRemovalPending();
        return true;
    }

    public String toString() {
        StringBuffer result = new StringBuffer(getName());
        if (specVersion !is null) {
            result.append("; ").append(Constants.VERSION_ATTRIBUTE); //$NON-NLS-1$
            result.append("=\"").append(specVersion).append("\""); //$NON-NLS-1$//$NON-NLS-2$
        }
        return result.toString();
    }
}
