/*******************************************************************************
 * Copyright (c) 2003, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.osgi.framework.internal.core.SingleSourcePackage;

import dwt.dwthelper.utils;

import java.net.URL;
import java.util.Enumeration;

public class SingleSourcePackage extends PackageSource {
    BundleLoaderProxy supplier;

    public SingleSourcePackage(String id, BundleLoaderProxy supplier) {
        super(id);
        this.supplier = supplier;
    }

    public SingleSourcePackage[] getSuppliers() {
        return new SingleSourcePackage[] {this};
    }

    public String toString() {
        return id + " -> " + supplier; //$NON-NLS-1$
    }

    public Class loadClass(String name) throws ClassNotFoundException {
        return supplier.getBundleLoader().findLocalClass(name);
    }

    public URL getResource(String name) {
        return supplier.getBundleLoader().findLocalResource(name);
    }

    public Enumeration getResources(String name) {
        return supplier.getBundleLoader().findLocalResources(name);
    }

    public bool equals(Object source) {
        if (this is source)
            return true;
        if (!(source instanceof SingleSourcePackage))
            return false;
        SingleSourcePackage singleSource = (SingleSourcePackage) source;
        // we do an is test on id because the id is interned in the constructor of PackageSource
        return supplier is singleSource.supplier && id is singleSource.getId();
    }
}
