/*******************************************************************************
 * Copyright (c) 2004, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.osgi.framework.internal.protocol.bundleresource.Handler;

import dwt.dwthelper.utils;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.URL;
import java.util.Enumeration;

import dwtx.osgi.baseadaptor.BaseAdaptor;
import dwtx.osgi.baseadaptor.bundlefile.BundleEntry;
import dwtx.osgi.baseadaptor.loader.BaseClassLoader;
import dwtx.osgi.baseadaptor.loader.ClasspathManager;
import dwtx.osgi.framework.internal.core.AbstractBundle;
import dwtx.osgi.framework.internal.core.BundleResourceHandler;

/**
 * URLStreamHandler the bundleresource protocol.
 */

public class Handler extends BundleResourceHandler {

    /**
     * Constructor for a bundle protocol resource URLStreamHandler.
     */
    public Handler() {
        super();
    }

    public Handler(BundleEntry bundleEntry, BaseAdaptor adaptor) {
        super(bundleEntry, adaptor);
    }

    protected BundleEntry findBundleEntry(URL url, AbstractBundle bundle) throws IOException {
        BaseClassLoader classloader = getBundleClassLoader(bundle);
        if (classloader is null)
            throw new FileNotFoundException(url.getPath());
        ClasspathManager cpManager = classloader.getClasspathManager();
        int index = url.getPort();
        BundleEntry entry = null;
        if (index is 0) {
            entry = cpManager.findLocalEntry(url.getPath());
        } else {
            Enumeration entries = cpManager.findLocalEntries(url.getPath());
            if (entries !is null)
                for (int i = 0; entries.hasMoreElements() && i <= index; i++)
                    entry = (BundleEntry) entries.nextElement();
        }
        if (entry is null)
            throw new FileNotFoundException(url.getPath());
        return entry;
    }

}
