/*******************************************************************************
 * Copyright (c) 2005, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.osgi.internal.baseadaptor.BundleUninstall;

import dwt.dwthelper.utils;

import java.io.IOException;

import org.osgi.framework.BundleEvent;
import org.osgi.framework.BundleException;

import dwtx.osgi.baseadaptor.BaseData;
import dwtx.osgi.framework.adaptor.BundleData;
import dwtx.osgi.framework.adaptor.BundleOperation;

public class BundleUninstall implements BundleOperation {
    private BaseData data;
    private BaseStorage storage;

    public BundleUninstall(BaseData data, BaseStorage storage) {
        this.data = data;
        this.storage = storage;
    }

    /**
     * Perform the change to persistent storage.
     *
     * @return Bundle object for the target bundle.
     * @throws BundleException If a failure occured modifiying peristent storage.
     */
    public BundleData begin() throws BundleException {
        return data;
    }

    /**
     * Commit the change to persistent storage.
     *
     * @param postpone If true, the bundle's persistent
     * storage cannot be immediately reclaimed.
     * @throws BundleException If a failure occured modifiying peristent storage.
     */
    public void commit(bool postpone) throws BundleException {
        BaseStorageHook storageHook = (BaseStorageHook) data.getStorageHook(BaseStorageHook.KEY);
        try {
            storageHook.delete(postpone, BaseStorageHook.DEL_BUNDLE_STORE);
        } catch (IOException e) {
            // nothing we can do
        }
        storage.processExtension(data, BaseStorage.EXTENSION_UNINSTALLED);
        data.setLastModified(System.currentTimeMillis());
        storage.updateState(data, BundleEvent.UNINSTALLED);
    }

    /**
     * Undo the change to persistent storage.
     *
     * @throws BundleException If a failure occured modifiying peristent storage.
     */
    public void undo() throws BundleException {
        // do nothing
    }

}
