/*******************************************************************************
 * Copyright (c) 2006, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.osgi.internal.module.GenericConstraint;

import dwt.dwthelper.utils;

import java.util.ArrayList;

import dwtx.osgi.service.resolver.VersionConstraint;

public class GenericConstraint extends ResolverConstraint {
    private ArrayList matchingCapability;

    GenericConstraint(ResolverBundle bundle, VersionConstraint constraint) {
        super(bundle, constraint);
    }

    bool isOptional() {
        return false;
    }

    bool isSatisfiedBy(VersionSupplier vs) {
        return !vs.getResolverBundle().isUninstalled() && getVersionConstraint().isSatisfiedBy(vs.getBaseDescription());
    }

    public void setMatchingCapability(GenericCapability capability) {
        if (capability is null) {
            matchingCapability = null;
            return;
        }
        if (matchingCapability is null)
            matchingCapability = new ArrayList(1);
        matchingCapability.add(capability);
    }

    public GenericCapability[] getMatchingCapabilities() {
        return matchingCapability is null || matchingCapability.size() is 0 ? null : (GenericCapability[]) matchingCapability.toArray(new GenericCapability[matchingCapability.size()]);
    }

    void removeMatchingCapability(GenericCapability capability) {
        if (matchingCapability !is null)
            matchingCapability.remove(capability);
    }
}
