/*******************************************************************************
 * Copyright (c) 2003, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.osgi.internal.resolver.HostSpecificationImpl;

import dwt.dwthelper.utils;

import dwtx.osgi.service.resolver.BaseDescription;
import dwtx.osgi.service.resolver.BundleDescription;
import dwtx.osgi.service.resolver.HostSpecification;

public class HostSpecificationImpl extends VersionConstraintImpl implements HostSpecification {

    private BundleDescription[] hosts;
    private bool multihost = false;

    public bool isSatisfiedBy(BaseDescription supplier) {
        if (!(supplier instanceof BundleDescription))
            return false;
        BundleDescription candidate = (BundleDescription) supplier;
        if (candidate.getHost() !is null)
            return false;
        if (getName() !is null && getName().equals(candidate.getSymbolicName()) && (getVersionRange() is null || getVersionRange().isIncluded(candidate.getVersion())))
            return true;
        return false;
    }

    public BundleDescription[] getHosts() {
        return hosts is null ? BundleDescriptionImpl.EMPTY_BUNDLEDESCS : hosts;
    }

    public bool isResolved() {
        return hosts !is null && hosts.length > 0;
    }

    /*
     * The resolve algorithm will call this method to set the hosts.
     */
    void setHosts(BundleDescription[] hosts) {
        this.hosts = hosts;
    }

    public String toString() {
        return "Fragment-Host: " + getName() + "; bundle-version=\"" + getVersionRange() + "\""; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
    }

    public BaseDescription getSupplier() {
        if (hosts is null || hosts.length is 0)
            return null;
        return hosts[0];
    }

    public bool isMultiHost() {
        return multihost;
    }

    void setIsMultiHost(bool multihost) {
        this.multihost = multihost;
    }
}
