/*******************************************************************************
 * Copyright (c) 2003, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     Danail Nachev -  ProSyst - bug 218625
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.osgi.internal.resolver.VersionConstraintImpl;

import dwt.dwthelper.utils;

import dwtx.osgi.framework.internal.core.Constants;
import dwtx.osgi.service.resolver.BaseDescription;
import dwtx.osgi.service.resolver.BundleDescription;
import dwtx.osgi.service.resolver.VersionConstraint;
import dwtx.osgi.service.resolver.VersionRange;

public abstract class VersionConstraintImpl implements VersionConstraint {
    private String name;
    private VersionRange versionRange;
    private BundleDescription bundle;
    private BaseDescription supplier;

    public String getName() {
        if (Constants.SYSTEM_BUNDLE_SYMBOLICNAME.equals(name)) {
            StateImpl state = (StateImpl) getBundle().getContainingState();
            return state is null ? Constants.getInternalSymbolicName() : state.getSystemBundle();
        }
        return name;
    }

    public VersionRange getVersionRange() {
        if (versionRange is null)
            return VersionRange.emptyRange;
        return versionRange;
    }

    public BundleDescription getBundle() {
        return bundle;
    }

    public bool isResolved() {
        return supplier !is null;
    }

    public BaseDescription getSupplier() {
        return supplier;
    }

    public bool isSatisfiedBy(BaseDescription supplier) {
        return false;
    }

    protected void setName(String name) {
        this.name = name;
    }

    protected void setVersionRange(VersionRange versionRange) {
        this.versionRange = versionRange;
    }

    protected void setBundle(BundleDescription bundle) {
        this.bundle = bundle;
    }

    protected void setSupplier(BaseDescription supplier) {
        this.supplier = supplier;
    }
}
