/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.osgi.internal.signedcontent.BundleInstallListener;

import dwt.dwthelper.utils;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleEvent;
import org.osgi.framework.SynchronousBundleListener;

import dwtx.osgi.baseadaptor.BaseData;
import dwtx.osgi.framework.internal.core.AbstractBundle;
import dwtx.osgi.internal.provisional.service.security.AuthorizationEngine;
import dwtx.osgi.signedcontent.SignedContent;

public class BundleInstallListener implements SynchronousBundleListener {

    public void bundleChanged(BundleEvent event) {
        Bundle bundle = event.getBundle();
        switch (event.getType()) {
            case BundleEvent.UPDATED :
                // fall through to INSTALLED
            case BundleEvent.INSTALLED :
                TrustEngineListener listener = TrustEngineListener.getInstance();
                AuthorizationEngine authEngine = listener is null ? null : listener.getAuthorizationEngine();
                if (authEngine !is null) {
                    BaseData baseData = (BaseData) ((AbstractBundle) bundle).getBundleData();
                    SignedStorageHook hook = (SignedStorageHook) baseData.getStorageHook(SignedStorageHook.KEY);
                    SignedContent signedContent = hook !is null ? hook.signedContent : null;
                    authEngine.authorize(signedContent, bundle);
                }
                break;
            default :
                break;
        }
    }
}
