/*******************************************************************************
 * Copyright (c) 2006, 2007 IBM Corporation and others. All rights reserved.
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v1.0 which accompanies this distribution,
 * and is available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 ******************************************************************************/
module dwtx.osgi.internal.signedcontent.PKCS7DateParser;

import dwt.dwthelper.utils;

import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.SignatureException;
import java.security.cert.CertificateException;
import java.util.Arrays;
import java.util.Date;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

public class PKCS7DateParser {

    static Date parseDate(PKCS7Processor pkcs7Processor, String signer, String file) throws CertificateException, NoSuchAlgorithmException, InvalidKeyException, SignatureException, NoSuchProviderException {
        Map unsignedAttrs = pkcs7Processor.getUnsignedAttrs();
        if (unsignedAttrs !is null) {
            // get the timestamp construct
            byte[] timeStampConstruct = retrieveTimeStampConstruct(unsignedAttrs);

            // there is a timestamp in the signer info
            if (timeStampConstruct !is null) {
                PKCS7Processor timestampProcess = new PKCS7Processor(timeStampConstruct, 0, timeStampConstruct.length, signer, file);
                timestampProcess.verifyCerts();
                pkcs7Processor.setTSACertificates(timestampProcess.getCertificates());
                return timestampProcess.getSigningTime();
            }
        }
        return null;
    }

    private static byte[] retrieveTimeStampConstruct(Map unsignedAttrs) {
        Set objIDs = unsignedAttrs.keySet();
        Iterator iter = objIDs.iterator();
        while (iter.hasNext()) {
            int[] objID = (int[]) iter.next();
            if (Arrays.equals(SignedContentConstants.TIMESTAMP_OID, objID)) {
                return (byte[]) unsignedAttrs.get(objID);
            }
        }
        return null;
    }
}
