/*******************************************************************************
 * Copyright (c) 2004, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.PerspectiveAdapter;

import dwt.dwthelper.utils;

/**
 * This adapter class provides default implementations for the methods described
 * by the <code>IPerspectiveListener</code> interface and its extension
 * interfaces.
 * <p>
 * Classes that wish to deal with events which occur as perspectives are added,
 * removed, activated and changed, can extend this class and override only the
 * methods which they are interested in.
 * </p>
 * 
 * @see dwtx.ui.IPerspectiveListener
 * @see dwtx.ui.IPerspectiveListener2
 * @see dwtx.ui.IPerspectiveListener3
 * @see dwtx.ui.IPerspectiveListener4
 * @since 3.1
 */
public class PerspectiveAdapter implements IPerspectiveListener4 {

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.IPerspectiveListener3#perspectiveOpened(dwtx.ui.IWorkbenchPage,
     *      dwtx.ui.IPerspectiveDescriptor)
     */
    public void perspectiveOpened(IWorkbenchPage page,
            IPerspectiveDescriptor perspective) {
        // do nothing
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.IPerspectiveListener3#perspectiveClosed(dwtx.ui.IWorkbenchPage,
     *      dwtx.ui.IPerspectiveDescriptor)
     */
    public void perspectiveClosed(IWorkbenchPage page,
            IPerspectiveDescriptor perspective) {
        // do nothing
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.IPerspectiveListener2#perspectiveChanged(dwtx.ui.IWorkbenchPage,
     *      dwtx.ui.IPerspectiveDescriptor,
     *      dwtx.ui.IWorkbenchPartReference, java.lang.String)
     */
    public void perspectiveChanged(IWorkbenchPage page,
            IPerspectiveDescriptor perspective,
            IWorkbenchPartReference partRef, String changeId) {
        // do nothing
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.IPerspectiveListener#perspectiveActivated(dwtx.ui.IWorkbenchPage,
     *      dwtx.ui.IPerspectiveDescriptor)
     */
    public void perspectiveActivated(IWorkbenchPage page,
            IPerspectiveDescriptor perspective) {
        // do nothing
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.IPerspectiveListener#perspectiveChanged(dwtx.ui.IWorkbenchPage,
     *      dwtx.ui.IPerspectiveDescriptor, java.lang.String)
     */
    public void perspectiveChanged(IWorkbenchPage page,
            IPerspectiveDescriptor perspective, String changeId) {
        // do nothing
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.IPerspectiveListener3#perspectiveDeactivated(dwtx.ui.IWorkbenchPage,
     *      dwtx.ui.IPerspectiveDescriptor)
     */
    public void perspectiveDeactivated(IWorkbenchPage page,
            IPerspectiveDescriptor perspective) {
        // do nothing
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.IPerspectiveListener3#perspectiveSavedAs(dwtx.ui.IWorkbenchPage,
     *      dwtx.ui.IPerspectiveDescriptor,
     *      dwtx.ui.IPerspectiveDescriptor)
     */
    public void perspectiveSavedAs(IWorkbenchPage page,
            IPerspectiveDescriptor oldPerspective,
            IPerspectiveDescriptor newPerspective) {
        // do nothing
    }

    /**
     * {@inheritDoc}
     * @since 3.2
     */
    public void perspectivePreDeactivate(IWorkbenchPage page,
            IPerspectiveDescriptor perspective) {
        // do nothing
    }
}
