/*******************************************************************************
 * Copyright (c) 2000, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.commands.KeyConfigurationEvent;

import dwt.dwthelper.utils;

/**
 * An instance of this class describes changes to an instance of
 * <code>IKeyConfiguration</code>.
 * <p>
 * This class is not intended to be extended by clients.
 * </p>
 * 
 * @since 3.0
 * @see IKeyConfigurationListener#keyConfigurationChanged(KeyConfigurationEvent)
 * @deprecated Please use the bindings support in the "dwtx.jface"
 * plug-in instead.
 * @see dwtx.jface.bindings.SchemeEvent
 */
public final class KeyConfigurationEvent {

    /**
     * whether the key configuration has become or active or inactive.
     */
    private final bool activeChanged;

    /**
     * Whether the key configuration has become defined or undefined.
     */
    private final bool definedChanged;

    /**
     * The key configuration that has changed; this value is never
     * <code>null</code>.
     */
    private final IKeyConfiguration keyConfiguration;

    /**
     * Whether the name of the key configuration has changed.
     */
    private final bool nameChanged;

    /**
     * Whether the parent identifier has changed.
     */
    private final bool parentIdChanged;

    /**
     * Creates a new instance of this class.
     * 
     * @param keyConfiguration
     *            the instance of the interface that changed.
     * @param activeChanged
     *            true, iff the active property changed.
     * @param definedChanged
     *            true, iff the defined property changed.
     * @param nameChanged
     *            true, iff the name property changed.
     * @param parentIdChanged
     *            true, iff the parentId property changed.
     */
    public KeyConfigurationEvent(IKeyConfiguration keyConfiguration,
            bool activeChanged, bool definedChanged, bool nameChanged,
            bool parentIdChanged) {
        if (keyConfiguration is null) {
            throw new NullPointerException();
        }

        this.keyConfiguration = keyConfiguration;
        this.activeChanged = activeChanged;
        this.definedChanged = definedChanged;
        this.nameChanged = nameChanged;
        this.parentIdChanged = parentIdChanged;
    }

    /**
     * Returns the instance of the interface that changed.
     * 
     * @return the instance of the interface that changed. Guaranteed not to be
     *         <code>null</code>.
     */
    public IKeyConfiguration getKeyConfiguration() {
        return keyConfiguration;
    }

    /**
     * Returns whether or not the active property changed.
     * 
     * @return true, iff the active property changed.
     */
    public bool hasActiveChanged() {
        return activeChanged;
    }

    /**
     * Returns whether or not the defined property changed.
     * 
     * @return true, iff the defined property changed.
     */
    public bool hasDefinedChanged() {
        return definedChanged;
    }

    /**
     * Returns whether or not the name property changed.
     * 
     * @return true, iff the name property changed.
     */
    public bool hasNameChanged() {
        return nameChanged;
    }

    /**
     * Returns whether or not the parentId property changed.
     * 
     * @return true, iff the parentId property changed.
     */
    public bool hasParentIdChanged() {
        return parentIdChanged;
    }
}
