/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 ******************************************************************************/

module dwtx.ui.internal.AbstractEvaluationHandler;

import dwt.dwthelper.utils;

import dwtx.core.expressions.Expression;
import dwtx.jface.util.IPropertyChangeListener;
import dwtx.jface.util.PropertyChangeEvent;
import dwtx.ui.PlatformUI;
import dwtx.ui.services.IEvaluationReference;
import dwtx.ui.services.IEvaluationService;

/**
 * This internal class serves as a foundation for any handler that would like
 * its enabled state controlled by core expressions and the IEvaluationService.
 * 
 * @since 3.3
 */
public abstract class AbstractEvaluationHandler extends AbstractEnabledHandler {
    private final static String PROP_ENABLED = "enabled"; //$NON-NLS-1$
    private IEvaluationService evaluationService;
    private IPropertyChangeListener enablementListener;
    private IEvaluationReference enablementRef;

    protected IEvaluationService getEvaluationService() {
        if (evaluationService is null) {
            evaluationService = (IEvaluationService) PlatformUI.getWorkbench()
                    .getService(IEvaluationService.class);
        }
        return evaluationService;
    }

    protected void registerEnablement() {
        enablementRef = getEvaluationService().addEvaluationListener(
                getEnabledWhenExpression(), getEnablementListener(),
                PROP_ENABLED);
    }

    protected abstract Expression getEnabledWhenExpression();

    /**
     * @return
     */
    private IPropertyChangeListener getEnablementListener() {
        if (enablementListener is null) {
            enablementListener = new IPropertyChangeListener() {
                public void propertyChange(PropertyChangeEvent event) {
                    if (event.getProperty() is PROP_ENABLED) {
                        if (event.getNewValue() instanceof Boolean) {
                            setEnabled(((Boolean) event.getNewValue())
                                    .booleanValue());
                        } else {
                            setEnabled(false);
                        }
                    }
                }
            };
        }
        return enablementListener;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.core.commands.AbstractHandler#dispose()
     */
    public void dispose() {
        if (enablementRef !is null) {
            evaluationService.removeEvaluationListener(enablementRef);
            enablementRef = null;
            enablementListener = null;
            evaluationService = null;
        }
        super.dispose();
    }
}
