/*******************************************************************************
 * Copyright (c) 2000, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.ActionSetSeparator;

import dwt.dwthelper.utils;


import dwt.DWT;
import dwt.widgets.Menu;
import dwt.widgets.MenuItem;
import dwt.widgets.ToolBar;
import dwt.widgets.ToolItem;
import dwtx.jface.action.ContributionItem;

/**
 * This class represents a pseudo-group defined by an action set.
 * It is not a real group ( aka GroupMarker ) because that would interfere with
 * the pre-existing groups in a menu or toolbar.
 */
public class ActionSetSeparator extends ContributionItem implements
        IActionSetContributionItem {
    private String actionSetId;

    /**
     * Constructs a new group marker.
     */
    public ActionSetSeparator(String groupName, String newActionSetId) {
        super(groupName);
        actionSetId = newActionSetId;
    }

    /* (non-Javadoc)
     * Method declared on IContributionItem.
     * Fills the given menu with a DWT separator MenuItem.
     */
    public void fill(Menu menu, int index) {
        if (index >= 0) {
            new MenuItem(menu, DWT.SEPARATOR, index);
        } else {
            new MenuItem(menu, DWT.SEPARATOR);
        }
    }

    /* (non-Javadoc)
     * Method declared on IContributionItem.
     * Fills the given tool bar with a DWT separator ToolItem.
     */
    public void fill(ToolBar toolbar, int index) {
        if (index >= 0) {
            new ToolItem(toolbar, DWT.SEPARATOR, index);
        } else {
            new ToolItem(toolbar, DWT.SEPARATOR);
        }
    }

    /**
     * Returns the action set id.
     */
    public String getActionSetId() {
        return actionSetId;
    }

    /** 
     * The <code>Separator</code> implementation of this <code>IContributionItem</code> 
     * method returns <code>true</code>
     */
    public bool isSeparator() {
        return true;
    }

    /**
     * Sets the action set id.
     */
    public void setActionSetId(String newActionSetId) {
        actionSetId = newActionSetId;
    }
}
