/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 ******************************************************************************/

module dwtx.ui.internal.ActivateEditorHandler;

import dwt.dwthelper.utils;

import dwtx.core.commands.ExecutionEvent;
import dwtx.core.commands.ExecutionException;
import dwtx.core.expressions.EvaluationResult;
import dwtx.core.expressions.Expression;
import dwtx.core.expressions.ExpressionInfo;
import dwtx.core.expressions.IEvaluationContext;
import dwtx.core.runtime.CoreException;
import dwtx.ui.IEditorPart;
import dwtx.ui.ISources;
import dwtx.ui.IViewReference;
import dwtx.ui.IWorkbenchPage;
import dwtx.ui.IWorkbenchPartReference;
import dwtx.ui.IWorkbenchWindow;
import dwtx.ui.handlers.HandlerUtil;

/**
 * Activates the most recently used editor in the current window.
 * <p>
 * Replacement for: ActivateEditorAction
 * </p>
 * 
 * @since 3.3
 */
public class ActivateEditorHandler extends AbstractEvaluationHandler {

    private Expression enabledWhen;

    public ActivateEditorHandler() {
        registerEnablement();
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.core.commands.IHandler#execute(dwtx.core.commands.ExecutionEvent)
     */
    public Object execute(ExecutionEvent event) throws ExecutionException {
        IWorkbenchWindow window = HandlerUtil
                .getActiveWorkbenchWindowChecked(event);
        IWorkbenchPage page = window.getActivePage();
        if (page !is null) {
            IEditorPart part = HandlerUtil.getActiveEditor(event);
            if (part !is null) {
                page.activate(part);
                part.setFocus();
            } else {
                IWorkbenchPartReference ref = page.getActivePartReference();
                if (ref instanceof IViewReference) {
                    if (((WorkbenchPage) page).isFastView((IViewReference) ref)) {
                        ((WorkbenchPage) page)
                                .toggleFastView((IViewReference) ref);
                    }
                }
            }
        }
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.internal.AbstractEvaluationHandler#getEnabledWhenExpression()
     */
    protected Expression getEnabledWhenExpression() {
        if (enabledWhen is null) {
            enabledWhen = new Expression() {
                public EvaluationResult evaluate(IEvaluationContext context)
                        throws CoreException {
                    IWorkbenchWindow window = InternalHandlerUtil
                            .getActiveWorkbenchWindow(context);
                    if (window !is null) {
                        if (window.getActivePage() !is null) {
                            return EvaluationResult.TRUE;
                        }
                    }
                    return EvaluationResult.FALSE;
                }

                /*
                 * (non-Javadoc)
                 * 
                 * @see dwtx.core.expressions.Expression#collectExpressionInfo(dwtx.core.expressions.ExpressionInfo)
                 */
                public void collectExpressionInfo(ExpressionInfo info) {
                    info
                            .addVariableNameAccess(ISources.ACTIVE_WORKBENCH_WINDOW_NAME);
                }
            };
        }
        return enabledWhen;
    }
}
