/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 ******************************************************************************/

module dwtx.ui.internal.CloseEditorHandler;

import dwt.dwthelper.utils;

import dwtx.core.commands.ExecutionEvent;
import dwtx.core.commands.ExecutionException;
import dwtx.core.expressions.EvaluationResult;
import dwtx.core.expressions.Expression;
import dwtx.core.expressions.ExpressionInfo;
import dwtx.core.expressions.IEvaluationContext;
import dwtx.core.runtime.CoreException;
import dwtx.ui.IEditorPart;
import dwtx.ui.ISources;
import dwtx.ui.IWorkbenchWindow;
import dwtx.ui.handlers.HandlerUtil;

/**
 * Closes the active editor.
 * <p>
 * Replacement for CloseEditorAction
 * </p>
 * 
 * @since 3.3
 * 
 */
public class CloseEditorHandler extends AbstractEvaluationHandler {

    private Expression enabledWhen;

    public CloseEditorHandler() {
        registerEnablement();
    }

    public Object execute(ExecutionEvent event) throws ExecutionException {
        IWorkbenchWindow window = HandlerUtil
                .getActiveWorkbenchWindowChecked(event);
        IEditorPart part = HandlerUtil.getActiveEditorChecked(event);
        window.getActivePage().closeEditor(part, true);

        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.internal.AbstractEvaluationHandler#getEnabledWhenExpression()
     */
    protected Expression getEnabledWhenExpression() {
        if (enabledWhen is null) {
            enabledWhen = new Expression() {
                public EvaluationResult evaluate(IEvaluationContext context)
                        throws CoreException {
                    IEditorPart part = InternalHandlerUtil
                            .getActiveEditor(context);
                    if (part !is null) {
                        return EvaluationResult.TRUE;

                    }
                    return EvaluationResult.FALSE;
                }

                /*
                 * (non-Javadoc)
                 * 
                 * @see dwtx.core.expressions.Expression#collectExpressionInfo(dwtx.core.expressions.ExpressionInfo)
                 */
                public void collectExpressionInfo(ExpressionInfo info) {
                    info.addVariableNameAccess(ISources.ACTIVE_EDITOR_NAME);
                }
            };
        }
        return enabledWhen;
    }
}
