/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 ******************************************************************************/

module dwtx.ui.internal.CloseOthersHandler;

import dwt.dwthelper.utils;

import dwtx.core.commands.ExecutionEvent;
import dwtx.core.commands.ExecutionException;
import dwtx.core.expressions.EvaluationResult;
import dwtx.core.expressions.Expression;
import dwtx.core.expressions.ExpressionInfo;
import dwtx.core.expressions.IEvaluationContext;
import dwtx.core.runtime.CoreException;
import dwtx.ui.IEditorReference;
import dwtx.ui.ISources;
import dwtx.ui.IWorkbenchPage;
import dwtx.ui.IWorkbenchWindow;
import dwtx.ui.handlers.HandlerUtil;

/**
 * Closes all editors except the one that is active.
 * <p>
 * Replacement for CloseOthersHandler
 * </p>
 * 
 * @since 3.3
 * 
 */
public class CloseOthersHandler extends AbstractEvaluationHandler {
    private Expression enabledWhen;

    public CloseOthersHandler() {
        registerEnablement();
    }

    public Object execute(ExecutionEvent event) throws ExecutionException {
        IWorkbenchWindow window = HandlerUtil
                .getActiveWorkbenchWindowChecked(event);
        IWorkbenchPage page = window.getActivePage();
        if (page !is null) {
            IEditorReference[] refArray = page.getEditorReferences();
            if (refArray !is null && refArray.length > 1) {
                IEditorReference[] otherEditors = new IEditorReference[refArray.length - 1];
                IEditorReference activeEditor = (IEditorReference) page
                        .getReference(page.getActiveEditor());
                for (int i = 0; i < refArray.length; i++) {
                    if (refArray[i] !is activeEditor)
                        continue;
                    System.arraycopy(refArray, 0, otherEditors, 0, i);
                    System.arraycopy(refArray, i + 1, otherEditors, i,
                            refArray.length - 1 - i);
                    break;
                }
                page.closeEditors(otherEditors, true);
            }
        }

        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.internal.AbstractEvaluationHandler#getEnabledWhenExpression()
     */
    protected Expression getEnabledWhenExpression() {
        if (enabledWhen is null) {
            enabledWhen = new Expression() {
                public EvaluationResult evaluate(IEvaluationContext context)
                        throws CoreException {
                    IWorkbenchWindow window = InternalHandlerUtil
                            .getActiveWorkbenchWindow(context);
                    if (window !is null) {
                        IWorkbenchPage page = window.getActivePage();
                        if (page !is null) {
                            IEditorReference[] refArray = page
                                    .getEditorReferences();
                            if (refArray !is null && refArray.length > 1) {
                                return EvaluationResult.TRUE;

                            }
                        }
                    }
                    return EvaluationResult.FALSE;
                }

                /*
                 * (non-Javadoc)
                 * 
                 * @see dwtx.core.expressions.Expression#collectExpressionInfo(dwtx.core.expressions.ExpressionInfo)
                 */
                public void collectExpressionInfo(ExpressionInfo info) {
                    info
                            .addVariableNameAccess(ISources.ACTIVE_WORKBENCH_WINDOW_NAME);
                }
            };
        }
        return enabledWhen;
    }
}
