/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 ******************************************************************************/

module dwtx.ui.internal.CycleEditorHandler;

import dwt.dwthelper.utils;


import dwt.DWT;
import dwt.widgets.Table;
import dwt.widgets.TableItem;
import dwtx.core.commands.Command;
import dwtx.core.commands.ParameterizedCommand;
import dwtx.ui.IEditorReference;
import dwtx.ui.IWorkbenchPart;
import dwtx.ui.commands.ICommandService;

/**
 * This is the handler for NextEditor and PrevEditor commands.
 * <p>
 * Replacement for CycleEditorAction
 * </p>
 * 
 * @since 3.3
 */
public class CycleEditorHandler extends CycleBaseHandler {

    /* (non-Javadoc)
     * @see dwtx.ui.internal.CycleBaseHandler#addItems(dwt.widgets.Table, dwtx.ui.internal.WorkbenchPage)
     */
    protected void addItems(Table table, WorkbenchPage page) {
        // TODO Auto-generated method stub
        IEditorReference refs[] = page.getSortedEditors();
        for (int i = refs.length - 1; i >= 0; i--) {
            TableItem item = null;
            item = new TableItem(table, DWT.NONE);
            if (refs[i].isDirty()) {
                item.setText("*" + refs[i].getTitle()); //$NON-NLS-1$
            } else {
                item.setText(refs[i].getTitle());
            }
            item.setImage(refs[i].getTitleImage());
            item.setData(refs[i]);
        }
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.CycleBaseHandler#getBackwardCommand()
     */
    protected ParameterizedCommand getBackwardCommand() {
        final ICommandService commandService = (ICommandService) window.getWorkbench().getService(ICommandService.class);
        final Command command = commandService.getCommand("dwtx.ui.window.previousEditor"); //$NON-NLS-1$
        ParameterizedCommand commandBack = new ParameterizedCommand(command, null);
        return commandBack;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.CycleBaseHandler#getForwardCommand()
     */
    protected ParameterizedCommand getForwardCommand() {
        final ICommandService commandService = (ICommandService) window.getWorkbench().getService(ICommandService.class);
        final Command command = commandService.getCommand("dwtx.ui.window.nextEditor"); //$NON-NLS-1$
        ParameterizedCommand commandF = new ParameterizedCommand(command, null);
        return commandF;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.CycleBaseHandler#getTableHeader()
     */
    protected String getTableHeader(IWorkbenchPart activePart) {
        // TODO Auto-generated method stub
        return WorkbenchMessages.CycleEditorAction_header;
    }

}
