/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 ******************************************************************************/

module dwtx.ui.internal.CyclePerspectiveHandler;

import dwt.dwthelper.utils;


import dwt.DWT;
import dwt.widgets.Table;
import dwt.widgets.TableItem;
import dwtx.core.commands.Command;
import dwtx.core.commands.ParameterizedCommand;
import dwtx.ui.IPerspectiveDescriptor;
import dwtx.ui.IWorkbenchPart;
import dwtx.ui.commands.ICommandService;
import dwtx.ui.model.PerspectiveLabelProvider;

/**
 * This handler is used to switch between perspectives using the keyboard.
 * <p>
 * Replacement for CyclePerspectiveAction
 * </p>
 * 
 * @since 3.3
 */
public class CyclePerspectiveHandler extends CycleBaseHandler {
    private PerspectiveLabelProvider labelProvider = new PerspectiveLabelProvider(
            false);
    
    /* (non-Javadoc)
     * @see dwtx.ui.internal.CycleBaseHandler#addItems(dwt.widgets.Table, dwtx.ui.internal.WorkbenchPage)
     */
    protected void addItems(Table table, WorkbenchPage page) {
        IPerspectiveDescriptor perspectives[] = page.getSortedPerspectives();
        for (int i = perspectives.length - 1; i >= 0; i--) {
            TableItem item = new TableItem(table, DWT.NONE);
            IPerspectiveDescriptor desc = perspectives[i];
            String text = labelProvider.getText(desc);
            if(text is null) {
                text = "";//$NON-NLS-1$
            }
            item.setText(text);
            item.setImage(labelProvider.getImage(desc));
            item.setData(desc);
        }

    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.CycleBaseHandler#getBackwardCommand()
     */
    protected ParameterizedCommand getBackwardCommand() {
        final ICommandService commandService = (ICommandService) window.getWorkbench().getService(ICommandService.class);
        final Command command = commandService.getCommand("dwtx.ui.window.previousPerspective"); //$NON-NLS-1$
        ParameterizedCommand commandBack = new ParameterizedCommand(command, null);
        return commandBack;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.CycleBaseHandler#getForwardCommand()
     */
    protected ParameterizedCommand getForwardCommand() {
        final ICommandService commandService = (ICommandService) window.getWorkbench().getService(ICommandService.class);
        final Command command = commandService.getCommand("dwtx.ui.window.nextPerspective"); //$NON-NLS-1$
        ParameterizedCommand commandF = new ParameterizedCommand(command, null);
        return commandF;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.CycleBaseHandler#getTableHeader()
     */
    protected String getTableHeader(IWorkbenchPart activePart) {
        return WorkbenchMessages.CyclePerspectiveAction_header;
    }

    /* (non-Javadoc)
     * @see dwtx.core.commands.AbstractHandler#dispose()
     */
    public void dispose() {
        if (labelProvider !is null) {
            labelProvider.dispose();
            labelProvider = null;
        }
        super.dispose();
    }
}
