/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 ******************************************************************************/

module dwtx.ui.internal.CycleViewHandler;

import dwt.dwthelper.utils;


import dwt.DWT;
import dwt.widgets.Table;
import dwt.widgets.TableItem;
import dwtx.core.commands.Command;
import dwtx.core.commands.ParameterizedCommand;
import dwtx.ui.IEditorReference;
import dwtx.ui.IWorkbenchPart;
import dwtx.ui.IWorkbenchPartReference;
import dwtx.ui.commands.ICommandService;

/**
 * This handler is used to switch between parts using the keyboard.
 * <p>
 * Replacement for CyclePartAction
 * </p>
 * 
 * @since 3.3
 *
 */
public class CycleViewHandler extends CycleBaseHandler {

    /* (non-Javadoc)
     * @see dwtx.ui.internal.CycleBaseHandler#addItems(dwt.widgets.Table, dwtx.ui.internal.WorkbenchPage)
     */
    protected void addItems(Table table, WorkbenchPage page) {
        // TODO Auto-generated method stub
        IWorkbenchPartReference refs[] = page.getSortedParts();
        bool includeEditor = true;

        for (int i = refs.length - 1; i >= 0; i--) {
            if (refs[i] instanceof IEditorReference) {
                if (includeEditor) {
                    IEditorReference activeEditor = (IEditorReference) refs[i];
                    TableItem item = new TableItem(table, DWT.NONE);
                    item.setText(WorkbenchMessages.CyclePartAction_editor);
                    item.setImage(activeEditor.getTitleImage());
                    item.setData(activeEditor);
                    includeEditor = false;
                }
            } else {
                TableItem item = new TableItem(table, DWT.NONE);
                item.setText(refs[i].getTitle());
                item.setImage(refs[i].getTitleImage());
                item.setData(refs[i]);
            }
        }
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.CycleBaseHandler#getBackwardCommand()
     */
    protected ParameterizedCommand getBackwardCommand() {
        // TODO Auto-generated method stub
        final ICommandService commandService = (ICommandService) window.getWorkbench().getService(ICommandService.class);
        final Command command = commandService.getCommand("dwtx.ui.window.previousView"); //$NON-NLS-1$
        ParameterizedCommand commandBack = new ParameterizedCommand(command, null);
        return commandBack;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.CycleBaseHandler#getForwardCommand()
     */
    protected ParameterizedCommand getForwardCommand() {
        // TODO Auto-generated method stub
        final ICommandService commandService = (ICommandService) window.getWorkbench().getService(ICommandService.class);
        final Command command = commandService.getCommand("dwtx.ui.window.nextView"); //$NON-NLS-1$
        ParameterizedCommand commandF = new ParameterizedCommand(command, null);
        return commandF;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.CycleBaseHandler#getTableHeader()
     */
    protected String getTableHeader(IWorkbenchPart activePart) {
        // TODO Auto-generated method stub
        return WorkbenchMessages.CyclePartAction_header;
    }

}
