/*******************************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 ******************************************************************************/

module dwtx.ui.internal.DefaultSaveable;

import dwt.dwthelper.utils;


import dwt.graphics.Image;
import dwtx.core.runtime.IProgressMonitor;
import dwtx.jface.resource.ImageDescriptor;
import dwtx.ui.ISaveablePart;
import dwtx.ui.IViewPart;
import dwtx.ui.IWorkbenchPage;
import dwtx.ui.IWorkbenchPart;
import dwtx.ui.IWorkbenchPart2;
import dwtx.ui.IWorkbenchPartReference;
import dwtx.ui.PartInitException;
import dwtx.ui.Saveable;

/**
 * A default {@link Saveable} implementation that wrappers a regular
 * workbench part (one that does not itself adapt to Saveable).
 * 
 * @since 3.2
 */
public class DefaultSaveable extends Saveable {

    private IWorkbenchPart part;

    /**
     * Creates a new DefaultSaveable.
     * 
     * @param part
     *            the part represented by this model
     */
    public DefaultSaveable(IWorkbenchPart part) {
        this.part = part;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.Saveable#doSave(dwtx.core.runtime.IProgressMonitor)
     */
    public void doSave(IProgressMonitor monitor) {
        if (part instanceof ISaveablePart) {
            ISaveablePart saveable = (ISaveablePart) part;
            saveable.doSave(monitor);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.Saveable#getName()
     */
    public String getName() {
        if (part instanceof IWorkbenchPart2) {
            return ((IWorkbenchPart2) part).getPartName();
        }
        return part.getTitle();
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.Saveable#getImageDescriptor()
     */
    public ImageDescriptor getImageDescriptor() {
        Image image = part.getTitleImage();
        if (image is null) {
            return null;
        }
        return ImageDescriptor.createFromImage(image);
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.Saveable#getToolTipText()
     */
    public String getToolTipText() {
        return part.getTitleToolTip();
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.Saveable#isDirty()
     */
    public bool isDirty() {
        if (part instanceof ISaveablePart) {
            return ((ISaveablePart) part).isDirty();
        }
        return false;
    }

    /* (non-Javadoc)
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return part.hashCode();
    }

    /* (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public bool equals(Object obj) {
        if (this is obj)
            return true;
        if (obj is null)
            return false;
        if (getClass() !is obj.getClass())
            return false;
        final DefaultSaveable other = (DefaultSaveable) obj;
        if (part is null) {
            if (other.part !is null)
                return false;
        } else if (!part.equals(other.part))
            return false;
        return true;
    }
    
    /* (non-Javadoc)
     * @see dwtx.ui.Saveable#show(dwtx.ui.IWorkbenchPage)
     */
    public bool show(IWorkbenchPage page) {
        IWorkbenchPartReference reference = page.getReference(part);
        if (reference !is null) {
            page.activate(part);
            return true;
        }
        if (part instanceof IViewPart) {
            IViewPart viewPart = (IViewPart) part;
            try {
                page.showView(viewPart.getViewSite().getId(), viewPart
                        .getViewSite().getSecondaryId(),
                        IWorkbenchPage.VIEW_ACTIVATE);
            } catch (PartInitException e) {
                return false;
            }
            return true;
        }
        return false;
    }

}
