/*******************************************************************************
 * Copyright (c) 2004, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.DefaultStackPresentationSite;

import dwt.dwthelper.utils;


import dwt.graphics.Point;
import dwtx.jface.action.IMenuManager;
import dwtx.ui.presentations.IPresentablePart;
import dwtx.ui.presentations.IStackPresentationSite;
import dwtx.ui.presentations.StackPresentation;

/**
 * @since 3.0
 */
public abstract class DefaultStackPresentationSite implements
        IStackPresentationSite {

    private StackPresentation presentation;

    private int state = IStackPresentationSite.STATE_RESTORED;

    private int activeState = StackPresentation.AS_INACTIVE;

    public DefaultStackPresentationSite() {

    }

    public void setPresentation(StackPresentation newPresentation) {
        presentation = newPresentation;
        if (presentation !is null) {
            presentation.setState(state);
            presentation.setActive(activeState);
        }
    }

    public StackPresentation getPresentation() {
        return presentation;
    }

    public int getState() {
        return state;
    }

    public void setActive(int activeState) {
        if (activeState !is this.activeState) {
            this.activeState = activeState;
            if (presentation !is null) {
                presentation.setActive(activeState);
            }
        }
    }

    public int getActive() {
        return activeState;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.skins.IStackPresentationSite#selectPart(dwtx.ui.internal.skins.IPresentablePart)
     */
    public void selectPart(IPresentablePart toSelect) {

        if (presentation !is null) {
            presentation.selectPart(toSelect);
        }
    }

    public void dispose() {
        if (presentation !is null) {
            presentation.dispose();
        }
        setPresentation(null);
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.skins.IPresentationSite#setState(int)
     */
    public void setState(int newState) {
        setPresentationState(newState);
    }

    public void setPresentationState(int newState) {
        state = newState;
        if (presentation !is null) {
            presentation.setState(newState);
        }
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.skins.IPresentablePart#isClosable()
     */
    public bool isCloseable(IPresentablePart part) {
        return part.isCloseable();
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.skins.IPresentationSite#dragStart(dwtx.ui.internal.skins.IPresentablePart, bool)
     */
    public void dragStart(IPresentablePart beingDragged, Point initialPosition,
            bool keyboard) {
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.skins.IPresentationSite#close(dwtx.ui.internal.skins.IPresentablePart)
     */
    public void close(IPresentablePart toClose) {
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.skins.IPresentationSite#dragStart(bool)
     */
    public void dragStart(Point initialPosition, bool keyboard) {
    }

    /* (non-Javadoc)
     * @see dwtx.ui.presentations.IStackPresentationSite#supportsState(int)
     */
    public bool supportsState(int state) {
        return true;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.presentations.IStackPresentationSite#getSelectedPart()
     */
    public abstract IPresentablePart getSelectedPart();

    public void addSystemActions(IMenuManager menuManager) {

    }

    public abstract bool isPartMoveable(IPresentablePart toMove);

    public abstract bool isStackMoveable();

}
