/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.EditorAreaTrimToolBar;

import dwt.dwthelper.utils;


import dwt.DWT;
import dwt.events.SelectionEvent;
import dwt.events.SelectionListener;
import dwt.graphics.Image;
import dwt.widgets.ToolBar;
import dwt.widgets.ToolItem;
import dwtx.jface.action.ContributionItem;
import dwtx.jface.action.IContributionItem;
import dwtx.jface.action.ToolBarManager;
import dwtx.ui.IPageLayout;
import dwtx.ui.IViewReference;
import dwtx.ui.internal.layout.TrimToolBarBase;

public class EditorAreaTrimToolBar extends TrimToolBarBase {
    private LayoutPart editorArea;
    private bool restoreOnUnzoom = false;
    
    // The orientation of the fast view pane when showing a view
    private int paneOrientation;

    public EditorAreaTrimToolBar(WorkbenchWindow wbw, LayoutPart editorArea) {
        super(IPageLayout.ID_EDITOR_AREA, DWT.TOP, wbw);
        
        this.editorArea = editorArea;       
        dock(DWT.TOP);
    }
    
    /**
     * Put the stack back into the presentation
     */
    protected void restoreToPresentation() {
        EditorSashContainer esc = (EditorSashContainer)editorArea;
        EditorStack curStack = esc.getUpperRightEditorStack(esc.getChildren());
        curStack.setMinimized(false);
    }

    public void initToolBarManager(final ToolBarManager mgr) {
        // Set up the ToolBar with a restore button
        IContributionItem restoreContrib = new ContributionItem() {
            public void fill(ToolBar parent, int index) {
                ToolItem restoreItem = new  ToolItem(mgr.getControl(), DWT.PUSH, index);        
                Image tbImage = WorkbenchImages.getImage(IWorkbenchGraphicConstants.IMG_ETOOL_RESTORE_TRIMPART);
                restoreItem.setImage(tbImage);       
                String menuTip = WorkbenchMessages.StandardSystemToolbar_Restore;
                restoreItem.setToolTipText(menuTip);
                restoreItem.addSelectionListener(new SelectionListener() {
                    public void widgetDefaultSelected(SelectionEvent e) {
                        restoreToPresentation();
                    }
                    public void widgetSelected(SelectionEvent e) {
                        restoreToPresentation();
                    }
                });
            }
        };
        mgr.add(restoreContrib);

        // Set up the ToolBar with a button represing the Editor Area
        IContributionItem eaContrib = new ContributionItem() {
            public void fill(ToolBar parent, int index) {
                ToolItem editorAreaItem = new  ToolItem(mgr.getControl(), DWT.PUSH, index);        
                Image tbImage = WorkbenchImages.getImage(IWorkbenchGraphicConstants.IMG_ETOOL_EDITOR_TRIMPART);
                editorAreaItem.setImage(tbImage);       
                String menuTip = WorkbenchMessages.EditorArea_Tooltip;
                editorAreaItem.setToolTipText(menuTip);
                editorAreaItem.addSelectionListener(new SelectionListener() {
                    public void widgetDefaultSelected(SelectionEvent e) {
                        restoreToPresentation();
                    }
                    public void widgetSelected(SelectionEvent e) {
                        restoreToPresentation();
                    }
                });
            }
        };
        mgr.add(eaContrib);
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.layout.TrimToolBarBase#hookControl(dwt.widgets.ToolBar)
     */
    public void hookControl(ToolBarManager mgr) {
        // Hook a drop Listener to the control
        // NOTE: the drop target is self-managing...it
        // both hooks the new target and removes it on dispose
        new FastViewDnDHandler(id, mgr, wbw);
    }
    
    /**
     * Sets whether or not the stack gets restored on an unzoom
     * operation.
     * 
     * @param restoreOnUnzoom
     */
    public void setRestoreOnUnzoom(bool restoreOnUnzoom) {
        this.restoreOnUnzoom = restoreOnUnzoom;
    }
    
    public bool restoreOnUnzoom() {
        return restoreOnUnzoom;
    }

    /**
     * @param ref
     * @param selected
     */
    public void setIconSelection(IViewReference ref, bool selected) {
        ToolItem item = ShowFastViewContribution.getItem(tbMgr.getControl(), ref);
        if (item !is null)
            item.setSelection(selected);
    }

    /**
     * @return Returns the paneOrientation.
     */
    public int getPaneOrientation() {
        return paneOrientation;
    }
}
