/*******************************************************************************
 * Copyright (c) 2005, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.FastViewBarContextMenuContribution;

import dwt.dwthelper.utils;


import dwt.DWT;
import dwt.events.SelectionAdapter;
import dwt.events.SelectionEvent;
import dwt.widgets.Menu;
import dwt.widgets.MenuItem;
import dwtx.jface.action.ContributionItem;
import dwtx.ui.IViewReference;

public class FastViewBarContextMenuContribution extends ContributionItem {
    private MenuItem orientationItem;
    private MenuItem restoreItem;
    private MenuItem closeItem;
    private FastViewBar bar;
    private RadioMenu radioButtons;
    private IViewReference selectedView;
    private IntModel currentOrientation = new IntModel(DWT.VERTICAL);
    
    private IChangeListener orientationChangeListener = new IChangeListener() {
        public void update(bool changed) {
            if (changed && selectedView !is null) {
                bar.setOrientation(selectedView, currentOrientation.get());
            }
        }
    };
    
    public FastViewBarContextMenuContribution(FastViewBar bar) {
        this.bar = bar;
        currentOrientation.addChangeListener(orientationChangeListener);
    }

    public void fill(Menu menu, int index) {
        // TODO Auto-generated method stub
        super.fill(menu, index);
        

        orientationItem = new MenuItem(menu, DWT.CASCADE, index++);
        {
            orientationItem.setText(WorkbenchMessages.FastViewBar_view_orientation);

            Menu orientationSwtMenu = new Menu(orientationItem);
            RadioMenu orientationMenu = new RadioMenu(orientationSwtMenu,
                    currentOrientation);
            orientationMenu
                    .addMenuItem(
                            WorkbenchMessages.FastViewBar_horizontal, new Integer(DWT.HORIZONTAL)); 
            orientationMenu
                    .addMenuItem(
                            WorkbenchMessages.FastViewBar_vertical, new Integer(DWT.VERTICAL)); 

            orientationItem.setMenu(orientationSwtMenu);
        }

        restoreItem = new MenuItem(menu, DWT.CHECK, index++);
        restoreItem.setText(WorkbenchMessages.ViewPane_fastView);
        restoreItem.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                bar.restoreView(selectedView);
            }
        });

        closeItem = new MenuItem(menu, DWT.NONE, index++);
        closeItem.setText(WorkbenchMessages.WorkbenchWindow_close); 
        closeItem.addSelectionListener(new SelectionAdapter() {

            public void widgetSelected(SelectionEvent e) {
                if (selectedView !is null) {
                    WorkbenchPage page = bar.getWindow().getActiveWorkbenchPage();
                    if (page !is null) {
                        page.hideView(selectedView);
                    }
                }
            }
        });

       
        // Set menu item enablement etc based on whether a view is selected
        WorkbenchPage page = bar.getWindow().getActiveWorkbenchPage();
        
        if (selectedView !is null) {
            restoreItem.setEnabled(page !is null && page.isMoveable(selectedView));
        } else {
            restoreItem.setEnabled(false);
        }
        restoreItem.setSelection(true);
        
        if (selectedView !is null) {
            closeItem
                    .setEnabled(page !is null && page.isCloseable(selectedView));
        } else {
            closeItem.setEnabled(false);
        }
        
        orientationItem.setEnabled(selectedView !is null);
        if (selectedView !is null) {
            // Set the new orientation, but avoid re-sending the event to our own
            // listener
            currentOrientation.set(bar.getOrientation(selectedView),
                    orientationChangeListener);
        }
    }
    
    public void setTarget(IViewReference selectedView) {
        this.selectedView = selectedView;
    }

    public bool isDynamic() {
        return true;
    }
    
    public void dispose() {
        if (radioButtons !is null) {
            radioButtons.dispose();
        }
        super.dispose();
    }
}
