/*******************************************************************************
 * Copyright (c) 2000, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.MessageLine;

import dwt.dwthelper.utils;


import dwt.DWT;
import dwt.custom.CLabel;
import dwt.graphics.Color;
import dwt.graphics.Image;
import dwt.widgets.Composite;
import dwtx.core.runtime.IStatus;
import dwtx.jface.resource.JFaceColors;
import dwtx.ui.ISharedImages;
import dwtx.ui.PlatformUI;

/**
 * A message line displaying a status.
 */
public class MessageLine extends CLabel {

    private Color fNormalMsgAreaBackground;

    /**
     * Creates a new message line as a child of the given parent.
     */
    public MessageLine(Composite parent) {
        this(parent, DWT.LEFT);
    }

    /**
     * Creates a new message line as a child of the parent and with the given DWT stylebits.
     */
    public MessageLine(Composite parent, int style) {
        super(parent, style);
        fNormalMsgAreaBackground = null;
    }

    private Image findImage(IStatus status) {
        if (status.isOK()) {
            return null;
        } else if (status.matches(IStatus.ERROR)) {
            return PlatformUI.getWorkbench().getSharedImages().getImage(
                    ISharedImages.IMG_OBJS_ERROR_TSK);
        } else if (status.matches(IStatus.WARNING)) {
            return PlatformUI.getWorkbench().getSharedImages().getImage(
                    ISharedImages.IMG_OBJS_WARN_TSK);
        } else if (status.matches(IStatus.INFO)) {
            return PlatformUI.getWorkbench().getSharedImages().getImage(
                    ISharedImages.IMG_OBJS_INFO_TSK);
        }
        return null;
    }

    /**
     * Sets the message and image to the given status.
     * <code>null</code> is a valid argument and will set the empty text and no image
     */
    public void setErrorStatus(IStatus status) {
        if (status !is null) {
            String message = status.getMessage();
            if (message !is null && message.length() > 0) {
                setText(message);
                setImage(findImage(status));
                setBackground(JFaceColors.getErrorBackground(getDisplay()));
                return;
            }
        }
        setText(""); //$NON-NLS-1$  
        setImage(null);
        setBackground(fNormalMsgAreaBackground);
    }

}

