/*******************************************************************************
 * Copyright (c) 2000, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.NavigationHistoryEditorInfo;

import dwt.dwthelper.utils;

import dwtx.core.runtime.IAdaptable;
import dwtx.ui.IEditorInput;
import dwtx.ui.IEditorPart;
import dwtx.ui.IElementFactory;
import dwtx.ui.IMemento;
import dwtx.ui.IPersistableElement;
import dwtx.ui.PlatformUI;
import dwtx.ui.XMLMemento;

/**
 * Keeps the info to save, restore or identify and editor.
 * Instances of this class are shared between history entries and there should be
 * only one instance making reference to the same editor. 
 */
public class NavigationHistoryEditorInfo {
    String editorID;

    IEditorInput editorInput;

    int refCount = 0;

    IMemento memento;

    NavigationHistoryEditorInfo(IEditorPart part) {
        editorID = part.getSite().getId();
        editorInput = part.getEditorInput();
    }

    NavigationHistoryEditorInfo(IMemento memento) {
        this.memento = memento;
    }

    bool isPersistable() {
        if (editorInput !is null) {
            IPersistableElement persistable = editorInput.getPersistable();
            return persistable !is null;
        }
        return memento !is null;
    }

    void handlePartClosed() {
        if (!isPersistable()) {
            return;
        }
        if (memento is null) {
            IPersistableElement persistable = editorInput.getPersistable();
            memento = XMLMemento
                    .createWriteRoot(IWorkbenchConstants.TAG_EDITOR);
            memento.putString(IWorkbenchConstants.TAG_ID, editorID);
            memento.putString(IWorkbenchConstants.TAG_FACTORY_ID, persistable
                    .getFactoryId());
            persistable.saveState(memento);
        }
        editorID = null;
        editorInput = null;
    }

    void restoreEditor() {
        if (memento is null) {
            return;
        }
        String factoryID = memento
                .getString(IWorkbenchConstants.TAG_FACTORY_ID);
        IElementFactory factory = PlatformUI.getWorkbench().getElementFactory(
                factoryID);
        if (factory !is null) {
            IAdaptable element = factory.createElement(memento);
            if (element instanceof IEditorInput) {
                editorInput = (IEditorInput) element;
                editorID = memento.getString(IWorkbenchConstants.TAG_ID);
            }
        }
        memento = null;
    }

    void saveState(IMemento mem) {
        if (editorInput !is null) {
            IPersistableElement persistable = editorInput.getPersistable();
            mem.putString(IWorkbenchConstants.TAG_ID, editorID);
            mem.putString(IWorkbenchConstants.TAG_FACTORY_ID, persistable
                    .getFactoryId());
            persistable.saveState(mem);
        } else if (memento !is null) {
            mem.putMemento(memento);
        }
    }
}
