/*******************************************************************************
 * Copyright (c) 2000, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.OpenPreferencesAction;

import dwt.dwthelper.utils;

import dwtx.jface.action.Action;
import dwtx.jface.preference.PreferenceDialog;
import dwtx.ui.IWorkbenchWindow;
import dwtx.ui.PlatformUI;
import dwtx.ui.actions.ActionFactory;
import dwtx.ui.dialogs.PreferencesUtil;

/**
 * Open the preferences dialog
 */
public class OpenPreferencesAction extends Action implements ActionFactory.IWorkbenchAction {

    /**
     * The workbench window; or <code>null</code> if this
     * action has been <code>dispose</code>d.
     */
    private IWorkbenchWindow workbenchWindow;

    /**
     * Create a new <code>OpenPreferenceAction</code>
     * This default constructor allows the the action to be called from the welcome page.
     */
    public OpenPreferencesAction() {
        this(PlatformUI.getWorkbench().getActiveWorkbenchWindow());
    }

    /**
     * Create a new <code>OpenPreferenceAction</code> and initialize it 
     * from the given resource bundle.
     * @param window
     */
    public OpenPreferencesAction(IWorkbenchWindow window) {
        super(WorkbenchMessages.OpenPreferences_text); 
        if (window is null) {
            throw new IllegalArgumentException();
        }
        this.workbenchWindow = window;
        // @issue action id not set
        setToolTipText(WorkbenchMessages.OpenPreferences_toolTip); 
        window.getWorkbench().getHelpSystem().setHelp(this,
                IWorkbenchHelpContextIds.OPEN_PREFERENCES_ACTION);
    }

    /* (non-Javadoc)
     * Method declared on Action.
     */
    public void run() {
        if (workbenchWindow is null) {
            // action has been dispose
            return;
        }
        PreferenceDialog dialog = PreferencesUtil.createPreferenceDialogOn(null, null, null, null);
        dialog.open();
    }

    /* (non-Javadoc)
     * Method declared on ActionFactory.IWorkbenchAction.
     */
    public void dispose() {
        workbenchWindow = null;
    }

}
