/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 ******************************************************************************/

module dwtx.ui.internal.ShowViewMenuHandler;

import dwt.dwthelper.utils;

import dwtx.core.commands.ExecutionEvent;
import dwtx.core.commands.ExecutionException;
import dwtx.core.expressions.EvaluationResult;
import dwtx.core.expressions.Expression;
import dwtx.core.expressions.ExpressionInfo;
import dwtx.core.expressions.IEvaluationContext;
import dwtx.core.runtime.CoreException;
import dwtx.ui.ISources;
import dwtx.ui.IWorkbenchPart;
import dwtx.ui.IWorkbenchPartSite;
import dwtx.ui.handlers.HandlerUtil;

/**
 * Shows the View Menu
 * <p>
 * Replacement for: ShowViewMenuAction
 * </p>
 * 
 * @since 3.3
 * 
 */
public class ShowViewMenuHandler extends AbstractEvaluationHandler {

    private Expression enabledWhen;

    public ShowViewMenuHandler() {
        registerEnablement();
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.core.commands.AbstractHandler#execute(dwtx.core.commands.ExecutionEvent)
     */
    public Object execute(ExecutionEvent event) throws ExecutionException {

        IWorkbenchPart part = HandlerUtil.getActivePart(event);
        if (part !is null) {
            IWorkbenchPartSite site = part.getSite();
            if (site instanceof PartSite) {
                PartPane pane = ((PartSite) site).getPane();
                pane.showPaneMenu();
            }
        }
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.internal.AbstractEvaluationHandler#getEnabledWhenExpression()
     */
    protected Expression getEnabledWhenExpression() {
        // TODO Auto-generated method stub
        if (enabledWhen is null) {
            enabledWhen = new Expression() {
                public EvaluationResult evaluate(IEvaluationContext context)
                        throws CoreException {
                    IWorkbenchPart part = InternalHandlerUtil
                            .getActivePart(context);
                    if (part !is null) {
                        PartPane pane = ((PartSite) part.getSite()).getPane();
                        if ((pane instanceof ViewPane)
                                && ((ViewPane) pane).hasViewMenu()) {
                            return EvaluationResult.TRUE;
                        }
                    }
                    return EvaluationResult.FALSE;
                }

                /*
                 * (non-Javadoc)
                 * 
                 * @see dwtx.core.expressions.Expression#collectExpressionInfo(dwtx.core.expressions.ExpressionInfo)
                 */
                public void collectExpressionInfo(ExpressionInfo info) {
                    info.addVariableNameAccess(ISources.ACTIVE_PART_NAME);
                }
            };
        }
        return enabledWhen;
    }

}
