/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 ******************************************************************************/

module dwtx.ui.internal.StickyViewManager;

import dwt.dwthelper.utils;

import java.util.Set;

import dwtx.core.runtime.IStatus;
import dwtx.core.runtime.Status;
import dwtx.jface.preference.IPreferenceStore;
import dwtx.ui.IMemento;
import dwtx.ui.IWorkbenchPage;
import dwtx.ui.IWorkbenchPreferenceConstants;
import dwtx.ui.PartInitException;
import dwtx.ui.internal.util.PrefUtil;
import dwtx.ui.views.IStickyViewDescriptor;
import dwtx.ui.views.IViewRegistry;

/**
 * @since 3.3
 * 
 */
public class StickyViewManager implements IStickyViewManager {
    
    private IWorkbenchPage page;
    
    public StickyViewManager(IWorkbenchPage page) {
        this.page = page;
    }

    
    public static IStickyViewManager getInstance(IWorkbenchPage page) {
        IStickyViewManager stickyViewMan;
        IPreferenceStore preferenceStore = PrefUtil.getAPIPreferenceStore();
        bool enable32Behavior = preferenceStore
                .getBoolean(IWorkbenchPreferenceConstants.ENABLE_32_STICKY_CLOSE_BEHAVIOR);
        if (enable32Behavior)
            stickyViewMan = new StickyViewManager32(page);
        else 
            stickyViewMan = new StickyViewManager(page);
        
        return stickyViewMan;
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.internal.IStickyViewManager#add(java.lang.String,
     *      java.util.Set)
     */
    public void add(String perspectiveId, Set stickyViewSet) {
        // do nothing
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.internal.IStickyViewManager#clear()
     */
    public void clear() {
        // do nothing
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.internal.IStickyViewManager#remove(java.lang.String)
     */
    public void remove(String perspectiveId) {
        // do nothing
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.internal.IStickyViewManager#restore(dwtx.ui.IMemento)
     */
    public void restore(IMemento memento) {
        // do nothing
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.internal.IStickyViewManager#save(dwtx.ui.IMemento)
     */
    public void save(IMemento memento) {
        // do nothing
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.internal.IStickyViewManager#update(dwtx.ui.internal.Perspective,
     *      dwtx.ui.internal.Perspective)
     */
    public void update(Perspective oldPersp, Perspective newPersp) {
        if (oldPersp is null || newPersp is null) {
            return;
        }
        IViewRegistry viewReg = WorkbenchPlugin.getDefault().getViewRegistry();
        IStickyViewDescriptor[] stickyDescs = viewReg.getStickyViews();
        for (int i = 0; i < stickyDescs.length; i++) {
            final String viewId = stickyDescs[i].getId();
            try {
                // show a sticky view if it was in the last perspective and
                // hasn't already been activated in this one
                if (oldPersp.findView(viewId) !is null) {
                    page.showView(viewId, null, IWorkbenchPage.VIEW_CREATE);
                }
                // remove a view if it's sticky and its not visible in the old
                // perspective
                else if (newPersp.findView(viewId) !is null
                        && oldPersp.findView(viewId) is null) {
                    page.hideView(newPersp.findView(viewId));
                }
            } catch (PartInitException e) {
                WorkbenchPlugin
                        .log(
                                "Could not open view :" + viewId, new Status(IStatus.ERROR, WorkbenchPlugin.PI_WORKBENCH, IStatus.ERROR, "Could not open view :" + viewId, e)); //$NON-NLS-1$ //$NON-NLS-2$
            }
        }
    }

}
