/*******************************************************************************
 * Copyright (c) 2000, 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.ToggleEditorsVisibilityAction;

import dwt.dwthelper.utils;

import dwtx.ui.IPerspectiveDescriptor;
import dwtx.ui.IPerspectiveListener;
import dwtx.ui.IWorkbenchPage;
import dwtx.ui.IWorkbenchWindow;

/**
 * Hides or shows the editor area within the current
 * perspective of the workbench page.
 */
public class ToggleEditorsVisibilityAction extends PerspectiveAction implements
        IPerspectiveListener {

    /* (non-Javadoc)
     * Method declared on IPerspectiveListener
     */
    public void perspectiveActivated(IWorkbenchPage page,
            IPerspectiveDescriptor perspective) {
        if (page.isEditorAreaVisible()) {
            setText(WorkbenchMessages.ToggleEditor_hideEditors); 
        } else {
            setText(WorkbenchMessages.ToggleEditor_showEditors);
        }
    }

    /* (non-Javadoc)
     * Method declared on IPerspectiveListener
     */
    public void perspectiveChanged(IWorkbenchPage page,
            IPerspectiveDescriptor perspective, String changeId) {
        if (changeId is IWorkbenchPage.CHANGE_RESET
                || changeId is IWorkbenchPage.CHANGE_EDITOR_AREA_HIDE
                || changeId is IWorkbenchPage.CHANGE_EDITOR_AREA_SHOW) {
            if (page.isEditorAreaVisible()) {
                setText(WorkbenchMessages.ToggleEditor_hideEditors); 
            } else {
                setText(WorkbenchMessages.ToggleEditor_showEditors); 
            }
        }
    }

    /**
     * Creates a new <code>ToggleEditorsVisibilityAction</code>
     * 
     * @param window the window
     */
    public ToggleEditorsVisibilityAction(IWorkbenchWindow window) {
        super(window);
        setText(WorkbenchMessages.ToggleEditor_hideEditors);
        setActionDefinitionId("dwtx.ui.window.hideShowEditors"); //$NON-NLS-1$
        // @issue missing action id
        setToolTipText(WorkbenchMessages.ToggleEditor_toolTip);
        window.getWorkbench().getHelpSystem().setHelp(this,
                IWorkbenchHelpContextIds.TOGGLE_EDITORS_VISIBILITY_ACTION);
        window.addPerspectiveListener(this);
    }

    /* (non-Javadoc)
     * Method declared on PerspectiveAction.
     */
    protected void run(IWorkbenchPage page, IPerspectiveDescriptor persp) {
        bool visible = page.isEditorAreaVisible();
        if (visible) {
            page.setEditorAreaVisible(false);
            setText(WorkbenchMessages.ToggleEditor_showEditors); 
        } else {
            page.setEditorAreaVisible(true);
            setText(WorkbenchMessages.ToggleEditor_hideEditors);
        }
    }

    /* (non-Javadoc)
     * Method declared on ActionFactory.IWorkbenchAction.
     */
    public void dispose() {
        if (getWindow() !is null) {
            getWindow().removePerspectiveListener(this);
        }
        super.dispose();
    }

}
