/*******************************************************************************
 * Copyright (c) 2004, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.ViewIntroAdapterSite;

import dwt.dwthelper.utils;


import dwt.widgets.Shell;
import dwtx.jface.viewers.ISelectionProvider;
import dwtx.ui.IActionBars;
import dwtx.ui.IKeyBindingService;
import dwtx.ui.IViewSite;
import dwtx.ui.IWorkbenchPage;
import dwtx.ui.IWorkbenchWindow;
import dwtx.ui.internal.intro.IntroDescriptor;
import dwtx.ui.intro.IIntroSite;

/**
 * Simple <code>IIntroSite</code> that wraps a <code>IViewSite</code>.  For use in conjunction with 
 * <code>ViewIntroAdapterPart</code>.
 * 
 * @since 3.0
 */
final class ViewIntroAdapterSite implements IIntroSite {
    private IntroDescriptor descriptor;

    private IViewSite viewSite;

    public ViewIntroAdapterSite(IViewSite viewSite, IntroDescriptor descriptor) {
        this.viewSite = viewSite;
        this.descriptor = descriptor;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.intro.IIntroSite#getActionBars()
     */
    public IActionBars getActionBars() {
        return viewSite.getActionBars();
    }

    /* (non-Javadoc)
     * @see dwtx.core.runtime.IAdaptable#getAdapter(java.lang.Class)
     */
    public Object getAdapter(Class adapter) {
        return viewSite.getAdapter(adapter);
    }

    /* (non-Javadoc)
     * @see dwtx.ui.IWorkbenchPartSite#getId()
     */
    public String getId() {
        return descriptor.getId();
    }

    /* (non-Javadoc)
     * @see dwtx.ui.IWorkbenchPartSite#getKeyBindingService()
     */
    public IKeyBindingService getKeyBindingService() {
        return viewSite.getKeyBindingService();
    }

    /* (non-Javadoc)
     * @see dwtx.ui.IWorkbenchSite#getPage()
     */
    public IWorkbenchPage getPage() {
        return viewSite.getPage();
    }

    /* (non-Javadoc)
     * @see dwtx.ui.IWorkbenchPartSite#getPluginId()
     */
    public String getPluginId() {
        return descriptor.getPluginId();
    }

    /* (non-Javadoc)
     * @see dwtx.ui.IWorkbenchSite#getSelectionProvider()
     */
    public ISelectionProvider getSelectionProvider() {
        return viewSite.getSelectionProvider();
    }
    
    public final Object getService(final Class key) {
            return viewSite.getService(key);
    }

    /* (non-Javadoc)
     * @see dwtx.ui.IWorkbenchSite#getShell()
     */
    public Shell getShell() {
        return viewSite.getShell();
    }

    /* (non-Javadoc)
     * @see dwtx.ui.IWorkbenchSite#getWorkbenchWindow()
     */
    public IWorkbenchWindow getWorkbenchWindow() {
        return viewSite.getWorkbenchWindow();
    }

    public final bool hasService(final Class key) {
        return viewSite.hasService(key);
    }

    /* (non-Javadoc)
     * @see dwtx.ui.IWorkbenchSite#setSelectionProvider(dwtx.jface.viewers.ISelectionProvider)
     */
    public void setSelectionProvider(ISelectionProvider provider) {
        viewSite.setSelectionProvider(provider);
    }

    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return viewSite.toString();
    }
}
