/*******************************************************************************
 * Copyright (c) 2007, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.internal.WorkbenchSupportFactory;

import dwt.dwthelper.utils;

import dwtx.ui.IPageService;
import dwtx.ui.IPartService;
import dwtx.ui.ISelectionService;
import dwtx.ui.IWorkbench;
import dwtx.ui.IWorkbenchPartSite;
import dwtx.ui.IWorkbenchWindow;
import dwtx.ui.internal.services.IWorkbenchLocationService;
import dwtx.ui.progress.IProgressService;
import dwtx.ui.progress.IWorkbenchSiteProgressService;
import dwtx.ui.services.AbstractServiceFactory;
import dwtx.ui.services.IServiceLocator;

/**
 * Create singleton services to make the Workbench singletons available. This is
 * a "hack" to provide access to the Workbench singletons.
 * 
 * @since 3.4
 */
public class WorkbenchSupportFactory extends AbstractServiceFactory {

    public Object create(Class serviceInterface, IServiceLocator parentLocator,
            IServiceLocator locator) {

        IWorkbenchLocationService wls = (IWorkbenchLocationService) locator
                .getService(IWorkbenchLocationService.class);
        final IWorkbench wb = wls.getWorkbench();
        if (wb is null) {
            return null;
        }
        final IWorkbenchWindow window = wls.getWorkbenchWindow();
        final IWorkbenchPartSite site = wls.getPartSite();
        Object parent = parentLocator.getService(serviceInterface);

        if (parent is null) {
            // return top level services
            if (IProgressService.class.equals(serviceInterface)) {
                return wb.getProgressService();
            }
            if (IWorkbenchSiteProgressService.class.equals(serviceInterface)) {
                if (site instanceof PartSite) {
                    return ((PartSite) site).getSiteProgressService();
                }
            }
            if (IPartService.class.equals(serviceInterface)) {
                if (window !is null) {
                    return window.getPartService();
                }
            }
            if (IPageService.class.equals(serviceInterface)) {
                if (window !is null) {
                    return window;
                }
            }
            if (ISelectionService.class.equals(serviceInterface)) {
                if (window !is null) {
                    return window.getSelectionService();
                }
            }
            return null;
        }

        if (ISelectionService.class.equals(serviceInterface)) {
            if (parent instanceof WindowSelectionService && window !is null
                    && window.getActivePage() !is null) {
                return new SlaveSelectionService(window.getActivePage());
            }
            return new SlaveSelectionService((ISelectionService) parent);
        }

        if (IProgressService.class.equals(serviceInterface)) {
            if (site instanceof PartSite) {
                return ((PartSite) site).getSiteProgressService();
            }
        }
        if (IPartService.class.equals(serviceInterface)) {
            return new SlavePartService((IPartService) parent);
        }
        if (IPageService.class.equals(serviceInterface)) {
            return new SlavePageService((IPageService) parent);
        }

        return null;
    }
}
