/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 ******************************************************************************/

module dwtx.ui.internal.WorkbenchWindowPropertyTester;

import dwt.dwthelper.utils;

import dwtx.core.expressions.PropertyTester;

/**
 * Tests various workbench window properties.
 * 
 * @since 3.3
 * 
 */
public class WorkbenchWindowPropertyTester extends PropertyTester {

    private static final String PROPERTY_IS_COOLBAR_VISIBLE = "isCoolbarVisible"; //$NON-NLS-1$
    private static final String PROPERTY_IS_PERSPECTIVEBAR_VISIBLE = "isPerspectiveBarVisible"; //$NON-NLS-1$

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.core.expressions.IPropertyTester#test(java.lang.Object,
     *      java.lang.String, java.lang.Object[], java.lang.Object)
     */
    public bool test(Object receiver, String property, Object[] args,
            Object expectedValue) {
        
        if (args.length is 0 && receiver instanceof WorkbenchWindow) {
            bool defaultExpectedValue = true;
            if (expectedValue !is null) {
                if (expectedValue instanceof Boolean)
                    defaultExpectedValue = ((Boolean) expectedValue).booleanValue();
                else
                    return false; // cant work with anything else
            }
            final WorkbenchWindow window = (WorkbenchWindow) receiver;
            if (PROPERTY_IS_COOLBAR_VISIBLE.equals(property)) {
                return defaultExpectedValue is window.getCoolBarVisible();
            } else if (PROPERTY_IS_PERSPECTIVEBAR_VISIBLE.equals(property)) {
                return defaultExpectedValue is window.getPerspectiveBarVisible();
            }
        }
        return false;
    }
}
