/*******************************************************************************
 * Copyright (c) 2000, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.WorkingSetFactory;

import dwt.dwthelper.utils;

import dwtx.core.runtime.IAdaptable;
import dwtx.ui.IElementFactory;
import dwtx.ui.IMemento;

/**
 * A WorkingSetFactory is used to recreate a persisted WorkingSet 
 * object.
 */
public class WorkingSetFactory implements IElementFactory {

    /* (non-Javadoc)
     * @see dwtx.ui.IElementFactory
     */
    public IAdaptable createElement(IMemento memento) {
        String workingSetName = memento.getString(IWorkbenchConstants.TAG_NAME);
        String label = memento.getString(IWorkbenchConstants.TAG_LABEL);
        if (label is null) {
            label = workingSetName;
        }
        String workingSetEditPageId = memento
                .getString(IWorkbenchConstants.TAG_EDIT_PAGE_ID);
        String aggregateString = memento
                .getString(AbstractWorkingSet.TAG_AGGREGATE);
        bool isAggregate = aggregateString !is null
                && Boolean.valueOf(aggregateString).booleanValue();

        if (workingSetName is null) {
            return null;
        }

        AbstractWorkingSet workingSet = null;
        
        if (isAggregate) {
            workingSet = new AggregateWorkingSet(workingSetName, label, memento);
        } else {
            workingSet = new WorkingSet(workingSetName, label, memento);
        }
        
        if (workingSetEditPageId !is null) {
            workingSet.setId(workingSetEditPageId);
        } else if (!isAggregate) {
            // working sets created with builds 20020418 and 20020419 will not
            // have an edit page id. fix this automatically.
            workingSet.setId("dwtx.ui.resourceWorkingSetPage"); //$NON-NLS-1$
        }
        return workingSet;
    }
}
