/*******************************************************************************
 * Copyright (c) 2000, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.about.AboutAction;

import dwt.dwthelper.utils;

import dwtx.core.runtime.IProduct;
import dwtx.core.runtime.Platform;
import dwtx.jface.action.Action;
import dwtx.osgi.util.NLS;
import dwtx.ui.IWorkbenchWindow;
import dwtx.ui.actions.ActionFactory;
import dwtx.ui.internal.IWorkbenchHelpContextIds;
import dwtx.ui.internal.WorkbenchMessages;
import dwtx.ui.internal.dialogs.AboutDialog;

/**
 * Creates an About dialog and opens it.
 */
public class AboutAction extends Action implements
        ActionFactory.IWorkbenchAction {

    /**
     * The workbench window; or <code>null</code> if this action has been
     * <code>dispose</code>d.
     */
    private IWorkbenchWindow workbenchWindow;

    /**
     * Creates a new <code>AboutAction</code>.
     * 
     * @param window the window
     */
    public AboutAction(IWorkbenchWindow window) {
        if (window is null) {
            throw new IllegalArgumentException();
        }

        this.workbenchWindow = window;

        // use message with no fill-in
        IProduct product = Platform.getProduct();
        String productName = null;
        if (product !is null) {
            productName = product.getName();
        }
        if (productName is null) {
            productName = ""; //$NON-NLS-1$
        }
        setText(NLS.bind(WorkbenchMessages.AboutAction_text,productName)); 
        setToolTipText(NLS.bind(WorkbenchMessages.AboutAction_toolTip, productName));
        setId("about"); //$NON-NLS-1$
        setActionDefinitionId("dwtx.ui.help.aboutAction"); //$NON-NLS-1$
        window.getWorkbench().getHelpSystem().setHelp(this,
                IWorkbenchHelpContextIds.ABOUT_ACTION);
    }

    /*
     * (non-Javadoc) Method declared on IAction.
     */
    public void run() {
        // make sure action is not disposed
        if (workbenchWindow !is null) {
            new AboutDialog(workbenchWindow.getShell()).open();
        }
    }

    /*
     * (non-Javadoc) Method declared on ActionFactory.IWorkbenchAction.
     */
    public void dispose() {
        workbenchWindow = null;
    }
}
