/*******************************************************************************
 * Copyright (c) 2004, 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.actions.HelpSearchAction;

import dwt.dwthelper.utils;


import dwt.custom.BusyIndicator;
import dwtx.jface.action.Action;
import dwtx.ui.IWorkbenchPreferenceConstants;
import dwtx.ui.IWorkbenchWindow;
import dwtx.ui.PlatformUI;
import dwtx.ui.actions.ActionFactory.IWorkbenchAction;
import dwtx.ui.internal.IWorkbenchGraphicConstants;
import dwtx.ui.internal.IWorkbenchHelpContextIds;
import dwtx.ui.internal.WorkbenchImages;
import dwtx.ui.internal.WorkbenchMessages;
import dwtx.ui.internal.util.PrefUtil;

/**
 * Action to open the help search.
 * 
 * @since 3.1
 */
public class HelpSearchAction extends Action implements IWorkbenchAction {
    /**
     * The workbench window; or <code>null</code> if this
     * action has been <code>dispose</code>d.
     */
    private IWorkbenchWindow workbenchWindow;

    /**
     * Zero-arg constructor to allow cheat sheets to reuse this action.
     */
    public HelpSearchAction() {
        this(PlatformUI.getWorkbench().getActiveWorkbenchWindow());
    }

    /**
     * Constructor for use by ActionFactory.
     * 
     * @param window the window
     */
    public HelpSearchAction(IWorkbenchWindow window) {
        if (window is null) {
            throw new IllegalArgumentException();
        }
        this.workbenchWindow = window;
        setActionDefinitionId("dwtx.ui.help.helpSearch"); //$NON-NLS-1$

        // support for allowing a product to override the text for the action
        String overrideText = PrefUtil.getAPIPreferenceStore().getString(
                IWorkbenchPreferenceConstants.HELP_SEARCH_ACTION_TEXT);
        if ("".equals(overrideText)) { //$NON-NLS-1$
            setText(WorkbenchMessages.HelpSearchAction_text); 
            setToolTipText(WorkbenchMessages.HelpSearchAction_toolTip);
        } else {
            setText(overrideText);
            setToolTipText(Action.removeMnemonics(overrideText));
        }
        setImageDescriptor(WorkbenchImages
                .getImageDescriptor(IWorkbenchGraphicConstants.IMG_ETOOL_HELP_SEARCH));
        window.getWorkbench().getHelpSystem().setHelp(this,
                IWorkbenchHelpContextIds.HELP_SEARCH_ACTION);
    }

    /* (non-Javadoc)
     * Method declared on IAction.
     */
    public void run() {
        if (workbenchWindow is null) {
            // action has been disposed
            return;
        }
        //This may take a while, so use the busy indicator
        BusyIndicator.showWhile(null, new Runnable() {
            public void run() {
                workbenchWindow.getWorkbench().getHelpSystem().displaySearch();
            }
        });
    }

    /* (non-Javadoc)
     * Method declared on ActionFactory.IWorkbenchAction.
     */
    public void dispose() {
        workbenchWindow = null;
    }

}
