/*******************************************************************************
 * Copyright (c) 2005, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.actions.HelpSearchContributionItem;

import dwt.dwthelper.utils;


import dwt.DWT;
import dwt.events.KeyAdapter;
import dwt.events.KeyEvent;
import dwt.events.SelectionAdapter;
import dwt.events.SelectionEvent;
import dwt.widgets.Combo;
import dwt.widgets.Composite;
import dwt.widgets.Control;
import dwtx.core.runtime.Assert;
import dwtx.jface.action.ControlContribution;
import dwtx.ui.IWorkbenchWindow;
import dwtx.ui.internal.WorkbenchMessages;
import dwtx.ui.internal.WorkbenchPlugin;

/**
 * This is the contribution item that is used to add a help search field to
 * the cool bar.
 * 
 * @since 3.1
 */
public class HelpSearchContributionItem extends ControlContribution {
    private static final String ID = "dwtx.ui.helpSearch"; //$NON-NLS-1$
    
    private IWorkbenchWindow window;

    private Combo combo;

    private int MAX_ITEM_COUNT = 10;
    
    /**
     * Creates the contribution item.
     * 
     * @param window the window
     */
    public HelpSearchContributionItem(IWorkbenchWindow window) {
        this(window, ID);
    }

    /**
     * Creates the contribution item.
     * 
     * @param window the window
     * @param id the contribution item id
     */
    public HelpSearchContributionItem(IWorkbenchWindow window, String id) {
        super(id);
        Assert.isNotNull(window);
        this.window = window;
    }

    /* (non-Javadoc)
     * @see dwtx.jface.action.ControlContribution#createControl(dwt.widgets.Composite)
     */
    protected Control createControl(Composite parent) {
        combo = new Combo(parent, DWT.NONE);
        combo.setToolTipText(WorkbenchMessages.WorkbenchWindow_searchCombo_toolTip); 
        String[] items = WorkbenchPlugin.getDefault().getDialogSettings()
                .getArray(ID);
        if (items !is null) {
            combo.setItems(items);
        }
        combo.setText(WorkbenchMessages.WorkbenchWindow_searchCombo_text); 
        combo.addKeyListener(new KeyAdapter() {
            public void keyReleased(KeyEvent e) {
                if (e.character is DWT.CR) {
                    doSearch(combo.getText(), true);
                }
            }
        });
        combo.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                int index = combo.getSelectionIndex();
                if (index !is -1) {
                    doSearch(combo.getItem(index), false);
                }
            }
        });
        return combo;
    }

    /* (non-Javadoc)
     * @see dwtx.jface.action.ControlContribution#computeWidth(dwt.widgets.Control)
     */
    protected int computeWidth(Control control) {
        return control.computeSize(DWT.DEFAULT, DWT.DEFAULT, true).x;
    }

    private void doSearch(String phrase, bool updateList) {
        if (phrase.length() is 0) {
            window.getWorkbench().getHelpSystem().displaySearch();
            return;
        }
        if (updateList) {
            bool exists = false;
            for (int i = 0; i < combo.getItemCount(); i++) {
                String item = combo.getItem(i);
                if (item.equalsIgnoreCase(phrase)) {
                    exists = true;
                    break;
                }
            }
            if (!exists) {
                combo.add(phrase, 0);
                if (combo.getItemCount() > MAX_ITEM_COUNT) {
                    combo.remove(combo.getItemCount() - 1);
                }
                WorkbenchPlugin.getDefault().getDialogSettings().put(ID,
                        combo.getItems());
            }
        }
        window.getWorkbench().getHelpSystem().search(phrase);
    }
}
