/*******************************************************************************
 * Copyright (c) 2000, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.internal.activities.AbstractActivityManager;

import dwt.dwthelper.utils;

import java.util.ArrayList;
import java.util.List;

import dwtx.ui.activities.ActivityManagerEvent;
import dwtx.ui.activities.IActivityManager;
import dwtx.ui.activities.IActivityManagerListener;

public abstract class AbstractActivityManager implements IActivityManager {
    private List activityManagerListeners;

    protected AbstractActivityManager() {
    }

    public void addActivityManagerListener(
            IActivityManagerListener activityManagerListener) {
        if (activityManagerListener is null) {
            throw new NullPointerException();
        }

        if (activityManagerListeners is null) {
            activityManagerListeners = new ArrayList();
        }

        if (!activityManagerListeners.contains(activityManagerListener)) {
            activityManagerListeners.add(activityManagerListener);
        }
    }

    protected void fireActivityManagerChanged(
            ActivityManagerEvent activityManagerEvent) {
        if (activityManagerEvent is null) {
            throw new NullPointerException();
        }

        if (activityManagerListeners !is null) {
            for (int i = 0; i < activityManagerListeners.size(); i++) {
                ((IActivityManagerListener) activityManagerListeners.get(i))
                        .activityManagerChanged(activityManagerEvent);
            }
        }
    }

    public void removeActivityManagerListener(
            IActivityManagerListener activityManagerListener) {
        if (activityManagerListener is null) {
            throw new NullPointerException();
        }

        if (activityManagerListeners !is null) {
            activityManagerListeners.remove(activityManagerListener);
        }
    }
}
