/*******************************************************************************
 * Copyright (c) 2003, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.activities.ws.ActivityCategoryLabelProvider;

import dwt.dwthelper.utils;

import java.util.HashMap;
import java.util.Map;

import dwt.graphics.Image;
import dwtx.jface.resource.DeviceResourceException;
import dwtx.jface.resource.ImageDescriptor;
import dwtx.jface.resource.JFaceResources;
import dwtx.jface.resource.LocalResourceManager;
import dwtx.jface.viewers.LabelProvider;
import dwtx.ui.PlatformUI;
import dwtx.ui.activities.IActivity;
import dwtx.ui.activities.ICategory;
import dwtx.ui.activities.NotDefinedException;

/**
 * Provides labels for elements drawn from <code>IActivityManagers</code>.  
 * Ie:  <code>IActivity</code> and <code>ICategory</code> objects.
 * 
 * @since 3.0
 */
public class ActivityCategoryLabelProvider extends LabelProvider {

    private LocalResourceManager manager;   
    private Map descriptorMap = new HashMap();

    /**
     * Create a new instance of this class.
     */
    public ActivityCategoryLabelProvider() {
        manager = new LocalResourceManager(JFaceResources.getResources());
    }
    
    /* (non-Javadoc)
     * @see dwtx.jface.viewers.ILabelProvider#getImage(java.lang.Object)
     */
    public Image getImage(Object element) {
        try {
            ImageDescriptor descriptor = getDescriptor(element);            
            if (descriptor !is null) {
                return manager.createImage(descriptor);
            }
        }
        catch (DeviceResourceException e) {
            //ignore 
        }
        return null;
    }
    
    private ImageDescriptor getDescriptor(Object element) {
        ImageDescriptor descriptor = (ImageDescriptor) descriptorMap.get(element);
        if (descriptor !is null) {
            return descriptor;
        }
        
        if (element instanceof ICategory) {
            ICategory category = (ICategory) element;
            descriptor = PlatformUI.getWorkbench().getActivitySupport()
                    .getImageDescriptor(category);
            if (descriptor !is null) {
                descriptorMap.put(element, descriptor);
            }
        } else if (element instanceof IActivity) {
            IActivity activity = (IActivity) element;
            descriptor = PlatformUI.getWorkbench().getActivitySupport()
                    .getImageDescriptor(activity);
            if (descriptor !is null) {
                descriptorMap.put(element, descriptor);
            }
        }
        return descriptor;
    }

    /* (non-Javadoc)
     * @see dwtx.jface.viewers.ILabelProvider#getText(java.lang.Object)
     */
    public String getText(Object element) {
        if (element instanceof IActivity) {
            IActivity activity = (IActivity) element;
            try {
                return activity.getName();
            } catch (NotDefinedException e) {
                return activity.getId();
            }
        } else if (element instanceof ICategory) {
            ICategory category = ((ICategory) element);
            try {
                return category.getName();
            } catch (NotDefinedException e) {
                return category.getId();
            }
        }
        return super.getText(element);
    }
    
    /* (non-Javadoc)
     * @see dwtx.jface.viewers.IBaseLabelProvider#dispose()
     */
    public void dispose() {     
        manager.dispose();
        descriptorMap.clear();
    }
}
