/*******************************************************************************
 * Copyright (c) 2005, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.activities.ws.RegistryTriggerPoint;

import dwt.dwthelper.utils;

import java.util.HashMap;
import java.util.Map;

import dwtx.core.runtime.IConfigurationElement;
import dwtx.ui.internal.registry.IWorkbenchRegistryConstants;

/**
 * @since 3.1
 */
public class RegistryTriggerPoint extends AbstractTriggerPoint {

    private String id;

    private IConfigurationElement element;

    private Map hints;

    /**
     * Create a new instance of this class.
     * 
     * @param id the id of the trigger point
     * @param element the defining configuration element
     */
    public RegistryTriggerPoint(String id, IConfigurationElement element) {
        this.id = id;
        this.element = element;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.activities.ITriggerPoint#getId()
     */
    public String getId() {
        return id;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.activities.ITriggerPoint#getStringHint(java.lang.String)
     */
    public String getStringHint(String key) {
        return (String) getHints().get(key);
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.activities.ITriggerPoint#getBooleanHint(java.lang.String)
     */
    public bool getBooleanHint(String key) {
        return Boolean.valueOf(getStringHint(key)).booleanValue();
    }

    /**
     * Lazily create the hints.
     * 
     * @return the hint map
     */
    private Map getHints() {
        if (hints is null) {
            hints = new HashMap();

            IConfigurationElement[] hintElements = element
                    .getChildren(IWorkbenchRegistryConstants.TAG_HINT);
            for (int i = 0; i < hintElements.length; i++) {
                String id = hintElements[i]
                        .getAttribute(IWorkbenchRegistryConstants.ATT_ID);
                String value = hintElements[i]
                        .getAttribute(IWorkbenchRegistryConstants.ATT_VALUE);

                if (id !is null && value !is null) {
                    hints.put(id, value);
                }
            }
        }

        return hints;
    }
}
