/*******************************************************************************
 * Copyright (c) 2004, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.activities.ws.TriggerPointAdvisorDescriptor;

import dwt.dwthelper.utils;

import dwtx.core.runtime.CoreException;
import dwtx.core.runtime.IConfigurationElement;
import dwtx.ui.activities.ITriggerPointAdvisor;
import dwtx.ui.internal.registry.IWorkbenchRegistryConstants;
import dwtx.ui.internal.registry.RegistryReader;

/**
 * Descriptor for trigger point advisor extensions.
 * 
 * @since 3.1
 */
public final class TriggerPointAdvisorDescriptor {

    private String id;

    private IConfigurationElement element;

    /**
     * Create a new instance of this class.
     * 
     * @param element
     *            the configuration element
     * @throws IllegalArgumentException
     *             thrown if the element is missing an id attribute
     */
    public TriggerPointAdvisorDescriptor(IConfigurationElement element)
            throws IllegalArgumentException {
        id = element.getAttribute(IWorkbenchRegistryConstants.ATT_ID);
        if (id is null
                || RegistryReader.getClassValue(element,
                        IWorkbenchRegistryConstants.ATT_CLASS) is null) {
            throw new IllegalArgumentException();
        }
        this.element = element;
    }

    /**
     * Return the id.
     * 
     * @return the id
     */
    public String getId() {
        return id;
    }

    /**
     * Create the advisor for this descriptor.
     * 
     * @return the advisor
     * @throws CoreException
     *             thrown if there is an issue creating the advisor
     */
    public ITriggerPointAdvisor createAdvisor() throws CoreException {
        return (ITriggerPointAdvisor) element
                .createExecutableExtension(IWorkbenchRegistryConstants.ATT_CLASS);
    }
}
