/*******************************************************************************
 * Copyright (c) 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 ******************************************************************************/

module dwtx.ui.internal.commands.CommandServiceFactory;

import dwt.dwthelper.utils;

import dwtx.ui.IWorkbench;
import dwtx.ui.IWorkbenchPartSite;
import dwtx.ui.IWorkbenchWindow;
import dwtx.ui.commands.ICommandService;
import dwtx.ui.internal.services.IWorkbenchLocationService;
import dwtx.ui.services.AbstractServiceFactory;
import dwtx.ui.services.IServiceLocator;
import dwtx.ui.services.IServiceScopes;

/**
 * @since 3.4
 * 
 */
public class CommandServiceFactory extends AbstractServiceFactory {

    /*
     * (non-Javadoc)
     * 
     * @see
     * dwtx.ui.services.AbstractServiceFactory#create(java.lang.Class,
     * dwtx.ui.services.IServiceLocator,
     * dwtx.ui.services.IServiceLocator)
     */
    public Object create(Class serviceInterface, IServiceLocator parentLocator,
            IServiceLocator locator) {
        if (!ICommandService.class.equals(serviceInterface)) {
            return null;
        }
        IWorkbenchLocationService wls = (IWorkbenchLocationService) locator
                .getService(IWorkbenchLocationService.class);
        final IWorkbench wb = wls.getWorkbench();
        if (wb is null) {
            return null;
        }

        Object parent = parentLocator.getService(serviceInterface);
        if (parent is null) {
            // we are registering the global services in the Workbench
            return null;
        }
        final IWorkbenchWindow window = wls.getWorkbenchWindow();
        final IWorkbenchPartSite site = wls.getPartSite();

        if (site is null) {
            return new SlaveCommandService((ICommandService) parent,
                    IServiceScopes.WINDOW_SCOPE, window);
        }

        if (parent instanceof SlaveCommandService) {
            Object pageSite = wls.getPageSite();
            if (pageSite !is null) {
                return new SlaveCommandService((ICommandService) parent,
                        IServiceScopes.PAGESITE_SCOPE, pageSite);
            }
            Object mpepSite = wls.getMultiPageEditorSite();
            if (mpepSite !is null) {
                return new SlaveCommandService((ICommandService) parent,
                        IServiceScopes.MPESITE_SCOPE, mpepSite);
            }
        }

        return new SlaveCommandService((ICommandService) parent,
                IServiceScopes.PARTSITE_SCOPE, site);
    }

}
