/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 ******************************************************************************/

module dwtx.ui.internal.commands.ElementReference;

import dwt.dwthelper.utils;

import java.util.HashMap;
import java.util.Map;

import dwtx.ui.commands.IElementReference;
import dwtx.ui.menus.UIElement;

/**
 * Our element reference that is used during element
 * registration/unregistration.
 * 
 * @since 3.3
 */
public class ElementReference implements IElementReference {

    private String commandId;
    private UIElement element;
    private HashMap parameters;

    /**
     * Construct the reference.
     * 
     * @param id
     *            command id. Must not be <code>null</code>.
     * @param adapt
     *            the element. Must not be <code>null</code>.
     * @param parms.
     *            parameters used for filtering. Must not be <code>null</code>.
     */
    public ElementReference(String id, UIElement adapt, Map parms) {
        commandId = id;
        element = adapt;
        if (parms is null) {
            parameters = new HashMap();
        } else {
            parameters = new HashMap(parms);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.commands.IElementReference#getElement()
     */
    public UIElement getElement() {
        return element;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.commands.IElementReference#getCommandId()
     */
    public String getCommandId() {
        return commandId;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.commands.IElementReference#getParameters()
     */
    public Map getParameters() {
        return parameters;
    }
}
