/*******************************************************************************
 * Copyright (c) 2004, 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.commands.LegacyCommandListenerWrapper;

import dwt.dwthelper.utils;

import dwtx.core.commands.CommandEvent;
import dwtx.core.commands.ICommandListener;
import dwtx.jface.bindings.BindingManager;
import dwtx.ui.commands.ICommand;

/**
 * Wraps a legacy listener in a new listener interface. This simply forwards
 * incoming events through to the old interface.
 * 
 * @since 3.1
 */
final class LegacyCommandListenerWrapper implements ICommandListener {

    /**
     * The supporting binding manager; never <code>null</code>.
     */
    private final BindingManager bindingManager;

    /**
     * The listener which is being wrapped. This value should never be
     * <code>null</code>.
     */
    private final dwtx.ui.commands.ICommandListener listener;

    /**
     * Constructs a new instance of <code>CommandListenerWrapper</code> around
     * a legacy listener.
     * 
     * @param listener
     *            The listener to be wrapped; must not be <code>null</code>.
     */
    LegacyCommandListenerWrapper(
            final dwtx.ui.commands.ICommandListener listener,
            final BindingManager bindingManager) {
        if (listener is null) {
            throw new NullPointerException("Cannot wrap a null listener."); //$NON-NLS-1$
        }

        if (bindingManager is null) {
            throw new NullPointerException(
                    "Cannot create a listener wrapper without a binding manager"); //$NON-NLS-1$
        }

        this.listener = listener;
        this.bindingManager = bindingManager;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.commands.ICommandListener#commandChanged(dwtx.commands.CommandEvent)
     */
    public final void commandChanged(final CommandEvent commandEvent) {
        final ICommand command = new CommandLegacyWrapper(commandEvent.getCommand(),
                bindingManager);
        final bool definedChanged = commandEvent.isDefinedChanged();
        final bool descriptionChanged = commandEvent.isDescriptionChanged();
        final bool handledChanged = commandEvent.isHandledChanged();
        final bool nameChanged = commandEvent.isNameChanged();

        listener.commandChanged(new dwtx.ui.commands.CommandEvent(
                command, false, false, definedChanged, descriptionChanged,
                handledChanged, false, nameChanged, null));

    }

    public final bool equals(final Object object) {
        if (object instanceof LegacyCommandListenerWrapper) {
            final LegacyCommandListenerWrapper wrapper = (LegacyCommandListenerWrapper) object;
            return listener.equals(wrapper.listener);
        }

        if (object instanceof dwtx.ui.commands.ICommandListener) {
            final dwtx.ui.commands.ICommandListener other = (dwtx.ui.commands.ICommandListener) object;
            return listener.equals(other);
        }

        return false;
    }

    public final int hashCode() {
        return listener.hashCode();
    }
}
