/*******************************************************************************
 * Copyright (c) 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 ******************************************************************************/

module dwtx.ui.internal.contexts.ContextServiceFactory;

import dwt.dwthelper.utils;

import dwtx.core.expressions.Expression;
import dwtx.ui.IWorkbench;
import dwtx.ui.IWorkbenchPartSite;
import dwtx.ui.IWorkbenchWindow;
import dwtx.ui.contexts.IContextService;
import dwtx.ui.internal.expressions.ActivePartExpression;
import dwtx.ui.internal.expressions.WorkbenchWindowExpression;
import dwtx.ui.internal.services.IWorkbenchLocationService;
import dwtx.ui.services.AbstractServiceFactory;
import dwtx.ui.services.IServiceLocator;

/**
 * @since 3.4
 * 
 */
public class ContextServiceFactory extends AbstractServiceFactory {

    /*
     * (non-Javadoc)
     * 
     * @see
     * dwtx.ui.services.AbstractServiceFactory#create(java.lang.Class,
     * dwtx.ui.services.IServiceLocator,
     * dwtx.ui.services.IServiceLocator)
     */
    public Object create(Class serviceInterface, IServiceLocator parentLocator,
            IServiceLocator locator) {
        if (!IContextService.class.equals(serviceInterface)) {
            return null;
        }
        IWorkbenchLocationService wls = (IWorkbenchLocationService) locator
                .getService(IWorkbenchLocationService.class);
        final IWorkbench wb = wls.getWorkbench();
        if (wb is null) {
            return null;
        }

        Object parent = parentLocator.getService(serviceInterface);
        if (parent is null) {
            // we are registering the global services in the Workbench
            return null;
        }
        final IWorkbenchWindow window = wls.getWorkbenchWindow();
        final IWorkbenchPartSite site = wls.getPartSite();
        if (site is null) {
            Expression exp = new WorkbenchWindowExpression(window);
            return new SlaveContextService((IContextService) parent, exp);
        }
        if (parent instanceof SlaveContextService) {
            Expression exp = ((SlaveContextService) parent).fDefaultExpression;
            if (exp instanceof ActivePartExpression) {
                return new NestableContextService((IContextService) parent, exp);
            }
        }
        Expression exp = new ActivePartExpression(site.getPart());
        return new SlaveContextService((IContextService) parent, exp);
    }

}
