/*******************************************************************************
 * Copyright (c) 2000, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.decorators.DeclarativeDecorator;

import dwt.dwthelper.utils;

import java.net.URL;

import dwtx.core.runtime.IConfigurationElement;
import dwtx.jface.resource.ImageDescriptor;
import dwtx.jface.viewers.IDecoration;
import dwtx.jface.viewers.ILabelProviderListener;
import dwtx.jface.viewers.ILightweightLabelDecorator;
import dwtx.ui.internal.util.BundleUtility;

/**
 * The DeclarativeDecorator is a decorator that is made entirely from an XML
 * specification.
 */
public class DeclarativeDecorator implements ILightweightLabelDecorator {
 
    private String iconLocation;

    private IConfigurationElement configElement;

    private ImageDescriptor descriptor;

    DeclarativeDecorator(IConfigurationElement definingElement, String iconPath) {
        this.iconLocation = iconPath;
        this.configElement = definingElement;
    }

    /**
     * @see dwtx.jface.viewers.IBaseLabelProvider#addListener(dwtx.jface.viewers.ILabelProviderListener)
     */
    public void addListener(ILabelProviderListener listener) {
    }

    /**
     * @see dwtx.jface.viewers.IBaseLabelProvider#dispose()
     */
    public void dispose() {
        //Nothing to do here
    }

    /**
     * @see dwtx.jface.viewers.IBaseLabelProvider#isLabelProperty(java.lang.Object,
     *      java.lang.String)
     */
    public bool isLabelProperty(Object element, String property) {
        return false;
    }

    /**
     * @see dwtx.jface.viewers.IBaseLabelProvider#removeListener(dwtx.jface.viewers.ILabelProviderListener)
     */
    public void removeListener(ILabelProviderListener listener) {
    }

    /**
     * @see dwtx.jface.viewers.ILightweightLabelDecorator#decorate(java.lang.Object,
     *      dwtx.jface.viewers.IDecoration)
     */
    public void decorate(Object element, IDecoration decoration) {
        if (descriptor is null) {
            URL url = BundleUtility.find(configElement.getDeclaringExtension()
                    .getNamespace(), iconLocation);
            if (url is null) {
                return;
            }
            descriptor = ImageDescriptor.createFromURL(url);
        }
        decoration.addOverlay(descriptor);
    }
}
