/*******************************************************************************
 * Copyright (c) 2007, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.internal.dialogs.BundleSigningTray;

import dwt.dwthelper.utils;

import java.io.IOException;
import java.security.GeneralSecurityException;
import java.security.cert.Certificate;
import java.security.cert.X509Certificate;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.Map.Entry;

import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;

import dwt.DWT;
import dwt.custom.StyledText;
import dwt.graphics.Color;
import dwt.graphics.GC;
import dwt.graphics.Point;
import dwt.layout.GridData;
import dwt.layout.GridLayout;
import dwt.widgets.Composite;
import dwt.widgets.Control;
import dwt.widgets.Display;
import dwt.widgets.Label;
import dwt.widgets.Shell;
import dwt.widgets.Text;
import dwtx.core.runtime.IProgressMonitor;
import dwtx.core.runtime.IStatus;
import dwtx.core.runtime.OperationCanceledException;
import dwtx.core.runtime.Status;
import dwtx.core.runtime.jobs.Job;
import dwtx.jface.dialogs.Dialog;
import dwtx.jface.dialogs.DialogTray;
import dwtx.jface.dialogs.TrayDialog;
import dwtx.jface.resource.JFaceResources;
import dwtx.osgi.signedcontent.SignedContent;
import dwtx.osgi.signedcontent.SignedContentFactory;
import dwtx.osgi.signedcontent.SignerInfo;
import dwtx.osgi.util.NLS;
import dwtx.ui.internal.WorkbenchMessages;
import dwtx.ui.internal.WorkbenchPlugin;
import dwtx.ui.internal.about.AboutBundleData;
import dwtx.ui.statushandlers.StatusManager;

/**
 * @since 3.3
 *
 */
public class BundleSigningTray extends DialogTray {


    private Text date;
    private StyledText certificate;
    private AboutBundleData data;
    private TrayDialog dialog;
    
    /**
     * 
     */
    public BundleSigningTray(TrayDialog dialog) {
        this.dialog = dialog;
    }
    
    public void setData(AboutBundleData data) {
        this.data = data;
        startJobs();
    }

    /* (non-Javadoc)
     * @see dwtx.jface.dialogs.DialogTray#createContents(dwt.widgets.Composite)
     */
    protected Control createContents(Composite parent) {
        Composite content = new Composite(parent, DWT.NONE);
        content.setLayoutData(new GridData(DWT.FILL, DWT.FILL, true, true));
        GridLayout layout = new GridLayout(2, false);
        content.setLayout(layout);
        // date
        Color backgroundColor = parent.getDisplay().getSystemColor(
                DWT.COLOR_WIDGET_BACKGROUND);
        {
            Label label = new Label(content, DWT.NONE);
            label.setText(WorkbenchMessages.BundleSigningTray_Signing_Date); 
            GridData data = new GridData(DWT.FILL, DWT.BEGINNING, true, false);
            date = new Text(content, DWT.READ_ONLY);
            GC gc = new GC(date);
            gc.setFont(JFaceResources.getDialogFont());
            Point size = gc.stringExtent(DateFormat.getDateTimeInstance().format(new Date()));
            data.widthHint = size.x;
            gc.dispose();
            date.setText(WorkbenchMessages.BundleSigningTray_Working); 
            date.setLayoutData(data);
            date.setBackground(backgroundColor);
        }
        // signer
        {
            Label label = new Label(content, DWT.NONE);
            label.setText(WorkbenchMessages.BundleSigningTray_Signing_Certificate); 
            GridData data = new GridData(DWT.BEGINNING, DWT.BEGINNING, true, false);
            data.horizontalSpan = 2;
            data = new GridData(DWT.FILL, DWT.FILL, true, true);
            data.horizontalSpan = 2;
            certificate = new StyledText(content, DWT.READ_ONLY | DWT.MULTI | DWT.WRAP);
            certificate.setText(WorkbenchMessages.BundleSigningTray_Working); 
            certificate.setLayoutData(data);
        }
        
        // problems
//      {
//          Label label = new Label(content, DWT.NONE);
//          label.setText("Problems:"); //$NON-NLS-1$
//
//      }
        Dialog.applyDialogFont(content);

        startJobs(); // start the jobs that will prime the content

        return content;
    }
    
    /**
     * 
     */
    private void startJobs() {
        if (!isOpen())
            return;
        certificate.setText(WorkbenchMessages.BundleSigningTray_Working); 
        date.setText(WorkbenchMessages.BundleSigningTray_Working); 
        final BundleContext bundleContext = WorkbenchPlugin.getDefault()
                .getBundleContext();
        final ServiceReference factoryRef = bundleContext
                .getServiceReference(SignedContentFactory.class.getName());
        if (factoryRef is null) {
            StatusManager.getManager().handle(
                    new Status(IStatus.WARNING, WorkbenchPlugin.PI_WORKBENCH,
                            WorkbenchMessages.BundleSigningTray_Cant_Find_Service), 
                    StatusManager.LOG);
            return;
        }

        final SignedContentFactory contentFactory = (SignedContentFactory) bundleContext
                .getService(factoryRef);
        if (contentFactory is null) {
            StatusManager.getManager().handle(
                    new Status(IStatus.WARNING, WorkbenchPlugin.PI_WORKBENCH,
                            WorkbenchMessages.BundleSigningTray_Cant_Find_Service), 
                    StatusManager.LOG);
            return;
        }

        final AboutBundleData myData = data;
        final Job signerJob = new Job(NLS.bind(WorkbenchMessages.BundleSigningTray_Determine_Signer_For, myData.getId())) { 

            protected IStatus run(IProgressMonitor monitor) {
                try {
                    if (myData !is data)
                        return Status.OK_STATUS;
                    SignedContent signedContent = contentFactory.getSignedContent(myData
                            .getBundle());
                    if (myData !is data)
                        return Status.OK_STATUS;
                    SignerInfo[] signers = signedContent.getSignerInfos();
                    final String signerText, dateText;
                    final Shell dialogShell = dialog.getShell();
                    if (!isOpen() && BundleSigningTray.this.data is myData)
                        return Status.OK_STATUS;

                    if (signers.length is 0) {
                        signerText = WorkbenchMessages.BundleSigningTray_Unsigned; 
                        dateText = WorkbenchMessages.BundleSigningTray_Unsigned; 
                    } else {
                        Properties [] certs = parseCerts(signers[0].getCertificateChain());
                        if (certs.length is 0)
                            signerText = WorkbenchMessages.BundleSigningTray_Unknown; 
                        else {
                            StringBuffer buffer = new StringBuffer();
                            for (Iterator i = certs[0].entrySet().iterator(); i.hasNext(); ) {
                                Map.Entry entry = (Entry) i.next();
                                buffer.append(entry.getKey());
                                buffer.append('=');
                                buffer.append(entry.getValue());
                                if (i.hasNext())
                                    buffer.append('\n');
                            }
                            signerText = buffer.toString();
                        }

                        Date signDate = signedContent.getSigningTime(signers[0]);
                        if (signDate !is null)
                            dateText = DateFormat.getDateTimeInstance().format(
                                    signDate);
                        else
                            dateText = WorkbenchMessages.BundleSigningTray_Unknown; 
                    }
                    
                    Display display = dialogShell.getDisplay();
                    display.asyncExec(new Runnable() {

                        public void run() {
                            // check to see if the tray is still visible and if we're still looking at the same item
                            if (!isOpen() && BundleSigningTray.this.data !is myData)
                                return;
                            certificate.setText(signerText);
                            date.setText(dateText);
                        }
                    });

                } catch (IOException e) {
                    return new Status(IStatus.ERROR,
                            WorkbenchPlugin.PI_WORKBENCH, e.getMessage(), e);
                } catch (GeneralSecurityException e) {
                    return new Status(IStatus.ERROR,
                            WorkbenchPlugin.PI_WORKBENCH, e.getMessage(), e);
                }
                return Status.OK_STATUS;
            }
        };
        signerJob.setSystem(true);
        signerJob.belongsTo(signerJob);
        signerJob.schedule();

        Job cleanup = new Job(WorkbenchMessages.BundleSigningTray_Unget_Signing_Service) { 

            protected IStatus run(IProgressMonitor monitor) {
                try {
                    getJobManager().join(signerJob, monitor);
                } catch (OperationCanceledException e) {
                } catch (InterruptedException e) {
                }
                bundleContext.ungetService(factoryRef);
                return Status.OK_STATUS;
            }
        };
        cleanup.setSystem(true);
        cleanup.schedule();

    }

    /**
     * 
     */
    private bool isOpen() {
        return certificate !is null && !certificate.isDisposed();
    }

    private Properties[] parseCerts(Certificate[] chain) {
        List certs = new ArrayList(chain.length);
        for (int i = 0; i < chain.length; i++) {
            if (!(chain[i] instanceof X509Certificate))
                continue;
            Map cert = parseCert(((X509Certificate) chain[i]).getSubjectDN().getName());
            if (cert !is null)
                certs.add(cert);
        }
        return (Properties []) certs.toArray(new Properties[certs.size()]);


    }

    /**
     * @param certString
     * @return
     */
    private Properties parseCert(String certString) {
        StringTokenizer toker = new StringTokenizer(certString, ","); //$NON-NLS-1$
        Properties cert = new Properties();
        while (toker.hasMoreTokens()) {
            String pair = toker.nextToken();
            int idx = pair.indexOf('=');
            if (idx > 0 && idx < pair.length() - 2) {
                String key = pair.substring(0, idx).trim();
                String value = pair.substring(idx + 1).trim();
                if (value.length() > 2) {
                    if (value.charAt(0) is '\"')
                        value = value.substring(1);

                    if (value.charAt(value.length() - 1) is '\"')
                        value = value.substring(0, value.length() - 1);
                }
                cert.setProperty(key, value);
            }
        }
        return cert;
    }

}
