/*******************************************************************************
 * Copyright (c) 2000, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.dialogs.ImportWizard;

import dwt.dwthelper.utils;


import dwt.widgets.Composite;
import dwtx.core.runtime.CoreException;
import dwtx.core.runtime.Path;
import dwtx.jface.viewers.IStructuredSelection;
import dwtx.jface.wizard.IWizardNode;
import dwtx.jface.wizard.Wizard;
import dwtx.ui.IWorkbench;
import dwtx.ui.IWorkbenchWizard;
import dwtx.ui.internal.IWorkbenchGraphicConstants;
import dwtx.ui.internal.IWorkbenchHelpContextIds;
import dwtx.ui.internal.WorkbenchImages;
import dwtx.ui.internal.WorkbenchMessages;
import dwtx.ui.internal.WorkbenchPlugin;
import dwtx.ui.internal.activities.ws.WorkbenchTriggerPoints;
import dwtx.ui.internal.registry.WizardsRegistryReader;
import dwtx.ui.model.AdaptableList;
import dwtx.ui.wizards.IWizardCategory;

/**
 * The import wizard allows the user to choose which nested import wizard to
 * run. The set of available wizards comes from the import wizard extension
 * point.
 */
public class ImportWizard extends Wizard {

    //the list selection page
    class SelectionPage extends WorkbenchWizardListSelectionPage {
        SelectionPage(IWorkbench w, IStructuredSelection ss, AdaptableList e,
                String s) {
            super(w, ss, e, s, WorkbenchTriggerPoints.IMPORT_WIZARDS);
        }

        public void createControl(Composite parent) {
            super.createControl(parent);
            getWorkbench()
                    .getHelpSystem()
                    .setHelp(
                            getControl(),
                            IWorkbenchHelpContextIds.IMPORT_WIZARD_SELECTION_WIZARD_PAGE);
        }

        public IWizardNode createWizardNode(WorkbenchWizardElement element) {
            return new WorkbenchWizardNode(this, element) {
                public IWorkbenchWizard createWizard() throws CoreException {
                    return wizardElement.createWizard();
                }
            };
        }
        
        
    }

    private IStructuredSelection selection;

    private IWorkbench workbench;

    /**
     * Creates the wizard's pages lazily.
     */
    public void addPages() {
        addPage(new SelectionPage(this.workbench, this.selection,
                getAvailableImportWizards(), WorkbenchMessages.ImportWizard_selectSource)); 
    }

    /**
     * Returns the import wizards that are available for invocation.
     */
    protected AdaptableList getAvailableImportWizards() {
        // TODO: imports are still flat - we need to get at the flat list. All
        // wizards will be in the "other" category.
        IWizardCategory root = WorkbenchPlugin.getDefault()
                .getImportWizardRegistry().getRootCategory();
        WizardCollectionElement otherCategory = (WizardCollectionElement) root
                .findCategory(new Path(
                        WizardsRegistryReader.UNCATEGORIZED_WIZARD_CATEGORY));
        if (otherCategory is null) {
            return new AdaptableList();
        }
        return otherCategory.getWizardAdaptableList();
    }

    /**
     * Initializes the wizard.
     * 
     * @param aWorkbench the workbench
     * @param currentSelection the current selection
     */
    public void init(IWorkbench aWorkbench,
            IStructuredSelection currentSelection) {
        this.workbench = aWorkbench;
        this.selection = currentSelection;

        setWindowTitle(WorkbenchMessages.ImportWizard_title); 
        setDefaultPageImageDescriptor(WorkbenchImages
                .getImageDescriptor(IWorkbenchGraphicConstants.IMG_WIZBAN_IMPORT_WIZ));
        setNeedsProgressMonitor(true);
    }

    /* (non-Javadoc)
     * @see dwtx.jface.wizard.IWizard#performFinish()
     */
    public bool performFinish() {
        ((SelectionPage) getPages()[0]).saveWidgetValues();
        return true;
    }
}
