/*******************************************************************************
 * Copyright (c) 2005, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.dialogs.PreferenceNodeFilter;

import dwt.dwthelper.utils;

import java.util.Collection;
import java.util.HashSet;

import dwtx.jface.preference.IPreferenceNode;
import dwtx.jface.viewers.Viewer;
import dwtx.jface.viewers.ViewerFilter;

/**
 * The PreferenceNodeFilter is a filter that only matches
 * a set of ids.
 */
public class PreferenceNodeFilter extends ViewerFilter {

    Collection ids = new HashSet();

    /**
     * Create a new instance of the receiver on a
     * list of filteredIds.
     * @param filteredIds The collection of ids that
     * will be shown.
     */
    public PreferenceNodeFilter(String[] filteredIds) {
        super();
        for (int i = 0; i < filteredIds.length; i++) {
            ids.add(filteredIds[i]);
        }
    }

    /* (non-Javadoc)
     * @see dwtx.jface.viewers.ViewerFilter#select(dwtx.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
     */
    public bool select(Viewer viewer, Object parentElement, Object element) {
        return checkNodeAndChildren((IPreferenceNode) element);
    }

    /**
     * Check to see if the node or any of its children 
     * have an id in the ids.
     * @param node WorkbenchPreferenceNode
     * @return bool <code>true</code> if node or oe of its children
     * has an id in the ids.
     */
    private bool checkNodeAndChildren(IPreferenceNode node) {
        if(ids.contains(node.getId())) {
            return true;
        }
        
        IPreferenceNode[] subNodes = node.getSubNodes();
        for (int i = 0; i < subNodes.length; i++) {
            if(checkNodeAndChildren(subNodes[i])) {
                return true;
            }
            
        }
        return false;
    }

}
