/*******************************************************************************
 * Copyright (c) 2004, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.dialogs.WizardActivityFilter;

import dwt.dwthelper.utils;

import dwtx.jface.viewers.AbstractTreeViewer;
import dwtx.jface.viewers.ITreeContentProvider;
import dwtx.jface.viewers.Viewer;
import dwtx.jface.viewers.ViewerFilter;
import dwtx.ui.activities.WorkbenchActivityHelper;
import dwtx.ui.model.AdaptableList;

/**
 * Viewer filter designed to work with the new wizard page (and its input/content provider).
 * This will filter all non-primary wizards that are not enabled by activity.
 * 
 * @since 3.0
 */
public class WizardActivityFilter extends ViewerFilter {

    /* (non-Javadoc)
     * @see dwtx.jface.viewers.ViewerFilter#select(dwtx.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
     */
    public bool select(Viewer viewer, Object parentElement, Object element) {
        Object[] children = ((ITreeContentProvider) ((AbstractTreeViewer) viewer)
                .getContentProvider()).getChildren(element);
        if (children.length > 0) {
            return filter(viewer, element, children).length > 0;
        }

        if (parentElement.getClass().equals(AdaptableList.class)) {
            return true; //top-level ("primary") wizards should always be returned
        }

        if (WorkbenchActivityHelper.filterItem(element)) {
            return false;
        }

        return true;
    }
}
