/*******************************************************************************
 * Copyright (c) 2004, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.dialogs.WizardTagFilter;

import dwt.dwthelper.utils;

import dwtx.jface.viewers.AbstractTreeViewer;
import dwtx.jface.viewers.ITreeContentProvider;
import dwtx.jface.viewers.Viewer;
import dwtx.jface.viewers.ViewerFilter;
import dwtx.ui.wizards.IWizardDescriptor;

/**
 * A viewer filter that will exclude all IWizardDescriptors that do not have at
 * least one tag in a provided set.
 * 
 * @since 3.1
 */
public class WizardTagFilter extends ViewerFilter {

    private String [] myTags;
    
    /**
     * Create a new instance of this filter
     * @param tags the wizard tags to allow
     */
    public WizardTagFilter(String [] tags) {
        myTags = tags;
    }
    
    /* (non-Javadoc)
     * @see dwtx.jface.viewers.ViewerFilter#select(dwtx.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
     */
    public bool select(Viewer viewer, Object parentElement, Object element) {
        if (element instanceof IWizardDescriptor) {
            IWizardDescriptor desc = (IWizardDescriptor)element;
            String [] tags = desc.getTags();
            for (int i = 0; i < tags.length; i++) {
                for (int j = 0; j < myTags.length; j++) {
                    if (tags[i].equals(myTags[j])) {
                        return true;
                    }
                }
            }
            return false;
        }
        Object[] children = ((ITreeContentProvider) ((AbstractTreeViewer) viewer)
                .getContentProvider()).getChildren(element);
        if (children.length > 0) {
            return filter(viewer, element, children).length > 0;
        }

        return false;
    }
}
