/*******************************************************************************
 * Copyright (c) 2000, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.dialogs.WorkbenchPreferenceNode;

import dwt.dwthelper.utils;

import dwtx.core.runtime.CoreException;
import dwtx.core.runtime.IConfigurationElement;
import dwtx.core.runtime.IStatus;
import dwtx.ui.IWorkbenchPreferencePage;
import dwtx.ui.PlatformUI;
import dwtx.ui.internal.WorkbenchMessages;
import dwtx.ui.internal.WorkbenchPlugin;
import dwtx.ui.internal.misc.StatusUtil;
import dwtx.ui.internal.preferences.WorkbenchPreferenceExtensionNode;
import dwtx.ui.internal.registry.CategorizedPageRegistryReader;
import dwtx.ui.internal.registry.IWorkbenchRegistryConstants;
import dwtx.ui.statushandlers.StatusManager;

/**
 * A proxy for a preference page to avoid creation of preference page just to
 * show a node in the preference dialog tree.
 */
public class WorkbenchPreferenceNode extends WorkbenchPreferenceExtensionNode {

    /**
     * Create a new instance of the receiver.
     * @param nodeId
     * @param element
     */
    public WorkbenchPreferenceNode(String nodeId, IConfigurationElement element) {
        super(nodeId, element);
    }

    /**
     * Creates the preference page this node stands for.
     */
    public void createPage() {
        IWorkbenchPreferencePage page;
        try {
            page = (IWorkbenchPreferencePage) WorkbenchPlugin.createExtension(
                    getConfigurationElement(), IWorkbenchRegistryConstants.ATT_CLASS);
        } catch (CoreException e) {
            // Just inform the user about the error. The details are
            // written to the log by now.
            IStatus errStatus = StatusUtil.newStatus(e.getStatus(), WorkbenchMessages.PreferenceNode_errorMessage); 
            StatusManager.getManager().handle(errStatus, StatusManager.SHOW | StatusManager.LOG);
            page = new ErrorPreferencePage();
        }

        page.init(PlatformUI.getWorkbench());
        if (getLabelImage() !is null) {
            page.setImageDescriptor(getImageDescriptor());
        }
        page.setTitle(getLabelText());
        setPage(page);
    }

    /**
     * Return the category name for the node.
     * @return java.lang.String
     */
    public String getCategory() {
        return getConfigurationElement().getAttribute(
                CategorizedPageRegistryReader.ATT_CATEGORY);
    }   
}
