/*******************************************************************************
 * Copyright (c) 2005, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.internal.dialogs.WorkingSetLabelProvider;

import dwt.dwthelper.utils;


import dwt.graphics.Image;
import dwtx.core.runtime.Assert;
import dwtx.jface.resource.ImageDescriptor;
import dwtx.jface.resource.JFaceResources;
import dwtx.jface.resource.LocalResourceManager;
import dwtx.jface.resource.ResourceManager;
import dwtx.jface.viewers.LabelProvider;
import dwtx.ui.IWorkingSet;

public class WorkingSetLabelProvider extends LabelProvider {
    private ResourceManager images;

    /**
     * Create a new instance of the receiver.
     */
    public WorkingSetLabelProvider() {
        images = new LocalResourceManager(JFaceResources.getResources());
    }

    public void dispose() {
        images.dispose();

        super.dispose();
    }

    public Image getImage(Object object) {
        Assert.isTrue(object instanceof IWorkingSet);
        IWorkingSet workingSet = (IWorkingSet) object;
        ImageDescriptor imageDescriptor = workingSet.getImageDescriptor();

        if (imageDescriptor is null) {
            return null;
        }

        Image icon = (Image) images.get(imageDescriptor);
        return icon;
    }

    public String getText(Object object) {
        Assert.isTrue(object instanceof IWorkingSet);
        IWorkingSet workingSet = (IWorkingSet) object;
        return workingSet.getLabel();
    }
}
