/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 ******************************************************************************/

module dwtx.ui.internal.handlers.ClosePerspectiveHandler;

import dwt.dwthelper.utils;

import java.util.Map;

import dwtx.core.commands.AbstractHandler;
import dwtx.core.commands.ExecutionEvent;
import dwtx.core.commands.ExecutionException;
import dwtx.ui.IPerspectiveDescriptor;
import dwtx.ui.IWorkbenchWindow;
import dwtx.ui.handlers.HandlerUtil;
import dwtx.ui.internal.Perspective;
import dwtx.ui.internal.WorkbenchPage;

public class ClosePerspectiveHandler extends AbstractHandler {

    /**
     * The name of the parameter providing the perspective id.
     */
    private static final String PARAMETER_NAME_PERSPECTIVE_ID = "dwtx.ui.window.closePerspective.perspectiveId"; //$NON-NLS-1$

    public Object execute(ExecutionEvent event) throws ExecutionException {
        IWorkbenchWindow activeWorkbenchWindow = HandlerUtil
                .getActiveWorkbenchWindow(event);
        if (activeWorkbenchWindow !is null) {
            WorkbenchPage page = (WorkbenchPage) activeWorkbenchWindow
                    .getActivePage();
            if (page !is null) {
                Map parameters = event.getParameters();
                String value = (String) parameters
                        .get(PARAMETER_NAME_PERSPECTIVE_ID);
                if (value is null) {
                    page.closePerspective(page.getPerspective(), true, true);
                } else {
                    IPerspectiveDescriptor perspective = activeWorkbenchWindow
                            .getWorkbench().getPerspectiveRegistry()
                            .findPerspectiveWithId(value);
                    if (perspective !is null) {
                        page.closePerspective(perspective, true, true);
                    }
                }
            }
        }
        return null;
    }

    /**
     * Closes the specified perspective. Nothing will happen if the given page
     * or perspective are <code>null</null>.
     * @param page
     *      a reference to the page
     * @param persp
     *      the perspective to close
     */
    public static void closePerspective(WorkbenchPage page, Perspective persp) {
        if (page !is null && persp !is null) {
            page.closePerspective(persp.getDesc(), true, true);
        }
    }
}
