/*******************************************************************************
 * Copyright (c) 2007, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.internal.handlers.HandlerServiceFactory;

import dwt.dwthelper.utils;

import dwtx.core.expressions.Expression;
import dwtx.ui.IWorkbench;
import dwtx.ui.IWorkbenchPartSite;
import dwtx.ui.IWorkbenchWindow;
import dwtx.ui.commands.ICommandService;
import dwtx.ui.handlers.IHandlerService;
import dwtx.ui.internal.expressions.ActivePartExpression;
import dwtx.ui.internal.expressions.WorkbenchWindowExpression;
import dwtx.ui.internal.services.IWorkbenchLocationService;
import dwtx.ui.services.AbstractServiceFactory;
import dwtx.ui.services.IEvaluationService;
import dwtx.ui.services.IServiceLocator;

/**
 * @since 3.4
 * 
 */
public class HandlerServiceFactory extends AbstractServiceFactory {

    /*
     * (non-Javadoc)
     * 
     * @see
     * dwtx.ui.services.AbstractServiceFactory#create(java.lang.Class,
     * dwtx.ui.services.IServiceLocator,
     * dwtx.ui.services.IServiceLocator)
     */
    public Object create(Class serviceInterface, IServiceLocator parentLocator,
            IServiceLocator locator) {
        if (!IHandlerService.class.equals(serviceInterface)) {
            return null;
        }
        IWorkbenchLocationService wls = (IWorkbenchLocationService) locator
                .getService(IWorkbenchLocationService.class);
        final IWorkbench wb = wls.getWorkbench();
        if (wb is null) {
            return null;
        }

        Object parent = parentLocator.getService(serviceInterface);
        if (parent is null) {
            ICommandService commands = (ICommandService) locator
                    .getService(ICommandService.class);
            IEvaluationService evals = (IEvaluationService) locator
                    .getService(IEvaluationService.class);
            HandlerService handlerService = new HandlerService(commands, evals,
                    locator);
            handlerService.readRegistry();
            return handlerService;
        }

        final IWorkbenchWindow window = wls.getWorkbenchWindow();
        final IWorkbenchPartSite site = wls.getPartSite();
        if (site is null) {
            Expression exp = new WorkbenchWindowExpression(window);
            return new SlaveHandlerService((IHandlerService) parent, exp);
        }

        if (parent instanceof SlaveHandlerService) {
            Expression parentExp = ((SlaveHandlerService) parent)
                    .getDefaultExpression();
            if (parentExp instanceof ActivePartExpression) {
                return new NestableHandlerService((IHandlerService) parent,
                        parentExp);
            }
        }

        Expression exp = new ActivePartExpression(site.getPart());
        return new SlaveHandlerService((IHandlerService) parent, exp);
    }
}
