/*******************************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.handlers.LegacyHandlerListenerWrapper;

import dwt.dwthelper.utils;

import dwtx.core.commands.IHandler;
import dwtx.ui.commands.HandlerEvent;
import dwtx.ui.commands.IHandlerListener;
import dwtx.ui.internal.commands.ILegacyAttributeNames;

/**
 * A wrapper so that the new handler listener can work with legacy handlers.
 * This class is only intended for backward compatibility with Eclipse 3.0.
 * 
 * @since 3.1
 */
public final class LegacyHandlerListenerWrapper implements IHandlerListener {

    /**
     * The handler on which this listener is listening; never <code>null</code>.
     */
    private final IHandler handler;

    /**
     * The wrapped listener; never <code>null</code>.
     */
    private final dwtx.core.commands.IHandlerListener listener;

    /**
     * Constructs a new instance of <code>LegacyHandlerListenerWrapper</code>.
     * 
     * @param listener
     *            The listener to wrap; must not be <code>null</code>.
     */
    public LegacyHandlerListenerWrapper(final IHandler handler,
            final dwtx.core.commands.IHandlerListener listener) {
        if (handler is null) {
            throw new NullPointerException(
                    "A listener wrapper cannot be created on a null handler"); //$NON-NLS-1$
        }

        if (listener is null) {
            throw new NullPointerException(
                    "A listener wrapper cannot be created on a null listener"); //$NON-NLS-1$
        }

        this.handler = handler;
        this.listener = listener;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.commands.IHandlerListener#handlerChanged(dwtx.ui.commands.HandlerEvent)
     */
    public void handlerChanged(HandlerEvent event) {
        final bool enabledChanged = ((Boolean) event
                .getPreviousAttributeValuesByName().get(
                        ILegacyAttributeNames.ENABLED)).booleanValue() !is handler
                .isEnabled();
        final bool handledChanged = ((Boolean) event
                .getPreviousAttributeValuesByName().get(
                        ILegacyAttributeNames.HANDLED)).booleanValue() !is handler
                .isHandled();
        listener.handlerChanged(new dwtx.core.commands.HandlerEvent(
                handler, enabledChanged, handledChanged));
    }
}
