/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 ******************************************************************************/

module dwtx.ui.internal.handlers.NewEditorHandler;

import dwt.dwthelper.utils;

import dwtx.core.commands.AbstractHandler;
import dwtx.core.commands.ExecutionEvent;
import dwtx.core.commands.ExecutionException;
import dwtx.ui.IEditorPart;
import dwtx.ui.IPersistableEditor;
import dwtx.ui.IWorkbenchPage;
import dwtx.ui.IWorkbenchWindow;
import dwtx.ui.PartInitException;
import dwtx.ui.XMLMemento;
import dwtx.ui.handlers.HandlerUtil;
import dwtx.ui.internal.IWorkbenchConstants;
import dwtx.ui.internal.WorkbenchMessages;
import dwtx.ui.internal.WorkbenchPage;
import dwtx.ui.internal.dialogs.DialogUtil;

/**
 * Open a new editor on the active editor's input.
 * 
 * @since 3.4
 * 
 */
public class NewEditorHandler extends AbstractHandler {

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.core.commands.IHandler#execute(dwtx.core.commands.ExecutionEvent)
     */
    public Object execute(ExecutionEvent event) throws ExecutionException {
        IWorkbenchWindow activeWorkbenchWindow = HandlerUtil
                .getActiveWorkbenchWindow(event);
        IWorkbenchPage page = activeWorkbenchWindow.getActivePage();
        if (page is null) {
            return null;
        }
        IEditorPart editor = page.getActiveEditor();
        if (editor is null) {
            return null;
        }
        String editorId = editor.getSite().getId();
        if (editorId is null) {
            return null;
        }
        try {
            if (editor instanceof IPersistableEditor) {
                XMLMemento editorState = XMLMemento
                        .createWriteRoot(IWorkbenchConstants.TAG_EDITOR_STATE);
                ((IPersistableEditor) editor).saveState(editorState);
                ((WorkbenchPage) page).openEditor(editor.getEditorInput(),
                        editorId, true, IWorkbenchPage.MATCH_NONE, editorState);
            } else {
                page.openEditor(editor.getEditorInput(), editorId, true,
                        IWorkbenchPage.MATCH_NONE);
            }
        } catch (PartInitException e) {
            DialogUtil.openError(activeWorkbenchWindow.getShell(),
                    WorkbenchMessages.Error, e.getMessage(), e);
        }
        return null;
    }

}
