/*******************************************************************************
 * Copyright (c) 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 ******************************************************************************/

module dwtx.ui.internal.handlers.OpenInNewWindowHandler;

import dwt.dwthelper.utils;

import dwtx.core.commands.AbstractHandler;
import dwtx.core.commands.ExecutionEvent;
import dwtx.core.commands.ExecutionException;
import dwtx.core.runtime.IAdaptable;
import dwtx.ui.IWorkbenchPage;
import dwtx.ui.IWorkbenchWindow;
import dwtx.ui.WorkbenchException;
import dwtx.ui.handlers.HandlerUtil;
import dwtx.ui.internal.Workbench;
import dwtx.ui.internal.WorkbenchMessages;
import dwtx.ui.internal.misc.StatusUtil;
import dwtx.ui.statushandlers.StatusManager;

/**
 * @since 3.4
 * 
 */
public class OpenInNewWindowHandler extends AbstractHandler {

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.core.commands.IHandler#execute(dwtx.core.commands.ExecutionEvent)
     */
    public Object execute(ExecutionEvent event) throws ExecutionException {
        IWorkbenchWindow activeWorkbenchWindow = HandlerUtil
                .getActiveWorkbenchWindow(event);
        if (activeWorkbenchWindow is null) {
            return null;
        }
        try {
            String perspId = null;

            IWorkbenchPage page = activeWorkbenchWindow.getActivePage();
            IAdaptable pageInput = ((Workbench) activeWorkbenchWindow
                    .getWorkbench()).getDefaultPageInput();
            if (page !is null && page.getPerspective() !is null) {
                perspId = page.getPerspective().getId();
                pageInput = page.getInput();
            } else {
                perspId = activeWorkbenchWindow.getWorkbench()
                        .getPerspectiveRegistry().getDefaultPerspective();
            }
            
            activeWorkbenchWindow.getWorkbench().openWorkbenchWindow(perspId,
                    pageInput);
        } catch (WorkbenchException e) {
            StatusUtil.handleStatus(e.getStatus(),
                    WorkbenchMessages.OpenInNewWindowAction_errorTitle
                            + ": " + e.getMessage(), //$NON-NLS-1$
                    StatusManager.SHOW);
        }
        return null;

    }

}
