/*******************************************************************************
 * Copyright (c) 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 ******************************************************************************/

module dwtx.ui.internal.handlers.QuickMenuHandler;

import dwt.dwthelper.utils;

import dwtx.core.commands.AbstractHandler;
import dwtx.core.commands.ExecutionEvent;
import dwtx.core.commands.ExecutionException;
import dwtx.core.runtime.IProgressMonitor;
import dwtx.core.runtime.IStatus;
import dwtx.core.runtime.Status;
import dwtx.jface.action.ContributionManager;
import dwtx.jface.action.IMenuListener2;
import dwtx.jface.action.IMenuManager;
import dwtx.ui.PlatformUI;
import dwtx.ui.actions.QuickMenuCreator;
import dwtx.ui.menus.IMenuService;
import dwtx.ui.progress.UIJob;

/**
 * Support for a command based QuickMenuAction that can pop up a
 * menu-contribution based context menu.
 * <p>
 * This is experimental and should not be moved.
 * </p>
 * 
 * @since 3.4
 */
public class QuickMenuHandler extends AbstractHandler implements IMenuListener2 {
    private QuickMenuCreator creator = new QuickMenuCreator() {

        protected void fillMenu(IMenuManager menu) {
            if (!(menu instanceof ContributionManager)) {
                return;
            }
            IMenuService service = (IMenuService) PlatformUI.getWorkbench()
                    .getService(IMenuService.class);
            service.populateContributionManager((ContributionManager) menu,
                    locationURI);
            menu.addMenuListener(QuickMenuHandler.this);
        }

    };

    private String locationURI;

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.core.commands.IHandler#execute(dwtx.core.commands.ExecutionEvent)
     */
    public Object execute(ExecutionEvent event) throws ExecutionException {
        locationURI = event.getParameter("dwtx.ui.window.quickMenu.uri"); //$NON-NLS-1$
        if (locationURI is null) {
            throw new ExecutionException("locatorURI must not be null"); //$NON-NLS-1$
        }
        creator.createMenu();
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.core.commands.AbstractHandler#dispose()
     */
    public void dispose() {
        if (creator !is null) {
            creator.dispose();
            creator = null;
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.jface.action.IMenuListener2#menuAboutToHide(dwtx.jface.action.IMenuManager)
     */
    public void menuAboutToHide(final IMenuManager managerM) {
        new UIJob("quickMenuCleanup") { //$NON-NLS-1$

            public IStatus runInUIThread(IProgressMonitor monitor) {
                IMenuService service = (IMenuService) PlatformUI.getWorkbench()
                        .getService(IMenuService.class);
                service.releaseContributions((ContributionManager) managerM);
                return Status.OK_STATUS;
            }

        }.schedule();
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.jface.action.IMenuListener#menuAboutToShow(dwtx.jface.action.IMenuManager)
     */
    public void menuAboutToShow(IMenuManager manager) {
        // no-op
    }
}
