/*******************************************************************************
 * Copyright (c) 2006, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.internal.handlers.ShowPreferencePageHandler;

import dwt.dwthelper.utils;


import dwt.widgets.Shell;
import dwtx.core.commands.AbstractHandler;
import dwtx.core.commands.ExecutionEvent;
import dwtx.core.commands.ExecutionException;
import dwtx.jface.preference.PreferenceDialog;
import dwtx.ui.IWorkbenchWindow;
import dwtx.ui.dialogs.PreferencesUtil;
import dwtx.ui.handlers.HandlerUtil;

/**
 * <p>
 * Shows the given preference page. If no preference page id is specified in the
 * parameters, then this opens the preferences dialog to whatever page was
 * active the last time the dialog was shown.
 * </p>
 * <p>
 * This class is not intended for use outside of the
 * <code>dwtx.ui.workbench</code> plug-in.
 * </p>
 * 
 * @since 3.2
 */
public final class ShowPreferencePageHandler extends AbstractHandler {

    /**
     * The name of the parameter providing the view identifier.
     */
    private static final String PARAMETER_ID_PREFERENCE_PAGE_ID = "preferencePageId"; //$NON-NLS-1$

    public final Object execute(final ExecutionEvent event)
            throws ExecutionException {
        final String preferencePageId = event
                .getParameter(PARAMETER_ID_PREFERENCE_PAGE_ID);
        final IWorkbenchWindow activeWorkbenchWindow = HandlerUtil
                .getActiveWorkbenchWindowChecked(event);

        final Shell shell = activeWorkbenchWindow.getShell();
        if (shell is null) {
            throw new ExecutionException("no shell for active workbench window"); //$NON-NLS-1$
        }

        final PreferenceDialog dialog = PreferencesUtil
                .createPreferenceDialogOn(shell, preferencePageId, null, null);
        dialog.open();

        return null;
    }

}
