/*******************************************************************************
 * Copyright (c) 2007, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.internal.handlers.ToggleCoolbarHandler;

import dwt.dwthelper.utils;

import java.util.HashMap;
import java.util.Map;

import dwtx.core.commands.AbstractHandler;
import dwtx.core.commands.ExecutionEvent;
import dwtx.core.commands.ExecutionException;
import dwtx.ui.IWorkbenchWindow;
import dwtx.ui.commands.ICommandService;
import dwtx.ui.commands.IElementUpdater;
import dwtx.ui.handlers.HandlerUtil;
import dwtx.ui.internal.WorkbenchMessages;
import dwtx.ui.internal.WorkbenchWindow;
import dwtx.ui.internal.services.IWorkbenchLocationService;
import dwtx.ui.menus.UIElement;
import dwtx.ui.services.IServiceScopes;

/**
 * Handler that toggles the visibility of the coolbar/perspective bar in a given
 * window.
 * 
 * @since 3.3
 */
public class ToggleCoolbarHandler extends AbstractHandler implements
        IElementUpdater {

    /*
     * (non-Javadoc)
     * 
     * @see
     * dwtx.core.commands.IHandler#execute(dwtx.core.commands.
     * ExecutionEvent)
     */
    public Object execute(ExecutionEvent event) throws ExecutionException {
        final IWorkbenchWindow activeWorkbenchWindow = HandlerUtil
                .getActiveWorkbenchWindowChecked(event);
        if (activeWorkbenchWindow instanceof WorkbenchWindow) {
            WorkbenchWindow window = (WorkbenchWindow) activeWorkbenchWindow;
            window.toggleToolbarVisibility();
            ICommandService commandService = (ICommandService) activeWorkbenchWindow
                    .getService(ICommandService.class);
            Map filter = new HashMap();
            filter.put(IServiceScopes.WINDOW_SCOPE, window);
            commandService.refreshElements(event.getCommand().getId(), filter);
        }

        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * dwtx.ui.commands.IElementUpdater#updateElement(dwtx.ui.
     * menus.UIElement, java.util.Map)
     */
    public void updateElement(UIElement element, Map parameters) {
        IWorkbenchLocationService wls = (IWorkbenchLocationService) element
                .getServiceLocator()
                .getService(IWorkbenchLocationService.class);
        IWorkbenchWindow window = wls.getWorkbenchWindow();
        if (window is null || !(window instanceof WorkbenchWindow))
            return;
        element
                .setText(isCoolbarVisible((WorkbenchWindow) window) ? WorkbenchMessages.ToggleCoolbarVisibilityAction_hide_text
                        : WorkbenchMessages.ToggleCoolbarVisibilityAction_show_text);
    }

    /**
     * Return whether the coolbar is currently visible.
     * 
     * @param window
     *      the window to test
     * @return whether or not the coolbar is visible
     */
    private bool isCoolbarVisible(WorkbenchWindow window) {
        return window.getCoolBarVisible() || window.getPerspectiveBarVisible();
    }
}
