/*******************************************************************************
 * Copyright (c) 2004, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.intro.IntroDescriptor;

import dwt.dwthelper.utils;

import dwtx.core.runtime.CoreException;
import dwtx.core.runtime.IConfigurationElement;
import dwtx.core.runtime.IStatus;
import dwtx.core.runtime.Status;
import dwtx.jface.resource.ImageDescriptor;
import dwtx.ui.IPluginContribution;
import dwtx.ui.internal.registry.IWorkbenchRegistryConstants;
import dwtx.ui.intro.IIntroPart;
import dwtx.ui.intro.IntroContentDetector;
import dwtx.ui.plugin.AbstractUIPlugin;

/**
 * Describes an introduction extension.
 * 
 * @since 3.0
 */
public class IntroDescriptor implements IIntroDescriptor, IPluginContribution {


    private IConfigurationElement element;

    private ImageDescriptor imageDescriptor;

    /**
     * Create a new IntroDescriptor for an extension.
     */
    public IntroDescriptor(IConfigurationElement configElement)
            throws CoreException {
        element = configElement;  

        if (configElement.getAttribute(IWorkbenchRegistryConstants.ATT_CLASS) is null) {
            throw new CoreException(new Status(IStatus.ERROR, configElement
                    .getNamespace(), 0,
                    "Invalid extension (Missing class name): " + getId(), //$NON-NLS-1$
                    null));
        }
    }

    /* (non-Javadoc)
     * @see dwtx.ui.intro.IIntroDescriptor#createIntro()
     */
    public IIntroPart createIntro() throws CoreException {
        return (IIntroPart) element.createExecutableExtension(IWorkbenchRegistryConstants.ATT_CLASS);
    }
    
    public IntroContentDetector getIntroContentDetector() throws CoreException {
        if (element.getAttribute(IWorkbenchRegistryConstants.ATT_CONTENT_DETECTOR) is null) {
            return null;
        }
        return (IntroContentDetector) element.createExecutableExtension(IWorkbenchRegistryConstants.ATT_CONTENT_DETECTOR);
    }

    /* (non-Javadoc)
     * @see dwtx.ui.IIntroDescriptor#getId()
     */
    public String getId() {     
        return element.getAttribute(IWorkbenchRegistryConstants.ATT_ID);
    }

    /* (non-Javadoc)
     * @see dwtx.ui.IIntroDescriptor#getImageDescriptor()
     */
    public ImageDescriptor getImageDescriptor() {
        if (imageDescriptor !is null) {
            return imageDescriptor;
        }        
        String iconName = element.getAttribute(IWorkbenchRegistryConstants.ATT_ICON);
        if (iconName is null) {
            return null;
        }
        
        imageDescriptor = AbstractUIPlugin.imageDescriptorFromPlugin(element
                .getNamespace(), iconName);
        return imageDescriptor;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.IPluginContribution#getLocalId()
     */
    public String getLocalId() {
        return element.getAttribute(IWorkbenchRegistryConstants.ATT_ID);
    }

    /* (non-Javadoc)
     * @see dwtx.ui.IPluginContribution#getPluginId()
     */
    public String getPluginId() {
        return element.getNamespace();
    }
    
    /**
     * Returns the configuration element.
     * 
     * @return the configuration element
     * @since 3.1
     */
    public IConfigurationElement getConfigurationElement() {
        return element;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.intro.IIntroDescriptor#getLabelOverride()
     */
    public String getLabelOverride() {
        return element.getAttribute(IWorkbenchRegistryConstants.ATT_LABEL);
    }
}
