/*******************************************************************************
 * Copyright (c) 2007, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.internal.keys.CategoryPatternFilter;

import dwt.dwthelper.utils;

import dwtx.core.commands.Category;
import dwtx.core.commands.ParameterizedCommand;
import dwtx.core.commands.common.NotDefinedException;
import dwtx.jface.bindings.Binding;
import dwtx.jface.viewers.Viewer;
import dwtx.ui.dialogs.PatternFilter;
import dwtx.ui.internal.keys.model.BindingElement;

class CategoryPatternFilter extends PatternFilter {
    private bool filterCategories;
    final Category uncategorized;

    public CategoryPatternFilter(bool filterCategories, Category c) {
        uncategorized = c;
        filterCategories(filterCategories);
    }

    public void filterCategories(bool b) {
        filterCategories = b;
        if (filterCategories) {
            setPattern("dwtx.ui.keys.optimization.false"); //$NON-NLS-1$
        } else {
            setPattern("dwtx.ui.keys.optimization.true"); //$NON-NLS-1$
        }
    }
    
    public bool isFilteringCategories() {
        return filterCategories;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.dialogs.PatternFilter#isLeafMatch(dwtx.jface.viewers.Viewer,
     *      java.lang.Object)
     */
    protected bool isLeafMatch(Viewer viewer, Object element) {
        if (filterCategories) {
            final ParameterizedCommand cmd = getCommand(element);
            try {
                if (cmd !is null
                        && cmd.getCommand().getCategory() is uncategorized) {
                    return false;
                }
            } catch (NotDefinedException e) {
                return false;
            }
        }
        return super.isLeafMatch(viewer, element);
    }

    private ParameterizedCommand getCommand(Object element) {
        if (element instanceof BindingElement) {
            Object modelObject = ((BindingElement) element).getModelObject();
            if (modelObject instanceof Binding) {
                return ((Binding) modelObject).getParameterizedCommand();
            } else if (modelObject instanceof ParameterizedCommand) {
                return (ParameterizedCommand) modelObject;
            }
        }
        return null;
    }
}
