/*******************************************************************************
 * Copyright (c) 2007, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.internal.keys.KeysPreferenceFiltersDialog;

import dwt.dwthelper.utils;


import dwt.DWT;
import dwt.layout.GridData;
import dwt.layout.GridLayout;
import dwt.widgets.Button;
import dwt.widgets.Composite;
import dwt.widgets.Control;
import dwt.widgets.Shell;
import dwtx.ui.preferences.ViewSettingsDialog;

/**
 * Creates a dialog box for applying filter selection of When combo box in
 * NewKeysPreferencePage
 * 
 * @since 3.3
 * 
 */
public class KeysPreferenceFiltersDialog extends ViewSettingsDialog {

    private Button actionSetFilterCheckBox;
    private Button internalFilterCheckBox;
    private Button uncategorizedFilterCheckBox;

    private bool filterActionSet;
    private bool filterInternal;
    private bool filterUncategorized;
    private bool filterShowUnboundCommands;

    void setFilterActionSet(bool b) {
        filterActionSet = b;
    }

    void setFilterInternal(bool b) {
        filterInternal = b;
    }
    
    void setFilterUncategorized(bool b) {
        filterUncategorized = b;
    }

    bool getFilterActionSet() {
        return filterActionSet;
    }

    bool getFilterInternal() {
        return filterInternal;
    }
    
    bool getFilterUncategorized() {
        return filterUncategorized;
    }

    /**
     * @param parentShell
     */
    public KeysPreferenceFiltersDialog(Shell parentShell) {
        super(parentShell);
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.preferences.ViewSettingsDialog#performDefaults()
     */
    protected void performDefaults() {
        actionSetFilterCheckBox.setSelection(true);
        internalFilterCheckBox.setSelection(true);
        uncategorizedFilterCheckBox.setSelection(true);
        super.performDefaults();
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.jface.dialogs.Dialog#createDialogArea(dwt.widgets.Composite)
     */
    protected Control createDialogArea(Composite parent) {
        Composite topComposite = (Composite) super.createDialogArea(parent);
        GridLayout layout = new GridLayout(1, false);
        topComposite.setLayout(layout);
        topComposite.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL));
        actionSetFilterCheckBox = new Button(topComposite, DWT.CHECK);
        actionSetFilterCheckBox
                .setText(NewKeysPreferenceMessages.ActionSetFilterCheckBox_Text);
        internalFilterCheckBox = new Button(topComposite, DWT.CHECK);
        internalFilterCheckBox
                .setText(NewKeysPreferenceMessages.InternalFilterCheckBox_Text);
        uncategorizedFilterCheckBox = new Button(topComposite, DWT.CHECK);
        uncategorizedFilterCheckBox
                .setText(NewKeysPreferenceMessages.UncategorizedFilterCheckBox_Text);

        actionSetFilterCheckBox.setSelection(filterActionSet);
        internalFilterCheckBox.setSelection(filterInternal);
        uncategorizedFilterCheckBox.setSelection(filterUncategorized);
        applyDialogFont(topComposite);

        return topComposite;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.jface.dialogs.Dialog#okPressed()
     */
    protected void okPressed() {
        filterActionSet = actionSetFilterCheckBox.getSelection();
        filterInternal = internalFilterCheckBox.getSelection();
        filterUncategorized = uncategorizedFilterCheckBox.getSelection();
        super.okPressed();
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.jface.window.Window#configureShell(dwt.widgets.Shell)
     */
    protected void configureShell(Shell newShell) {
        super.configureShell(newShell);
        newShell
                .setText(NewKeysPreferenceMessages.KeysPreferenceFilterDialog_Title);
    }

    bool getFilterShowUnboundCommands() {
        return filterShowUnboundCommands;
    }

    void setFilterUnboundCommands(bool filterUnboundCommands) {
        this.filterShowUnboundCommands = filterUnboundCommands;
    }

}
