/*******************************************************************************
 * Copyright (c) 2004, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.part.NullEditorInput;

import dwt.dwthelper.utils;



import dwtx.core.runtime.Assert;
import dwtx.jface.resource.ImageDescriptor;
import dwtx.ui.IEditorInput;
import dwtx.ui.IPersistableElement;
import dwtx.ui.internal.EditorReference;

/**
 * @since 3.1
 */
public class NullEditorInput implements IEditorInput {

    private EditorReference editorReference;

    /**
     * Creates a <code>NullEditorInput</code>.
     */
    public NullEditorInput() {
    }

    /**
     * Creates a <code>NullEditorInput</code> for the
     * given editor reference.
     * 
     * @param editorReference the editor reference
     * @since 3.4
     */
    public NullEditorInput(EditorReference editorReference) {
        Assert.isLegal(editorReference !is null);
        this.editorReference= editorReference;

    }

    /* (non-Javadoc)
     * @see dwtx.ui.IEditorInput#exists()
     */
    public bool exists() {
        return false;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.IEditorInput#getImageDescriptor()
     */
    public ImageDescriptor getImageDescriptor() {
        return ImageDescriptor.getMissingImageDescriptor();
    }

    /* (non-Javadoc)
     * @see dwtx.ui.IEditorInput#getName()
     */
    public String getName() {
        if (editorReference !is null)
            return editorReference.getName();
        return ""; //$NON-NLS-1$
    }

    /* (non-Javadoc)
     * @see dwtx.ui.IEditorInput#getPersistable()
     */
    public IPersistableElement getPersistable() {
        return null;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.IEditorInput#getToolTipText()
     */
    public String getToolTipText() {
        if (editorReference !is null)
            return editorReference.getTitleToolTip();
        return ""; //$NON-NLS-1$
    }

    /* (non-Javadoc)
     * @see dwtx.core.runtime.IAdaptable#getAdapter(java.lang.Class)
     */
    public Object getAdapter(Class adapter) {
        return null;
    }

}
