/*******************************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 ******************************************************************************/

module dwtx.ui.internal.preferences.SettingsTransferRegistryReader;

import dwt.dwthelper.utils;

import java.util.ArrayList;
import java.util.Collection;

import dwtx.core.runtime.IConfigurationElement;
import dwtx.core.runtime.IExtensionRegistry;
import dwtx.core.runtime.Platform;
import dwtx.ui.internal.WorkbenchPlugin;
import dwtx.ui.internal.registry.IWorkbenchRegistryConstants;
import dwtx.ui.internal.registry.RegistryReader;

/**
 * The SettingsTransferRegistryReader is the class that supplies all of the
 * transfer settings used by the settingsTransfer in the preferencesTransfer
 * extension point.
 * 
 * 
 * @since 3.3
 * 
 */
public class SettingsTransferRegistryReader extends RegistryReader {

    Collection settingsTransfers = new ArrayList();

    /**
     * Create an instance of the receiver.
     */
    public SettingsTransferRegistryReader() {

    }

    /**
     * Get all of the currently registered settings transfers.
     * 
     * @return IConfigurationElement[]
     */
    public IConfigurationElement[] getSettingTransfers() {

        settingsTransfers = new ArrayList();
        IExtensionRegistry registry = Platform.getExtensionRegistry();
        readRegistry(registry, WorkbenchPlugin.PI_WORKBENCH,
                IWorkbenchRegistryConstants.PL_PREFERENCE_TRANSFER);

        IConfigurationElement[] transfers = new IConfigurationElement[settingsTransfers
                .size()];
        settingsTransfers.toArray(transfers);
        return transfers;

    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.internal.registry.RegistryReader#readElement(dwtx.core.runtime.IConfigurationElement)
     */
    protected bool readElement(IConfigurationElement element) {
        if (element.getName().equals(
                IWorkbenchRegistryConstants.TAG_SETTINGS_TRANSFER)) {

            settingsTransfers.add(element);
            return true;
        }

        //Ignore the preference transfers
        return element.getName().equals(
                IWorkbenchRegistryConstants.TAG_TRANSFER);
    }

}
