/*******************************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 ******************************************************************************/

module dwtx.ui.internal.preferences.WorkbenchSettingsTransfer;

import dwt.dwthelper.utils;

import dwtx.core.runtime.IPath;
import dwtx.core.runtime.IStatus;
import dwtx.core.runtime.Platform;
import dwtx.core.runtime.Status;
import dwtx.ui.internal.WorkbenchMessages;
import dwtx.ui.internal.WorkbenchPlugin;
import dwtx.ui.preferences.SettingsTransfer;

/**
 * The WorkbenchSettingsTransfer is the abstract superclass of settings
 * transfers in the workbench.
 * 
 * @since 3.3
 * 
 */
public abstract class WorkbenchSettingsTransfer extends SettingsTransfer {

    /**
     * Return a status message for missing workspace settings.
     * @return IStatus
     */
    protected IStatus noWorkingSettingsStatus() {
        return new Status(IStatus.ERROR, WorkbenchPlugin.PI_WORKBENCH,
                WorkbenchMessages.WorkbenchSettings_CouldNotFindLocation);
    }

    /**
     * Return the workbench settings location for the new root 
     * @param newWorkspaceRoot
     * @return IPath or <code>null</code> if it can't be determined.
     */
    protected IPath getNewWorkbenchStateLocation(IPath newWorkspaceRoot) {
        IPath currentWorkspaceRoot = Platform.getLocation();
    
        IPath dataLocation = WorkbenchPlugin.getDefault().getDataLocation();
    
        if (dataLocation is null)
            return null;
        int segmentsToRemove = dataLocation
                .matchingFirstSegments(currentWorkspaceRoot);
    
        // Strip it down to the extension
        dataLocation = dataLocation.removeFirstSegments(segmentsToRemove);
        // Now add in the
        dataLocation = newWorkspaceRoot.append(dataLocation);
        return dataLocation;
    }

}
