/*******************************************************************************
 * Copyright (c) 2004, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.presentations.SystemMenuDetach;

import dwt.dwthelper.utils;

import dwtx.jface.action.Action;
import dwtx.ui.internal.ViewPane;
import dwtx.ui.internal.WorkbenchMessages;
import dwtx.ui.internal.WorkbenchPage;
import dwtx.ui.presentations.IPresentablePart;
import dwtx.ui.presentations.IStackPresentationSite;

public class SystemMenuDetach extends Action implements ISelfUpdatingAction {

    private ViewPane viewPane;
    private IStackPresentationSite site;
    private WorkbenchPage page;

    public SystemMenuDetach(IStackPresentationSite site) {
        this.site = site;
        setText(WorkbenchMessages.PartPane_detach);
        update();
    }
    
    public void update() {
        IPresentablePart presentablePart = site.getSelectedPart();
        setEnabled(presentablePart !is null && site.isPartMoveable(presentablePart));
        if(viewPane !is null){
            setChecked(!viewPane.isDocked());
            page = viewPane.getPage();
        }
    }
    
    public bool shouldBeVisible() {
        if(page !is null) {
            return page.getActivePerspective().getPresentation().canDetach();
        }
        return false;
    }
    
    public void dispose() {
        site = null;
    }

    public void setPane(ViewPane current){
        viewPane = current;
        update();
    }
        
    public void run() {
        if(site !is null){
            if(!isChecked()){
                viewPane.doDetach();            
            }
            else{
                viewPane.doAttach();
            }
        }
    }

}
