/*******************************************************************************
 * Copyright (c) 2004, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.presentations.SystemMenuFastViewOrientation;

import dwt.dwthelper.utils;


import dwt.DWT;
import dwt.widgets.Menu;
import dwt.widgets.MenuItem;
import dwtx.jface.action.ContributionItem;
import dwtx.ui.IViewReference;
import dwtx.ui.IWorkbenchPartReference;
import dwtx.ui.internal.FastViewBar;
import dwtx.ui.internal.IChangeListener;
import dwtx.ui.internal.IntModel;
import dwtx.ui.internal.PartPane;
import dwtx.ui.internal.RadioMenu;
import dwtx.ui.internal.ViewStackTrimToolBar;
import dwtx.ui.internal.WorkbenchMessages;
import dwtx.ui.internal.WorkbenchWindow;

/**
 * @since 3.0
 */
public class SystemMenuFastViewOrientation extends ContributionItem {

    private PartPane viewPane;

    private IntModel currentOrientation = new IntModel(DWT.VERTICAL);

    private ViewStackTrimToolBar minimizedStack = null;
    
    public SystemMenuFastViewOrientation(PartPane newViewPane) {
        this(newViewPane, null);
    }

    /**
     * @param pane
     * @param vstt
     */
    public SystemMenuFastViewOrientation(PartPane newViewPane,
            final ViewStackTrimToolBar vstt) {
        this.viewPane = newViewPane;
        this.minimizedStack = vstt;
        
        currentOrientation.addChangeListener(new IChangeListener() {
            public void update(bool changed) {
                if (changed) {
                    WorkbenchWindow workbenchWindow = (WorkbenchWindow) viewPane
                            .getPage().getWorkbenchWindow();
                    
                    if (vstt is null) {
                        FastViewBar bar = workbenchWindow.getFastViewBar();
                        if (bar !is null && viewPane !is null) {
                            IWorkbenchPartReference ref = viewPane.getPartReference();
                            
                            if (ref instanceof IViewReference) {
                                bar.setOrientation((IViewReference)ref,
                                        currentOrientation.get());
                            }
                        }
                    }
                    else {
                        vstt.setOrientation(currentOrientation.get(), workbenchWindow);
                    }
                }
            }
        });
    }

    public void dispose() {
        viewPane = null;
    }

    public void fill(Menu menu, int index) {
        WorkbenchWindow workbenchWindow = (WorkbenchWindow) viewPane.getPage()
                .getWorkbenchWindow();

        IWorkbenchPartReference ref = viewPane.getPartReference();
        if (!(ref instanceof IViewReference))
            return;
        
        if (minimizedStack is null) {
            FastViewBar bar = workbenchWindow.getFastViewBar();
            if (bar !is null && viewPane !is null) {
                currentOrientation.set(bar.getOrientation((IViewReference)ref));
            }
        }
        else {
            currentOrientation.set(minimizedStack.getPaneOrientation());
        }
        
        MenuItem orientationItem = new MenuItem(menu, DWT.CASCADE, index);
        {
            orientationItem.setText(WorkbenchMessages.FastViewBar_view_orientation);

            Menu orientationSwtMenu = new Menu(orientationItem);
            RadioMenu orientationMenu = new RadioMenu(orientationSwtMenu,
                    currentOrientation);
            orientationMenu
                    .addMenuItem(
                            WorkbenchMessages.FastViewBar_horizontal, new Integer(DWT.HORIZONTAL)); 
            orientationMenu
                    .addMenuItem(
                            WorkbenchMessages.FastViewBar_vertical, new Integer(DWT.VERTICAL)); 

            orientationItem.setMenu(orientationSwtMenu);
        }
    }

    public bool isDynamic() {
        return true;
    }
}
