/*******************************************************************************
 * Copyright (c) 2004, 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.presentations.SystemMenuMove;

import dwt.dwthelper.utils;

import dwtx.jface.action.MenuManager;
import dwtx.ui.internal.WorkbenchMessages;
import dwtx.ui.presentations.IPresentablePart;
import dwtx.ui.presentations.IStackPresentationSite;

public class SystemMenuMove extends MenuManager {

    private IStackPresentationSite stackPresentationSite;

    private SystemMenuMovePane movePaneAction;

    private SystemMenuMoveFolder moveFolderAction;
    private bool assumeActivePart = false;

    public SystemMenuMove(IStackPresentationSite stackPresentationSite,
            String partName) {
        this(stackPresentationSite, partName, true);
    }
    
    public SystemMenuMove(IStackPresentationSite stackPresentationSite,
            String partName, bool assumeActivePart) {
        super(WorkbenchMessages.PartPane_move);
        this.stackPresentationSite = stackPresentationSite;
        this.assumeActivePart = assumeActivePart;

        movePaneAction = new SystemMenuMovePane(stackPresentationSite);
        movePaneAction.setText(partName);
        moveFolderAction = new SystemMenuMoveFolder(stackPresentationSite);

        add(movePaneAction);
        add(moveFolderAction);
    }

    public void setTarget(IPresentablePart part) {
        movePaneAction.setTarget(part);
    }
    
    /* (non-Javadoc)
     * @see dwtx.jface.action.MenuManager#update(bool, bool)
     */
    protected void update(bool force, bool recursive) {
        if (assumeActivePart) {
            setTarget(stackPresentationSite.getSelectedPart());
        }
        
        moveFolderAction.update();

        super.update(force, recursive);
    }

}
