/*******************************************************************************
 * Copyright (c) 2004, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.presentations.SystemMenuPinEditor;

import dwt.dwthelper.utils;

import dwtx.jface.action.Action;
import dwtx.jface.preference.IPreferenceStore;
import dwtx.ui.internal.EditorPane;
import dwtx.ui.internal.IPreferenceConstants;
import dwtx.ui.internal.WorkbenchMessages;
import dwtx.ui.internal.WorkbenchPartReference;
import dwtx.ui.internal.WorkbenchPlugin;
import dwtx.ui.internal.tweaklets.TabBehaviour;
import dwtx.ui.internal.tweaklets.Tweaklets;

public class SystemMenuPinEditor extends Action implements ISelfUpdatingAction {

    private EditorPane editorPane;

    public SystemMenuPinEditor(EditorPane pane) {
        setText(WorkbenchMessages.EditorPane_pinEditor);
        setPane(pane);
    }

    public void dispose() {
        editorPane = null;
    }

    public void setPane(EditorPane pane) {
        editorPane = pane;
        update();
    }

    public void run() {
        WorkbenchPartReference ref = (WorkbenchPartReference)editorPane.getPartReference();

        ref.setPinned(!isChecked());
    }

    public void update() {
        if (editorPane is null) {
            setEnabled(false);
            return;
        }

        WorkbenchPartReference ref = (WorkbenchPartReference)editorPane.getPartReference();
        setEnabled(true);
        setChecked(ref.isPinned());
    }

    public bool shouldBeVisible() {
        if (editorPane is null) {
            return false;
        }

        IPreferenceStore store = WorkbenchPlugin.getDefault().getPreferenceStore();
        bool reuseEditor = store
                .getBoolean(IPreferenceConstants.REUSE_EDITORS_BOOLEAN)
                | ((TabBehaviour)Tweaklets.get(TabBehaviour.KEY)).alwaysShowPinAction();
        return reuseEditor;
    }

}
