/*******************************************************************************
 * Copyright (c) 2007, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.internal.presentations.classic.WorkbenchPresentationFactoryClassic;

import dwt.dwthelper.utils;


import dwt.DWT;
import dwt.widgets.Composite;
import dwtx.jface.preference.IPreferenceStore;
import dwtx.ui.IWorkbenchPreferenceConstants;
import dwtx.ui.PlatformUI;
import dwtx.ui.internal.presentations.defaultpresentation.DefaultSimpleTabListener;
import dwtx.ui.internal.presentations.defaultpresentation.DefaultTabFolder;
import dwtx.ui.internal.presentations.defaultpresentation.DefaultThemeListener;
import dwtx.ui.internal.presentations.util.PresentablePartFolder;
import dwtx.ui.internal.presentations.util.StandardViewSystemMenu;
import dwtx.ui.internal.presentations.util.TabbedStackPresentation;
import dwtx.ui.presentations.IStackPresentationSite;
import dwtx.ui.presentations.StackPresentation;
import dwtx.ui.presentations.WorkbenchPresentationFactory;

/**
 * The classic presentation factory for the Workbench (the 3.0 look).
 * 
 * @since 3.4
 * 
 */
public class WorkbenchPresentationFactoryClassic extends
        WorkbenchPresentationFactory {

    private static int viewTabPosition = PlatformUI.getPreferenceStore()
            .getInt(IWorkbenchPreferenceConstants.VIEW_TAB_POSITION);

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.presentations.WorkbenchPresentationFactory#createViewPresentation(dwt.widgets.Composite,
     *      dwtx.ui.presentations.IStackPresentationSite)
     */
    public StackPresentation createViewPresentation(Composite parent,
            IStackPresentationSite site) {

        DefaultTabFolder folder = new DefaultTabFolder(parent, viewTabPosition
                | DWT.BORDER, site
                .supportsState(IStackPresentationSite.STATE_MINIMIZED), site
                .supportsState(IStackPresentationSite.STATE_MAXIMIZED));

        final IPreferenceStore store = PlatformUI.getPreferenceStore();
        final int minimumCharacters = store
                .getInt(IWorkbenchPreferenceConstants.VIEW_MINIMUM_CHARACTERS);
        if (minimumCharacters >= 0) {
            folder.setMinimumCharacters(minimumCharacters);
        }

        PresentablePartFolder partFolder = new PresentablePartFolder(folder);

        folder.setUnselectedCloseVisible(false);
        folder.setUnselectedImageVisible(false);

        TabbedStackPresentation result = new TabbedStackPresentation(site,
                partFolder, new StandardViewSystemMenu(site));

        DefaultThemeListener themeListener = new DefaultThemeListener(folder,
                result.getTheme());
        result.getTheme().addListener(themeListener);

        new DefaultSimpleTabListener(result.getApiPreferences(),
                IWorkbenchPreferenceConstants.SHOW_TRADITIONAL_STYLE_TABS,
                folder);

        return result;
    }
}
