/*******************************************************************************
 * Copyright (c) 2004, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.presentations.defaultpresentation.DefaultPartList;

import dwt.dwthelper.utils;


import dwt.DWT;
import dwt.graphics.Point;
import dwt.graphics.Rectangle;
import dwt.widgets.Control;
import dwt.widgets.Event;
import dwt.widgets.Listener;
import dwt.widgets.Monitor;
import dwtx.ui.internal.presentations.BasicPartList;
import dwtx.ui.internal.presentations.util.ISystemMenu;
import dwtx.ui.internal.presentations.util.PresentablePartFolder;
import dwtx.ui.presentations.IPresentablePart;
import dwtx.ui.presentations.IStackPresentationSite;

/**
 * @since 3.1
 */
public class DefaultPartList implements ISystemMenu {

    private IStackPresentationSite site;
    private PresentablePartFolder folder;
    
    public DefaultPartList(IStackPresentationSite site, PresentablePartFolder folder) {
        this.site = site;
        this.folder = folder;
    }
    
    /* (non-Javadoc)
     * @see dwtx.ui.internal.presentations.util.ISystemMenu#show(dwt.widgets.Control, dwt.graphics.Point, dwtx.ui.presentations.IPresentablePart)
     */
    public void show(Control control, Point displayCoordinates,
            IPresentablePart currentSelection) {

        int shellStyle = DWT.RESIZE | DWT.ON_TOP | DWT.NO_TRIM;
        int tableStyle = DWT.V_SCROLL | DWT.H_SCROLL;
        final BasicPartList editorList = new BasicPartList(control.getShell(), 
                shellStyle, tableStyle, site, folder);
        editorList.setInput(folder);
        Point size = editorList.computeSizeHint();
        int x = displayCoordinates.x;
        int y = displayCoordinates.y;

        Monitor mon = folder.getTabFolder().getControl().getMonitor();
        Rectangle bounds = mon.getClientArea();
        if (x + size.x > bounds.x + bounds.width) {
            x = bounds.x + bounds.width - size.x;
        }
        if (y + size.y > bounds.y + bounds.height) {
            y = bounds.y + bounds.height - size.y;
        }
        editorList.setLocation(new Point(x, y));
        editorList.setVisible(true);
        editorList.setFocus();
        editorList.getTableViewer().getTable().getShell().addListener(
                DWT.Deactivate, new Listener() {

                    public void handleEvent(Event event) {
                        editorList.setVisible(false);
                    }
                }
            );
    }
    
    public void dispose() {
        
    }

}
