/*******************************************************************************
 * Copyright (c) 2004, 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.presentations.util.StandardSystemToolbar;

import dwt.dwthelper.utils;


import dwt.DWT;
import dwt.events.MouseAdapter;
import dwt.events.MouseEvent;
import dwt.events.MouseListener;
import dwt.events.SelectionAdapter;
import dwt.events.SelectionEvent;
import dwt.graphics.Image;
import dwt.graphics.Point;
import dwt.graphics.Rectangle;
import dwt.widgets.Composite;
import dwt.widgets.Control;
import dwt.widgets.ToolBar;
import dwt.widgets.ToolItem;
import dwtx.core.commands.common.EventManager;
import dwtx.jface.action.ContributionItem;
import dwtx.jface.action.ToolBarManager;
import dwtx.jface.util.Geometry;
import dwtx.ui.IPropertyListener;
import dwtx.ui.internal.IWorkbenchGraphicConstants;
import dwtx.ui.internal.WorkbenchImages;
import dwtx.ui.internal.WorkbenchMessages;
import dwtx.ui.presentations.IStackPresentationSite;

/**
 * @since 3.1
 */
public class StandardSystemToolbar extends EventManager {
    private ToolBarManager toolbarManager;
    private Composite control;
    
    private class SystemMenuContribution extends ContributionItem {
        ToolItem item;
        Image img;
        String text;
        int flags;
        
        public SystemMenuContribution() {
            this(DWT.PUSH);
        }
        
        public SystemMenuContribution(int flags) {
            this.flags = flags;
        }
        
        public void setToolTipText(String text) {
            this.text = text;
            if (item !is null) {
                item.setToolTipText(text);
            }
        }
        
        public void setImage(Image img) {
            this.img = img;
            
            if (item !is null) {
                item.setImage(img);
            }
        }
        
        /* (non-Javadoc)
         * @see dwtx.jface.action.ContributionItem#setVisible(bool)
         */
        public void setVisible(bool visible) {
            if (visible !is isVisible()) {
                toolbarManager.markDirty();
            }
            
            super.setVisible(visible);
        }
        
        /* (non-Javadoc)
         * @see dwtx.jface.action.ContributionItem#fill(dwt.widgets.ToolBar, int)
         */
        public void fill(ToolBar parent, int index) {
            if (!isVisible()) {
                return;
            }
            
            item = new ToolItem(parent, flags, index);
            if (img !is null) {
                item.setImage(img);
            }
            if (text !is null) {
                item.setToolTipText(text);
            }
            item.addSelectionListener(selectionListener);
        }
    }
    
    private class PaneMenu extends SystemMenuContribution {
        
        public PaneMenu() {
            super(DWT.NONE);
        }
                
        public void setToolTipText(String text) {
            super.setToolTipText(text);
        }
        
        public void setImage(Image img) {
            
            super.setImage(img);
        }
        
        /* (non-Javadoc)
         * @see dwtx.jface.action.ContributionItem#fill(dwt.widgets.ToolBar, int)
         */
        public void fill(ToolBar parent, int index) {
            if (!isVisible()) {
                return;
            }
            super.fill(parent, index);
        }
    }
    
    
    private MouseListener mouseListener = new MouseAdapter() {
        /* (non-Javadoc)
         * @see dwt.events.MouseAdapter#mouseDown(dwt.events.MouseEvent)
         */
        public void mouseDown(MouseEvent e) {
            ToolItem item = toolbarManager.getControl().getItem(new Point(e.x, e.y));
            
            if (item is paneMenu.item && e.button is 1) {
                fireEvent(TabFolderEvent.EVENT_PANE_MENU);
            }
        }
    };
    
    private SystemMenuContribution paneMenu = new PaneMenu();
    private SystemMenuContribution showToolbar = new SystemMenuContribution();
    private SystemMenuContribution min = new SystemMenuContribution();
    private SystemMenuContribution max = new SystemMenuContribution();
    private SystemMenuContribution close = new SystemMenuContribution();
    
    private int state = IStackPresentationSite.STATE_RESTORED;
    private bool showingToolbar = true;
    
    private SelectionAdapter selectionListener = new SelectionAdapter() {
        public void widgetSelected(SelectionEvent e) {
            if (e.widget is paneMenu.item) {
                //fireEvent(TabFolderEvent.EVENT_PANE_MENU);
            } else if (e.widget is showToolbar.item) {
                if (showingToolbar) {
                    fireEvent(TabFolderEvent.EVENT_HIDE_TOOLBAR);
                } else {
                    fireEvent(TabFolderEvent.EVENT_SHOW_TOOLBAR);
                }
            } else if (e.widget is min.item) {
                if (state is IStackPresentationSite.STATE_MINIMIZED) {
                    fireEvent(TabFolderEvent.EVENT_RESTORE);
                } else {
                    fireEvent(TabFolderEvent.EVENT_MINIMIZE);
                }
            } else if (e.widget is max.item) {
                if (state is IStackPresentationSite.STATE_MAXIMIZED) {
                    fireEvent(TabFolderEvent.EVENT_RESTORE);
                } else {
                    fireEvent(TabFolderEvent.EVENT_MAXIMIZE);
                }               
            } else if (e.widget is close.item) {
                fireEvent(TabFolderEvent.EVENT_CLOSE);
            }
        }
    };
    
    public StandardSystemToolbar(Composite parent, bool showPaneMenu, 
            bool showHideToolbar, bool showMinimize, bool showMaximize, bool enableClose) {
        
        control = new Composite(parent, DWT.NONE);
        control.setLayout(new EnhancedFillLayout());
        
        toolbarManager = new ToolBarManager(DWT.FLAT);
        toolbarManager.createControl(control);
        toolbarManager.getControl().addMouseListener(mouseListener);
        
        toolbarManager.add(paneMenu);
        paneMenu.setImage(WorkbenchImages.getImage(IWorkbenchGraphicConstants.IMG_LCL_VIEW_MENU_THIN));
        paneMenu.setVisible(showPaneMenu);
        paneMenu.setToolTipText(WorkbenchMessages.Menu); 

        toolbarManager.add(showToolbar);
        showToolbar.setImage(WorkbenchImages.getImage(IWorkbenchGraphicConstants.IMG_LCL_HIDE_TOOLBAR_THIN));
        showToolbar.setVisible(showHideToolbar);

        toolbarManager.add(min);
        min.setVisible(showMinimize);

        toolbarManager.add(max);
        max.setVisible(showMaximize);

        toolbarManager.add(close);
        close.setImage(WorkbenchImages.getImage(IWorkbenchGraphicConstants.IMG_LCL_CLOSE_VIEW_THIN));
        close.setVisible(enableClose);
        
        setState(IStackPresentationSite.STATE_RESTORED);
        
        toolbarManager.update(true);
    }
    
    public Point getPaneMenuLocation() {
        
        Rectangle bounds = Geometry.toDisplay(paneMenu.item.getParent(), paneMenu.item.getBounds());
        return new Point(bounds.x, bounds.y + bounds.height);
    }
    
    public void enableClose(bool enabled) {
        close.setVisible(enabled);
        toolbarManager.update(false);
    }
    
    public void enableMinimize(bool enabled) {
        min.setVisible(enabled);
        toolbarManager.update(false);
    }
    
    public void enableMaximize(bool enabled) {
        max.setVisible(enabled);
        toolbarManager.update(false);
    }
    
   public void enableShowToolbar(bool enabled) {
        showToolbar.setVisible(enabled);
        toolbarManager.update(false);
    }
    
    public void enablePaneMenu(bool enabled) {
        paneMenu.setVisible(enabled);
        toolbarManager.update(false);
    }
    
    /**
     * Updates the icons on the state buttons to match the given state
     * @param newState
     * @since 3.1
     */
    public void setState(int newState) {
        if (min !is null) {
            if (newState is IStackPresentationSite.STATE_MINIMIZED) {
                min.setToolTipText(WorkbenchMessages.StandardSystemToolbar_Restore);
                min.setImage(WorkbenchImages.getImage(IWorkbenchGraphicConstants.IMG_LCL_RESTORE_VIEW_THIN));
            } else {
                min.setToolTipText(WorkbenchMessages.StandardSystemToolbar_Minimize); 
                min.setImage(WorkbenchImages.getImage(IWorkbenchGraphicConstants.IMG_LCL_MIN_VIEW_THIN));
            }
        }
        if (max !is null) {
            if (newState is IStackPresentationSite.STATE_MAXIMIZED) {
                max.setToolTipText(WorkbenchMessages.StandardSystemToolbar_Restore);
                max.setImage(WorkbenchImages.getImage(IWorkbenchGraphicConstants.IMG_LCL_RESTORE_VIEW_THIN));
            } else {
                max.setToolTipText(WorkbenchMessages.StandardSystemToolbar_Maximize);
                max.setImage(WorkbenchImages.getImage(IWorkbenchGraphicConstants.IMG_LCL_MAX_VIEW_THIN));
            }
        }
        
        state = newState;
    }
    
    public void setToolbarShowing(bool isShowing) {
        showingToolbar = isShowing;
        if (showToolbar !is null) {
            if (isShowing) {
                showToolbar.setImage(WorkbenchImages.getImage(IWorkbenchGraphicConstants.IMG_LCL_HIDE_TOOLBAR_THIN));
            } else {
                showToolbar.setImage(WorkbenchImages.getImage(IWorkbenchGraphicConstants.IMG_LCL_SHOW_TOOLBAR_THIN));
            }
        } 
    }
    
    public void addListener(IPropertyListener propertyListener) {
        addListenerObject(propertyListener);
    }
    
    public void removeListener(IPropertyListener propertyListener) {
        removeListenerObject(propertyListener);
    }
    
    public Control getControl() {
        return control;
    }
    
    private void fireEvent(int event) {
        Object[] list = getListeners();
        
        for (int i = 0; i < list.length; i++) {
            IPropertyListener listener = (IPropertyListener)list[i];
            
            listener.propertyChanged(this, event);
        }
    }
}
