/*******************************************************************************
 * Copyright (c) 2005, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     Sebastian Davids  <sdavids@gmx.de> - Fix for Bug 132156 [Dialogs] Progress Preferences dialog problems
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.progress.JobsViewPreferenceDialog;

import dwt.dwthelper.utils;


import dwt.DWT;
import dwt.layout.GridData;
import dwt.layout.GridLayout;
import dwt.widgets.Composite;
import dwt.widgets.Control;
import dwt.widgets.Shell;
import dwtx.jface.dialogs.Dialog;
import dwtx.jface.preference.BooleanFieldEditor;
import dwtx.ui.IWorkbenchPreferenceConstants;
import dwtx.ui.internal.util.PrefUtil;
import dwtx.ui.preferences.ViewSettingsDialog;

/**
 * The JobsViewPreferenceDialog is the dialog that
 * allows the user to set the preferences.
 */
public class JobsViewPreferenceDialog extends ViewSettingsDialog {

    private BooleanFieldEditor verboseEditor;

    /**
     * Create a new instance of the receiver.
     * @param parentShell
     */
    public JobsViewPreferenceDialog(Shell parentShell) {
        super(parentShell);
    }

    /* (non-Javadoc)
     * @see dwtx.jface.window.Window#configureShell(dwt.widgets.Shell)
     */
    protected void configureShell(Shell newShell) {
        super.configureShell(newShell);
        newShell.setText(ProgressMessages.JobsViewPreferenceDialog_Title);
    }

    /* (non-Javadoc)
     * @see dwtx.jface.dialogs.Dialog#createDialogArea(dwt.widgets.Composite)
     */
    protected Control createDialogArea(Composite parent) {
        Composite top = (Composite) super.createDialogArea(parent);
        
        Composite editArea = new Composite(top, DWT.NONE);
        editArea.setLayout(new GridLayout());
        editArea.setLayoutData(new GridData(GridData.FILL_BOTH | GridData.GRAB_HORIZONTAL | GridData.GRAB_VERTICAL));
        
        verboseEditor = new BooleanFieldEditor("verbose", ProgressMessages.JobsViewPreferenceDialog_Note, editArea);//$NON-NLS-1$ 
        verboseEditor.setPreferenceName(IWorkbenchPreferenceConstants.SHOW_SYSTEM_JOBS);
        verboseEditor.setPreferenceStore(PrefUtil.getAPIPreferenceStore());
        verboseEditor.load();
        
        Dialog.applyDialogFont(top);
        
        return top;
    }
    
    /* (non-Javadoc)
     * @see dwtx.jface.dialogs.Dialog#okPressed()
     */
    protected void okPressed() {
        verboseEditor.store();
        super.okPressed();
    }
    
    /* (non-Javadoc)
     * @see dwtx.ui.internal.preferences.ViewSettingsDialog#performDefaults()
     */
    protected void performDefaults() {
        verboseEditor.loadDefault();
    }
}
