/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 ******************************************************************************/

module dwtx.ui.internal.progress.StatusAdapterHelper;

import dwt.dwthelper.utils;

import java.util.HashMap;

import dwtx.ui.progress.IProgressConstants;
import dwtx.ui.statushandlers.StatusAdapter;

/**
 * StatusAdapterHelper is a class for caching {@link StatusAdapter} instances to make sure
 * they are not created twice within the progress service.
 * @since 3.3
 */
public class StatusAdapterHelper {
    private static StatusAdapterHelper instance;

    private HashMap map;

    private StatusAdapterHelper() {
    }

    /**
     * Return the singleton.
     * @return StatusAdapterHelper
     */
    public static StatusAdapterHelper getInstance() {
        if (instance is null) {
            instance = new StatusAdapterHelper();
        }
        return instance;
    }

    /**
     * Set the {@link StatusAdapter} for the {@link JobInfo}
     * @param info
     * @param statusAdapter
     */
    public void putStatusAdapter(JobInfo info, StatusAdapter statusAdapter) {
        if (map is null) {
            map = new HashMap();
        }
        map.put(info, statusAdapter);
    }

    /**
     * Return the adapter for this info.
     * @param info
     * @return
     */
    public StatusAdapter getStatusAdapter(JobInfo info) {
        if (map is null) {
            return null;
        }
        StatusAdapter statusAdapter = (StatusAdapter) map.remove(info);
        statusAdapter.setProperty(
                IProgressConstants.NO_IMMEDIATE_ERROR_PROMPT_PROPERTY,
                Boolean.FALSE);
        return statusAdapter;
    }

    public void clear() {
        if (map !is null) {
            map.clear();
        }
    }
}
