/*******************************************************************************
 * Copyright (c) 2003, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.progress.SubTaskInfo;

import dwt.dwthelper.utils;

/**
 * SubTaskInfo is the class that displays a subtask in the 
 * tree.
 */
class SubTaskInfo extends JobTreeElement {

    protected String taskName;

    JobInfo jobInfo;

    /**
     * Create a new instance of the receiver.
     * @param parentJob
     * @param name
     */
    SubTaskInfo(JobInfo parentJob, String name) {
        taskName = name;
        jobInfo = parentJob;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.progress.JobTreeElement#getChildren()
     */
    Object[] getChildren() {
        return ProgressManagerUtil.EMPTY_OBJECT_ARRAY;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.progress.JobTreeElement#getDisplayString()
     */
    String getDisplayString() {
        if (taskName is null) {
            return ProgressMessages.SubTaskInfo_UndefinedTaskName;
        }
        return taskName;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.progress.JobTreeElement#hasChildren()
     */
    bool hasChildren() {
        return false;
    }

    /**
     * Set the taskName of the receiver.
     * @param taskName
     */
    void setTaskName(String name) {
        this.taskName = name;
    }

    /**
     * Returns the taskName of the receiver.
     */
    String getTaskName() {
        return taskName;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.progress.JobTreeElement#getParent()
     */
    Object getParent() {
        return jobInfo;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.progress.JobTreeElement#isJobInfo()
     */
    bool isJobInfo() {
        return false;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.progress.JobTreeElement#isActive()
     */
    bool isActive() {
        return jobInfo.isActive();
    }
}
