/*******************************************************************************
 * Copyright (c) 2006, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.internal.quickaccess.ActionElement;

import dwt.dwthelper.utils;

import dwtx.jface.action.ActionContributionItem;
import dwtx.jface.action.IAction;
import dwtx.jface.action.LegacyActionTools;
import dwtx.jface.resource.ImageDescriptor;

/**
 * @since 3.3
 * 
 */
public class ActionElement extends QuickAccessElement {

    private static final String separator = " - "; //$NON-NLS-1$

    private ActionContributionItem item;

    /* package */ActionElement(ActionContributionItem item, ActionProvider actionProvider) {
        super(actionProvider);
        this.item = item;
    }

    public void execute() {
        item.getAction().run();
    }

    public String getId() {
        return item.getId();
    }

    public ImageDescriptor getImageDescriptor() {
        return item.getAction().getImageDescriptor();
    }

    public String getLabel() {
        IAction action = item.getAction();
        if (action.getToolTipText() !is null
                && action.getToolTipText().length() !is 0) {
            return LegacyActionTools.removeMnemonics(action.getText()
                    + separator + action.getToolTipText());
        }
        return LegacyActionTools.removeMnemonics(action.getText());
    }

    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((item is null) ? 0 : item.hashCode());
        return result;
    }

    public bool equals(Object obj) {
        if (this is obj)
            return true;
        if (obj is null)
            return false;
        if (getClass() !is obj.getClass())
            return false;
        final ActionElement other = (ActionElement) obj;
        if (item is null) {
            if (other.item !is null)
                return false;
        } else if (!item.equals(other.item))
            return false;
        return true;
    }
}
