/*******************************************************************************
 * Copyright (c) 2006, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.internal.quickaccess.ActionProvider;

import dwt.dwthelper.utils;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import dwtx.jface.action.ActionContributionItem;
import dwtx.jface.action.IContributionItem;
import dwtx.jface.action.MenuManager;
import dwtx.jface.action.SubContributionItem;
import dwtx.jface.resource.ImageDescriptor;
import dwtx.ui.PlatformUI;
import dwtx.ui.internal.IWorkbenchGraphicConstants;
import dwtx.ui.internal.WorkbenchImages;
import dwtx.ui.internal.WorkbenchWindow;

/**
 * @since 3.3
 * 
 */
public class ActionProvider extends QuickAccessProvider {

    private Map idToElement;

    public String getId() {
        return "dwtx.ui.actions"; //$NON-NLS-1$
    }

    public QuickAccessElement getElementForId(String id) {
        getElements();
        return (ActionElement) idToElement.get(id);
    }

    public QuickAccessElement[] getElements() {
        if (idToElement is null) {
            idToElement = new HashMap();
            WorkbenchWindow window = (WorkbenchWindow) PlatformUI
                    .getWorkbench().getActiveWorkbenchWindow();
            if (window !is null) {
                MenuManager menu = window.getMenuManager();
                Set result = new HashSet();
                collectContributions(menu, result);
                ActionContributionItem[] actions = (ActionContributionItem[]) result
                        .toArray(new ActionContributionItem[result.size()]);
                for (int i = 0; i < actions.length; i++) {
                    ActionElement actionElement = new ActionElement(actions[i],
                            this);
                    idToElement.put(actionElement.getId(), actionElement);
                }
            }
        }
        return (ActionElement[]) idToElement.values().toArray(
                new ActionElement[idToElement.values().size()]);
    }

    private void collectContributions(MenuManager menu, Set result) {
        IContributionItem[] items = menu.getItems();
        for (int i = 0; i < items.length; i++) {
            IContributionItem item = items[i];
            if (item instanceof SubContributionItem) {
                item = ((SubContributionItem) item).getInnerItem();
            }
            if (item instanceof MenuManager) {
                collectContributions((MenuManager) item, result);
            } else if (item instanceof ActionContributionItem
                    && item.isEnabled()) {
                result.add(item);
            }
        }
    }

    public ImageDescriptor getImageDescriptor() {
        return WorkbenchImages
                .getImageDescriptor(IWorkbenchGraphicConstants.IMG_OBJ_NODE);
    }

    public String getName() {
        return QuickAccessMessages.QuickAccess_Menus;
    }
}
