/*******************************************************************************
 * Copyright (c) 2006, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.internal.quickaccess.CommandProvider;

import dwt.dwthelper.utils;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import dwtx.core.commands.Command;
import dwtx.core.commands.ParameterizedCommand;
import dwtx.core.commands.common.NotDefinedException;
import dwtx.core.expressions.IEvaluationContext;
import dwtx.jface.resource.ImageDescriptor;
import dwtx.ui.PlatformUI;
import dwtx.ui.commands.ICommandService;
import dwtx.ui.handlers.IHandlerService;
import dwtx.ui.internal.IWorkbenchGraphicConstants;
import dwtx.ui.internal.WorkbenchImages;

/**
 * @since 3.3
 * 
 */
public class CommandProvider extends QuickAccessProvider {

    private Map idToElement;
    private IEvaluationContext contextSnapshot;
    private IHandlerService handlerService;
    
    public CommandProvider() {
        // initialize eagerly
        saveApplicationContext();
        getElements();
    }

    public String getId() {
        return "dwtx.ui.commands"; //$NON-NLS-1$
    }

    public QuickAccessElement getElementForId(String id) {
        getElements();
        return (CommandElement) idToElement.get(id);
    }

    public QuickAccessElement[] getElements() {
        if (idToElement is null) {
            idToElement = new HashMap();
            ICommandService commandService = (ICommandService) PlatformUI
                    .getWorkbench().getService(ICommandService.class);
            final Collection commandIds = commandService.getDefinedCommandIds();
            final Iterator commandIdItr = commandIds.iterator();
            while (commandIdItr.hasNext()) {
                final String currentCommandId = (String) commandIdItr.next();
                final Command command = commandService
                        .getCommand(currentCommandId);
                if (command !is null && command.isHandled()
                        && command.isEnabled()) {
                    try {
                        Collection combinations = ParameterizedCommand
                                .generateCombinations(command);
                        for (Iterator it = combinations.iterator(); it
                                .hasNext();) {
                            ParameterizedCommand pc = (ParameterizedCommand) it.next();
                            String id = pc.serialize();
                            idToElement.put(id,
                                    new CommandElement(pc, id, this));
                        }
                    } catch (final NotDefinedException e) {
                        // It is safe to just ignore undefined commands.
                    }
                }
            }
        }
        return (QuickAccessElement[]) idToElement.values().toArray(
                new QuickAccessElement[idToElement.values().size()]);
    }

    public ImageDescriptor getImageDescriptor() {
        return WorkbenchImages
                .getImageDescriptor(IWorkbenchGraphicConstants.IMG_OBJ_NODE);
    }

    public String getName() {
        return QuickAccessMessages.QuickAccess_Commands;
    }
    
    private void saveApplicationContext() {
        handlerService = (IHandlerService) PlatformUI.getWorkbench().getService(IHandlerService.class);
        contextSnapshot = handlerService.createContextSnapshot(true);
    }
    
    IHandlerService getHandlerService() {
        return handlerService;
    }
    
    IEvaluationContext getContextSnapshot() {
        return contextSnapshot;
    }
}
