/*******************************************************************************
 * Copyright (c) 2006, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.internal.quickaccess.EditorElement;

import dwt.dwthelper.utils;

import dwtx.jface.resource.ImageDescriptor;
import dwtx.ui.IEditorReference;
import dwtx.ui.IWorkbenchPage;
import dwtx.ui.IWorkbenchPart;
import dwtx.ui.PlatformUI;

/**
 * @since 3.3
 * 
 */
public class EditorElement extends QuickAccessElement {

    private static final String DIRTY_MARK = "*"; //$NON-NLS-1$

    private static final String separator = " - "; //$NON-NLS-1$

    private IEditorReference editorReference;

    /* package */EditorElement(IEditorReference editorReference, EditorProvider editorProvider) {
        super(editorProvider);
        this.editorReference = editorReference;
    }

    public void execute() {
        IWorkbenchPart part = editorReference.getPart(true);
        if (part !is null) {
            IWorkbenchPage activePage = PlatformUI.getWorkbench()
                    .getActiveWorkbenchWindow().getActivePage();
            if (activePage !is null) {
                activePage.activate(part);
            }
        }
    }

    public String getId() {
        return editorReference.getId() + editorReference.getTitleToolTip();
    }

    public ImageDescriptor getImageDescriptor() {
        return ImageDescriptor.createFromImage(editorReference.getTitleImage());
    }

    public String getLabel() {
        bool dirty = editorReference.isDirty();
        return (dirty ? DIRTY_MARK : "") + editorReference.getTitle() + separator + editorReference.getTitleToolTip(); //$NON-NLS-1$
    }

    public String getSortLabel() {
        return editorReference.getTitle();
    }

    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result
                + ((editorReference is null) ? 0 : editorReference.hashCode());
        return result;
    }

    public bool equals(Object obj) {
        if (this is obj)
            return true;
        if (obj is null)
            return false;
        if (getClass() !is obj.getClass())
            return false;
        final EditorElement other = (EditorElement) obj;
        if (editorReference is null) {
            if (other.editorReference !is null)
                return false;
        } else if (!editorReference.equals(other.editorReference))
            return false;
        return true;
    }
}
