/*******************************************************************************
 * Copyright (c) 2006, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.internal.quickaccess.EditorProvider;

import dwt.dwthelper.utils;

import java.util.HashMap;
import java.util.Map;

import dwtx.jface.resource.ImageDescriptor;
import dwtx.ui.IEditorReference;
import dwtx.ui.IWorkbenchPage;
import dwtx.ui.PlatformUI;
import dwtx.ui.internal.IWorkbenchGraphicConstants;
import dwtx.ui.internal.WorkbenchImages;

/**
 * @since 3.3
 * 
 */
public class EditorProvider extends QuickAccessProvider {

    private Map idToElement;

    public QuickAccessElement getElementForId(String id) {
        getElements();
        return (EditorElement) idToElement.get(id);
    }

    public QuickAccessElement[] getElements() {
        if (idToElement is null) {
            idToElement = new HashMap();
            IWorkbenchPage activePage = PlatformUI.getWorkbench()
                    .getActiveWorkbenchWindow().getActivePage();
            if (activePage is null) {
                return new QuickAccessElement[0];
            }
            IEditorReference[] editors = activePage.getEditorReferences();
            for (int i = 0; i < editors.length; i++) {
                EditorElement editorElement = new EditorElement(editors[i],
                        this);
                idToElement.put(editorElement.getId(), editorElement);
            }
        }
        return (QuickAccessElement[]) idToElement.values().toArray(
                new QuickAccessElement[idToElement.values().size()]);
    }

    public String getId() {
        return "dwtx.ui.editors"; //$NON-NLS-1$
    }

    public ImageDescriptor getImageDescriptor() {
        return WorkbenchImages
                .getImageDescriptor(IWorkbenchGraphicConstants.IMG_OBJ_NODE);
    }

    public String getName() {
        return QuickAccessMessages.QuickAccess_Editors;
    }
}
