/*******************************************************************************
 * Copyright (c) 2006, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.internal.quickaccess.PerspectiveElement;

import dwt.dwthelper.utils;

import dwtx.jface.resource.ImageDescriptor;
import dwtx.ui.IPerspectiveDescriptor;
import dwtx.ui.IWorkbenchPage;
import dwtx.ui.PlatformUI;

/**
 * @since 3.3
 * 
 */
public class PerspectiveElement extends QuickAccessElement {

    private final IPerspectiveDescriptor descriptor;

    /* package */PerspectiveElement(IPerspectiveDescriptor descriptor, PerspectiveProvider perspectiveProvider) {
        super(perspectiveProvider);
        this.descriptor = descriptor;
    }

    public void execute() {
        IWorkbenchPage activePage = PlatformUI.getWorkbench()
                .getActiveWorkbenchWindow().getActivePage();
        if (activePage !is null) {
            activePage.setPerspective(descriptor);
        }
    }

    public String getId() {
        return descriptor.getId();
    }

    public ImageDescriptor getImageDescriptor() {
        return descriptor.getImageDescriptor();
    }

    public String getLabel() {
        return descriptor.getLabel();
    }

    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result
                + ((descriptor is null) ? 0 : descriptor.hashCode());
        return result;
    }

    public bool equals(Object obj) {
        if (this is obj)
            return true;
        if (obj is null)
            return false;
        if (getClass() !is obj.getClass())
            return false;
        final PerspectiveElement other = (PerspectiveElement) obj;
        if (descriptor is null) {
            if (other.descriptor !is null)
                return false;
        } else if (!descriptor.equals(other.descriptor))
            return false;
        return true;
    }
}
