/*******************************************************************************
 * Copyright (c) 2006, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.internal.quickaccess.PerspectiveProvider;

import dwt.dwthelper.utils;

import java.util.HashMap;
import java.util.Map;

import dwtx.jface.resource.ImageDescriptor;
import dwtx.ui.IPerspectiveDescriptor;
import dwtx.ui.ISharedImages;
import dwtx.ui.PlatformUI;
import dwtx.ui.internal.WorkbenchImages;

/**
 * @since 3.3
 * 
 */
public class PerspectiveProvider extends QuickAccessProvider {

    private QuickAccessElement[] cachedElements;
    private Map idToElement = new HashMap();

    public String getId() {
        return "dwtx.ui.perspectives"; //$NON-NLS-1$
    }

    public QuickAccessElement getElementForId(String id) {
        getElements();
        return (PerspectiveElement) idToElement.get(id);
    }

    public QuickAccessElement[] getElements() {
        if (cachedElements is null) {
            IPerspectiveDescriptor[] perspectives = PlatformUI.getWorkbench()
                    .getPerspectiveRegistry().getPerspectives();
            cachedElements = new QuickAccessElement[perspectives.length];
            for (int i = 0; i < perspectives.length; i++) {
                PerspectiveElement perspectiveElement = new PerspectiveElement(
                        perspectives[i], this);
                cachedElements[i] = perspectiveElement;
                idToElement.put(perspectiveElement.getId(), perspectiveElement);
            }
        }
        return cachedElements;
    }

    public ImageDescriptor getImageDescriptor() {
        return WorkbenchImages
                .getImageDescriptor(ISharedImages.IMG_ETOOL_DEF_PERSPECTIVE);
    }

    public String getName() {
        return QuickAccessMessages.QuickAccess_Perspectives;
    }
}
