/*******************************************************************************
 * Copyright (c) 2006, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.internal.quickaccess.PreferenceElement;

import dwt.dwthelper.utils;


import dwt.graphics.Image;
import dwtx.jface.preference.IPreferenceNode;
import dwtx.jface.resource.ImageDescriptor;
import dwtx.ui.IWorkbenchWindow;
import dwtx.ui.PlatformUI;
import dwtx.ui.internal.dialogs.WorkbenchPreferenceDialog;

/**
 * @since 3.3
 * 
 */
public class PreferenceElement extends QuickAccessElement {

    private static final String separator = " - "; //$NON-NLS-1$

    private IPreferenceNode preferenceNode;

    private String prefix;

    /* package */PreferenceElement(IPreferenceNode preferenceNode, String prefix, PreferenceProvider preferenceProvider) {
        super(preferenceProvider);
        this.preferenceNode = preferenceNode;
        this.prefix = prefix;
    }

    public void execute() {
        IWorkbenchWindow window = PlatformUI.getWorkbench()
                .getActiveWorkbenchWindow();
        if (window !is null) {
            WorkbenchPreferenceDialog dialog = WorkbenchPreferenceDialog
                    .createDialogOn(window.getShell(), preferenceNode.getId());
            dialog.open();
        }
    }

    public String getId() {
        return preferenceNode.getId();
    }

    public ImageDescriptor getImageDescriptor() {
        Image image = preferenceNode.getLabelImage();
        if (image !is null) {
            ImageDescriptor descriptor = ImageDescriptor.createFromImage(image);
            return descriptor;
        }
        return null;
    }

    public String getLabel() {
        if (prefix !is null && prefix.length() > 0) {
            return preferenceNode.getLabelText() + separator
                    + prefix;
        }
        return preferenceNode.getLabelText();
    }

    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result
                + ((preferenceNode is null) ? 0 : preferenceNode.hashCode());
        return result;
    }

    public bool equals(Object obj) {
        if (this is obj)
            return true;
        if (obj is null)
            return false;
        if (getClass() !is obj.getClass())
            return false;
        final PreferenceElement other = (PreferenceElement) obj;
        if (preferenceNode is null) {
            if (other.preferenceNode !is null)
                return false;
        } else if (!preferenceNode.equals(other.preferenceNode))
            return false;
        return true;
    }
}
