/*******************************************************************************
 * Copyright (c) 2006, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.internal.quickaccess.ViewElement;

import dwt.dwthelper.utils;

import dwtx.jface.resource.ImageDescriptor;
import dwtx.ui.IWorkbenchPage;
import dwtx.ui.PartInitException;
import dwtx.ui.PlatformUI;
import dwtx.ui.views.IViewDescriptor;

/**
 * @since 3.3
 * 
 */
public class ViewElement extends QuickAccessElement {

    private final IViewDescriptor viewDescriptor;

    /* package */ViewElement(IViewDescriptor viewDescriptor, ViewProvider viewProvider) {
        super(viewProvider);
        this.viewDescriptor = viewDescriptor;
    }

    public void execute() {
        IWorkbenchPage activePage = PlatformUI.getWorkbench()
                .getActiveWorkbenchWindow().getActivePage();
        if (activePage !is null) {
            try {
                activePage.showView(viewDescriptor.getId());
            } catch (PartInitException e) {
            }
        }
    }

    public String getId() {
        return viewDescriptor.getId();
    }

    public ImageDescriptor getImageDescriptor() {
        return viewDescriptor.getImageDescriptor();
    }

    public String getLabel() {
        return viewDescriptor.getLabel();
    }

    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result
                + ((viewDescriptor is null) ? 0 : viewDescriptor.hashCode());
        return result;
    }

    public bool equals(Object obj) {
        if (this is obj)
            return true;
        if (obj is null)
            return false;
        if (getClass() !is obj.getClass())
            return false;
        final ViewElement other = (ViewElement) obj;
        if (viewDescriptor is null) {
            if (other.viewDescriptor !is null)
                return false;
        } else if (!viewDescriptor.equals(other.viewDescriptor))
            return false;
        return true;
    }
}
