/*******************************************************************************
 * Copyright (c) 2006, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.internal.quickaccess.ViewProvider;

import dwt.dwthelper.utils;

import java.util.HashMap;
import java.util.Map;

import dwtx.jface.resource.ImageDescriptor;
import dwtx.ui.PlatformUI;
import dwtx.ui.internal.IWorkbenchGraphicConstants;
import dwtx.ui.internal.WorkbenchImages;
import dwtx.ui.views.IViewDescriptor;

/**
 * @since 3.3
 * 
 */
public class ViewProvider extends QuickAccessProvider {

    private QuickAccessElement[] cachedElements;
    private Map idToElement = new HashMap();

    public String getId() {
        return "dwtx.ui.views"; //$NON-NLS-1$
    }

    public QuickAccessElement getElementForId(String id) {
        getElements();
        return (ViewElement) idToElement.get(id);
    }

    public QuickAccessElement[] getElements() {
        if (cachedElements is null) {
            IViewDescriptor[] views = PlatformUI.getWorkbench()
                    .getViewRegistry().getViews();
            cachedElements = new QuickAccessElement[views.length];
            for (int i = 0; i < views.length; i++) {
                ViewElement viewElement = new ViewElement(views[i], this);
                cachedElements[i] = viewElement;
                idToElement.put(viewElement.getId(), viewElement);
            }
        }
        return cachedElements;
    }

    public ImageDescriptor getImageDescriptor() {
        return WorkbenchImages
                .getImageDescriptor(IWorkbenchGraphicConstants.IMG_VIEW_DEFAULTVIEW_MISC);
    }

    public String getName() {
        return QuickAccessMessages.QuickAccess_Views;
    }
}
