/*******************************************************************************
 * Copyright (c) 2006, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.internal.quickaccess.WizardElement;

import dwt.dwthelper.utils;

import dwtx.jface.resource.ImageDescriptor;
import dwtx.ui.IWorkbenchWindow;
import dwtx.ui.PlatformUI;
import dwtx.ui.internal.actions.NewWizardShortcutAction;
import dwtx.ui.wizards.IWizardDescriptor;

/**
 * @since 3.3
 * 
 */
public class WizardElement extends QuickAccessElement {

    private static final String separator = " - "; //$NON-NLS-1$

    private IWizardDescriptor wizardDescriptor;

    /* package */WizardElement(IWizardDescriptor wizardDescriptor, WizardProvider wizardProvider) {
        super(wizardProvider);
        this.wizardDescriptor = wizardDescriptor;
    }

    public void execute() {
        IWorkbenchWindow window = PlatformUI.getWorkbench()
                .getActiveWorkbenchWindow();
        if (window !is null) {
            NewWizardShortcutAction wizardAction = new NewWizardShortcutAction(
                    window, wizardDescriptor);
            wizardAction.run();
        }
    }

    public String getId() {
        return wizardDescriptor.getId();
    }

    public ImageDescriptor getImageDescriptor() {
        return wizardDescriptor.getImageDescriptor();
    }

    public String getLabel() {
        return wizardDescriptor.getLabel() + separator
                + wizardDescriptor.getDescription();
    }

    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime
                * result
                + ((wizardDescriptor is null) ? 0 : wizardDescriptor.hashCode());
        return result;
    }

    public bool equals(Object obj) {
        if (this is obj)
            return true;
        if (obj is null)
            return false;
        if (getClass() !is obj.getClass())
            return false;
        final WizardElement other = (WizardElement) obj;
        if (wizardDescriptor is null) {
            if (other.wizardDescriptor !is null)
                return false;
        } else if (!wizardDescriptor.equals(other.wizardDescriptor))
            return false;
        return true;
    }
}
