/*******************************************************************************
 * Copyright (c) 2004, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.internal.registry.ShowViewHandler;

import dwt.dwthelper.utils;

import dwtx.core.commands.AbstractHandler;
import dwtx.core.commands.ExecutionEvent;
import dwtx.core.commands.ExecutionException;
import dwtx.core.runtime.IStatus;
import dwtx.ui.IWorkbenchPage;
import dwtx.ui.IWorkbenchWindow;
import dwtx.ui.PartInitException;
import dwtx.ui.handlers.HandlerUtil;
import dwtx.ui.internal.misc.StatusUtil;
import dwtx.ui.statushandlers.StatusManager;

/**
 * Command handler to show a particular view.
 * 
 * @since 3.0
 */
public final class ShowViewHandler extends AbstractHandler {
    
    /**
     * The identifier of the view this handler should open. This value should
     * never be <code>null</code>.
     */
    private final String viewId;

    /**
     * Constructs a new instance of <code>ShowViewHandler</code>.
     * 
     * @param viewId
     *            The identifier of the view this handler should open; must not
     *            be <code>null</code>.
     */
    public ShowViewHandler(final String viewId) {
        this.viewId = viewId;
    }

    public final Object execute(final ExecutionEvent event) 
            throws ExecutionException {
        final IWorkbenchWindow activeWorkbenchWindow = HandlerUtil
                .getActiveWorkbenchWindowChecked(event);

        final IWorkbenchPage activePage = activeWorkbenchWindow.getActivePage();
        if (activePage is null) {
            return null;
        }

        try {
            activePage.showView(viewId);
        } catch (PartInitException e) {
            IStatus status = StatusUtil
                    .newStatus(e.getStatus(), e.getMessage());
            StatusManager.getManager().handle(status, StatusManager.SHOW);
        }

        return null;
    }
}
