/*******************************************************************************
 * Copyright (c) 2004, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     Jan-Hendrik Diederich, Bredex GmbH - bug 201052
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.registry.StickyViewDescriptor;

import dwt.dwthelper.utils;

import dwtx.core.runtime.CoreException;
import dwtx.core.runtime.IConfigurationElement;
import dwtx.core.runtime.IStatus;
import dwtx.core.runtime.Status;
import dwtx.ui.IPageLayout;
import dwtx.ui.IPluginContribution;
import dwtx.ui.views.IStickyViewDescriptor;

/**
 * @since 3.0
 */
public class StickyViewDescriptor implements IStickyViewDescriptor, 
    IPluginContribution {

    private IConfigurationElement configurationElement;

    private String id;
    
    /**
     * Folder constant for right sticky views.
     */
    public static final String STICKY_FOLDER_RIGHT = "stickyFolderRight"; //$NON-NLS-1$

    /**
     * Folder constant for left sticky views.
     */
    public static final String STICKY_FOLDER_LEFT = "stickyFolderLeft"; //$NON-NLS-1$

    /**
     * Folder constant for top sticky views.
     */
    public static final String STICKY_FOLDER_TOP = "stickyFolderTop"; //$NON-NLS-1$

    /**
     * Folder constant for bottom sticky views.
     */ 
    public static final String STICKY_FOLDER_BOTTOM = "stickyFolderBottom"; //$NON-NLS-1$

    /**
     * @param element
     * @throws CoreException
     */
    public StickyViewDescriptor(IConfigurationElement element)
            throws CoreException {
        this.configurationElement = element;
        id = configurationElement.getAttribute(IWorkbenchRegistryConstants.ATT_ID);
        if (id is null) {
            throw new CoreException(new Status(IStatus.ERROR, element
                    .getNamespace(), 0,
                    "Invalid extension (missing id) ", null));//$NON-NLS-1$
        }
    }
    
    /**
     * Return the configuration element.
     * 
     * @return the configuration element
     */
    public IConfigurationElement getConfigurationElement() {
        return configurationElement;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.views.IStickyViewDescriptor#getLocation()
     */
    public int getLocation() {
        int direction = IPageLayout.RIGHT;
        
        String location = configurationElement.getAttribute(IWorkbenchRegistryConstants.ATT_LOCATION);
        if (location !is null) {
            if (location.equalsIgnoreCase("left")) { //$NON-NLS-1$
                direction = IPageLayout.LEFT;
            } else if (location.equalsIgnoreCase("top")) { //$NON-NLS-1$
                direction = IPageLayout.TOP;
            } else if (location.equalsIgnoreCase("bottom")) { //$NON-NLS-1$
                direction = IPageLayout.BOTTOM;
            //no else for right - it is the default value;
            }
        }       
        return direction;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.registry.IStickyViewDescriptor#getId()
     */
    public String getId() {
        return id;
    }
    
    public String getLocalId() {
        return id;
    }

    public String getPluginId() {
        return configurationElement.getContributor().getName();
    }
    

    /* (non-Javadoc)
     * @see dwtx.ui.internal.registry.IStickyViewDescriptor#isFixed()
     */
    public bool isCloseable() {
        bool closeable = true;
        String closeableString = configurationElement.getAttribute(IWorkbenchRegistryConstants.ATT_CLOSEABLE);
        if (closeableString !is null) {
            closeable = !closeableString.equals("false"); //$NON-NLS-1$
        }
        return closeable;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.registry.IStickyViewDescriptor#isMoveable()
     */
    public bool isMoveable() {
        bool moveable = true;
        String moveableString = configurationElement.getAttribute(IWorkbenchRegistryConstants.ATT_MOVEABLE);
        if (moveableString !is null) {
            moveable = !moveableString.equals("false"); //$NON-NLS-1$
        }       
        return moveable;
    }
}
