/*******************************************************************************
 * Copyright (c) 2007, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.internal.services.EvaluationService;

import dwt.dwthelper.utils;


import dwt.widgets.Shell;
import dwtx.core.expressions.Expression;
import dwtx.core.expressions.IEvaluationContext;
import dwtx.core.runtime.Assert;
import dwtx.jface.util.IPropertyChangeListener;
import dwtx.ui.ISourceProvider;
import dwtx.ui.ISources;
import dwtx.ui.services.IEvaluationReference;
import dwtx.ui.services.IEvaluationService;

/**
 * @since 3.3
 * 
 */
public final class EvaluationService implements IEvaluationService {

    private EvaluationAuthority evaluationAuthority;

    public EvaluationService() {
        evaluationAuthority = new EvaluationAuthority();
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.internal.services.IEvaluationService#addEvaluationListener(dwtx.core.expressions.Expression,
     *      dwtx.jface.util.IPropertyChangeListener, java.lang.String)
     */
    public IEvaluationReference addEvaluationListener(Expression expression,
            IPropertyChangeListener listener, String property) {
        IEvaluationReference expressionReference = new EvaluationReference(
                expression, listener, property);

        evaluationAuthority.addEvaluationListener(expressionReference);
        return expressionReference;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.services.IEvaluationService#addEvaluationReference(dwtx.ui.services.IEvaluationReference)
     */
    public void addEvaluationReference(IEvaluationReference ref) {
        Assert.isLegal(ref instanceof EvaluationReference, "Invalid type: " //$NON-NLS-1$
                + ref.getClass().getName());
        evaluationAuthority.addEvaluationListener(ref);
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.internal.services.IEvaluationService#removeEvaluationListener(dwtx.ui.internal.services.IEvaluationReference)
     */
    public void removeEvaluationListener(IEvaluationReference ref) {
        evaluationAuthority.removeEvaluationListener(ref);
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.services.IServiceWithSources#addSourceProvider(dwtx.ui.ISourceProvider)
     */
    public void addSourceProvider(ISourceProvider provider) {
        evaluationAuthority.addSourceProvider(provider);
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.services.IServiceWithSources#removeSourceProvider(dwtx.ui.ISourceProvider)
     */
    public void removeSourceProvider(ISourceProvider provider) {
        evaluationAuthority.removeSourceProvider(provider);
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.services.IDisposable#dispose()
     */
    public void dispose() {
        evaluationAuthority.dispose();
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.internal.services.IEvaluationService#getCurrentState()
     */
    public IEvaluationContext getCurrentState() {
        return evaluationAuthority.getCurrentState();
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.internal.services.IEvaluationService#addServiceListener(dwtx.jface.util.IPropertyChangeListener)
     */
    public void addServiceListener(IPropertyChangeListener listener) {
        evaluationAuthority.addServiceListener(listener);
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.internal.services.IEvaluationService#removeServiceListener(dwtx.jface.util.IPropertyChangeListener)
     */
    public void removeServiceListener(IPropertyChangeListener listener) {
        evaluationAuthority.removeServiceListener(listener);
    }
    
    /* (non-Javadoc)
     * @see dwtx.ui.services.IEvaluationService#requestEvaluation(java.lang.String)
     */
    public void requestEvaluation(String propertyName) {
        evaluationAuthority.sourceChanged(new String[] { propertyName });
    }

    public void updateShellKludge() {
        evaluationAuthority.updateShellKludge();
    }

    /**
     * <p>
     * Bug 95792. A mechanism by which the key binding architecture can force an
     * update of the handlers (based on the active shell) before trying to
     * execute a command. This mechanism is required for GTK+ only.
     * </p>
     * <p>
     * DO NOT CALL THIS METHOD.
     * </p>
     * 
     * @param shell
     *            The shell that should be considered active; must not be
     *            <code>null</code>.
     */
    public final void updateShellKludge(final Shell shell) {
        final Shell currentActiveShell = evaluationAuthority.getActiveShell();
        if (currentActiveShell !is shell) {
            evaluationAuthority.sourceChanged(ISources.ACTIVE_SHELL,
                    ISources.ACTIVE_SHELL_NAME, shell);
        }
    }
}
