/*******************************************************************************
 * Copyright (c) 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 ******************************************************************************/

module dwtx.ui.internal.services.SlaveEvaluationService;

import dwt.dwthelper.utils;

import java.util.ArrayList;
import java.util.Collection;

import dwtx.core.expressions.Expression;
import dwtx.core.expressions.IEvaluationContext;
import dwtx.jface.util.IPropertyChangeListener;
import dwtx.ui.ISourceProvider;
import dwtx.ui.services.IEvaluationReference;
import dwtx.ui.services.IEvaluationService;

/**
 * @since 3.4
 * 
 */
public class SlaveEvaluationService implements IEvaluationService {

    private IEvaluationService parentService;

    private Collection sourceProviders = new ArrayList();

    private Collection serviceListeners = new ArrayList();

    private Collection evaluationReferences = new ArrayList();

    /**
     * @param parent
     */
    public SlaveEvaluationService(IEvaluationService parent) {
        parentService = parent;
    }

    /**
     * @see dwtx.ui.services.IEvaluationService#addEvaluationListener(dwtx.core.expressions.Expression,
     *      dwtx.jface.util.IPropertyChangeListener, java.lang.String)
     */
    public IEvaluationReference addEvaluationListener(Expression expression,
            IPropertyChangeListener listener, String property) {
        IEvaluationReference ref = parentService.addEvaluationListener(
                expression, listener, property);
        if (!evaluationReferences.contains(ref)) {
            evaluationReferences.add(ref);
        }
        return ref;
    }
    
    /* (non-Javadoc)
     * @see dwtx.ui.services.IEvaluationService#addEvaluationReference(dwtx.ui.services.IEvaluationReference)
     */
    public void addEvaluationReference(IEvaluationReference ref) {
        if (!evaluationReferences.contains(ref)) {
            evaluationReferences.add(ref);
        }
        parentService.addEvaluationReference(ref);
    }

    /**
     * @see dwtx.ui.services.IEvaluationService#addServiceListener(dwtx.jface.util.IPropertyChangeListener)
     */
    public void addServiceListener(IPropertyChangeListener listener) {
        if (!serviceListeners.contains(listener)) {
            serviceListeners.add(listener);
        }
        parentService.addServiceListener(listener);
    }

    /**
     * @see dwtx.ui.services.IServiceWithSources#addSourceProvider(dwtx.ui.ISourceProvider)
     */
    public void addSourceProvider(ISourceProvider provider) {
        if (!sourceProviders.contains(provider)) {
            sourceProviders.add(provider);
        }
        parentService.addSourceProvider(provider);
    }

    /**
     * @see dwtx.ui.services.IEvaluationService#getCurrentState()
     */
    public IEvaluationContext getCurrentState() {
        return parentService.getCurrentState();
    }

    /**
     * @see dwtx.ui.services.IEvaluationService#removeEvaluationListener(dwtx.ui.services.IEvaluationReference)
     */
    public void removeEvaluationListener(IEvaluationReference ref) {
        evaluationReferences.remove(ref);
        parentService.removeEvaluationListener(ref);
    }

    /**
     * @see dwtx.ui.services.IEvaluationService#removeServiceListener(dwtx.jface.util.IPropertyChangeListener)
     */
    public void removeServiceListener(IPropertyChangeListener listener) {
        serviceListeners.remove(listener);
        parentService.removeServiceListener(listener);
    }

    /**
     * @see dwtx.ui.services.IServiceWithSources#removeSourceProvider(dwtx.ui.ISourceProvider)
     */
    public void removeSourceProvider(ISourceProvider provider) {
        sourceProviders.remove(provider);
        parentService.removeSourceProvider(provider);
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.services.IDisposable#dispose()
     */
    public void dispose() {
        if (!evaluationReferences.isEmpty()) {
            Object[] array = evaluationReferences.toArray();
            for (int i = 0; i < array.length; i++) {
                parentService
                        .removeEvaluationListener((IEvaluationReference) array[i]);
            }
        }
        if (!serviceListeners.isEmpty()) {
            Object[] array = serviceListeners.toArray();
            for (int i = 0; i < array.length; i++) {
                parentService
                        .removeServiceListener((IPropertyChangeListener) array[i]);
            }
            serviceListeners.clear();
        }
        // Remove any "resource", like listeners, that were associated
        // with this service.
        if (!sourceProviders.isEmpty()) {
            Object[] array = sourceProviders.toArray();
            for (int i = 0; i < array.length; i++) {
                parentService.removeSourceProvider((ISourceProvider) array[i]);
            }
            sourceProviders.clear();
        }
    }

    /* (non-Javadoc)
     * @see dwtx.ui.services.IEvaluationService#requestEvaluation(java.lang.String)
     */
    public void requestEvaluation(String propertyName) {
        parentService.requestEvaluation(propertyName);
    }
}
