/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 ******************************************************************************/

module dwtx.ui.internal.statushandlers.StatusHandlerProductBindingDescriptor;

import dwt.dwthelper.utils;

import dwtx.core.runtime.IConfigurationElement;
import dwtx.ui.IPluginContribution;
import dwtx.ui.internal.registry.IWorkbenchRegistryConstants;

/**
 * The status handler product binding descriptor.
 * 
 * @since 3.3
 */
class StatusHandlerProductBindingDescriptor implements
        IPluginContribution {
    
    /**
     * Handler id attribute. Value <code>handlerId</code>.
     */
    private static String ATT_HANDLER_ID = "handlerId"; //$NON-NLS-1$

    private String id;

    private String pluginId;

    private String productId;

    private String handlerId;

    /**
     * @param configElement
     */
    public StatusHandlerProductBindingDescriptor(
            IConfigurationElement configElement) {
        super();
        id = configElement.getAttribute(IWorkbenchRegistryConstants.ATT_ID);
        pluginId = configElement.getContributor().getName();
        productId = configElement.getAttribute(IWorkbenchRegistryConstants.ATT_PRODUCTID);
        handlerId = configElement.getAttribute(ATT_HANDLER_ID);
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.IPluginContribution#getLocalId()
     */
    public String getLocalId() {
        return id;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.IPluginContribution#getPluginId()
     */
    public String getPluginId() {
        return pluginId;
    }

    /**
     * @return Returns the productId.
     */
    public String getProductId() {
        return productId;
    }

    /**
     * @return Returns the handlerId.
     */
    public String getHandlerId() {
        return handlerId;
    }
}
