/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 ******************************************************************************/

module dwtx.ui.internal.tweaklets.TabBehaviour;

import dwt.dwthelper.utils;


import dwt.widgets.Button;
import dwt.widgets.Composite;
import dwtx.jface.preference.IPreferenceStore;
import dwtx.ui.IEditorInput;
import dwtx.ui.IEditorReference;
import dwtx.ui.internal.EditorAreaHelper;
import dwtx.ui.internal.EditorManager;
import dwtx.ui.internal.IPreferenceConstants;
import dwtx.ui.internal.WorkbenchPage;
import dwtx.ui.internal.WorkbenchPlugin;
import dwtx.ui.internal.registry.EditorDescriptor;
import dwtx.ui.internal.tweaklets.Tweaklets.TweakKey;

/**
 * @since 3.3
 * 
 */
public abstract class TabBehaviour {

    public static TweakKey KEY = new Tweaklets.TweakKey(TabBehaviour.class);

    static {
        Tweaklets.setDefault(TabBehaviour.KEY, new TabBehaviourMRU());
    }

    /**
     * 
     * @return
     */
    public abstract bool alwaysShowPinAction();

    /**
     * 
     * @param page
     * @return
     */
    public abstract IEditorReference findReusableEditor(WorkbenchPage page);

    public abstract IEditorReference reuseInternalEditor(WorkbenchPage page,
            EditorManager manager, EditorAreaHelper editorPresentation,
            EditorDescriptor desc, IEditorInput input,
            IEditorReference reusableEditorRef);

    /**
     * Does nothing by default. Can be overridden by subclasses. 
     * 
     * @param editorReuseGroup
     * @param showMultipleEditorTabs
     */
    public void setPreferenceVisibility(Composite editorReuseGroup,
            Button showMultipleEditorTabs) {
    }

    /**
     * @return
     */
    public bool autoPinOnDirty() {
        return false;
    }

    /**
     * @return
     */
    public bool isPerTabHistoryEnabled() {
        return false;
    }

    /**
     * @param originalMatchFlags
     * @return
     */
    public int getReuseEditorMatchFlags(int originalMatchFlags) {
        return originalMatchFlags;
    }

    /**
     * @return
     */
    public int getEditorReuseThreshold() {
        IPreferenceStore store = WorkbenchPlugin.getDefault()
                .getPreferenceStore();
        return store.getInt(IPreferenceConstants.REUSE_EDITORS);
    }

    /**
     * @return
     */
    public bool enableMRUTabVisibility() {
        return true;
    }

}
