/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 ******************************************************************************/

module dwtx.ui.internal.tweaklets.TabBehaviourMRU;

import dwt.dwthelper.utils;

import dwtx.jface.dialogs.IDialogConstants;
import dwtx.jface.dialogs.MessageDialog;
import dwtx.jface.dialogs.ProgressMonitorDialog;
import dwtx.osgi.util.NLS;
import dwtx.ui.IEditorInput;
import dwtx.ui.IEditorPart;
import dwtx.ui.IEditorReference;
import dwtx.ui.IReusableEditor;
import dwtx.ui.internal.EditorAreaHelper;
import dwtx.ui.internal.EditorManager;
import dwtx.ui.internal.EditorReference;
import dwtx.ui.internal.EditorSite;
import dwtx.ui.internal.IPreferenceConstants;
import dwtx.ui.internal.Workbench;
import dwtx.ui.internal.WorkbenchMessages;
import dwtx.ui.internal.WorkbenchPage;
import dwtx.ui.internal.WorkbenchPlugin;
import dwtx.ui.internal.progress.ProgressMonitorJobsDialog;
import dwtx.ui.internal.registry.EditorDescriptor;

/**
 * @since 3.3
 * 
 */
public class TabBehaviourMRU extends TabBehaviour {

    public bool alwaysShowPinAction() {
        return false;
    }

    public IEditorReference findReusableEditor(WorkbenchPage page) {
        bool reuse = WorkbenchPlugin.getDefault().getPreferenceStore()
                .getBoolean(IPreferenceConstants.REUSE_EDITORS_BOOLEAN);
        if (!reuse) {
            return null;
        }

        IEditorReference editors[] = page.getSortedEditors();
        if (editors.length < page.getEditorReuseThreshold()) {
            return null;
        }

        IEditorReference dirtyEditor = null;

        // Find a editor to be reused
        for (int i = 0; i < editors.length; i++) {
            IEditorReference editor = editors[i];
            // if(editor is activePart)
            // continue;
            if (editor.isPinned()) {
                continue;
            }
            if (editor.isDirty()) {
                if (dirtyEditor is null) {
                    dirtyEditor = editor;
                }
                continue;
            }
            return editor;
        }
        if (dirtyEditor is null) {
            return null;
        }

        /* fix for 11122 */
        bool reuseDirty = WorkbenchPlugin.getDefault().getPreferenceStore()
                .getBoolean(IPreferenceConstants.REUSE_DIRTY_EDITORS);
        if (!reuseDirty) {
            return null;
        }

        MessageDialog dialog = new MessageDialog(page.getWorkbenchWindow()
                .getShell(),
                WorkbenchMessages.EditorManager_reuseEditorDialogTitle,
                null, // accept the default window icon
                NLS.bind(WorkbenchMessages.EditorManager_saveChangesQuestion,
                        dirtyEditor.getName()), MessageDialog.QUESTION,
                new String[] { IDialogConstants.YES_LABEL,
                        IDialogConstants.NO_LABEL,
                        WorkbenchMessages.EditorManager_openNewEditorLabel }, 0);
        int result = dialog.open();
        if (result is 0) { // YES
            ProgressMonitorDialog pmd = new ProgressMonitorJobsDialog(dialog
                    .getShell());
            pmd.open();
            dirtyEditor.getEditor(true).doSave(pmd.getProgressMonitor());
            pmd.close();
        } else if ((result is 2) || (result is -1)) {
            return null;
        }
        return dirtyEditor;
    }

    public IEditorReference reuseInternalEditor(WorkbenchPage page,
            EditorManager manager, EditorAreaHelper editorPresentation,
            EditorDescriptor desc, IEditorInput input,
            IEditorReference reusableEditorRef) {
        IEditorPart reusableEditor = reusableEditorRef.getEditor(false);
        if (reusableEditor is null) {
            IEditorReference result = new EditorReference(manager, input, desc);
            page.closeEditor(reusableEditorRef, false);
            return result;
        }

        EditorSite site = (EditorSite) reusableEditor.getEditorSite();
        EditorDescriptor oldDesc = site.getEditorDescriptor();
        if ((desc.getId().equals(oldDesc.getId()))
                && (reusableEditor instanceof IReusableEditor)) {
            Workbench wb = (Workbench) page.getWorkbenchWindow().getWorkbench();
            editorPresentation.moveEditor(reusableEditor, -1);
            wb.getEditorHistory().add(reusableEditor.getEditorInput(),
                    site.getEditorDescriptor());
            page.reuseEditor((IReusableEditor) reusableEditor, input);
            return reusableEditorRef;
        }
        // findReusableEditor(...) checks pinned and saves editor if
        // necessary, so it's OK to close "reusableEditor"
        IEditorReference ref = new EditorReference(manager, input, desc);
        reusableEditor.getEditorSite().getPage().closeEditor(reusableEditor,
                false);
        return ref;
    }

}
