/*******************************************************************************
 * Copyright (c) 2003, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.util.BundleUtility;

import dwt.dwthelper.utils;

import java.net.URL;

import org.osgi.framework.Bundle;

import dwtx.core.runtime.IStatus;
import dwtx.core.runtime.Path;
import dwtx.core.runtime.Platform;
import dwtx.core.runtime.Status;
import dwtx.ui.internal.WorkbenchPlugin;

/**
 * A set of static methods that provide an nicer interface to common platform
 * operations related to bundle management.
 */
public class BundleUtility {
    public static bool isActive(Bundle bundle) {
        if (bundle is null) {
            return false;
        }
        return bundle.getState() is Bundle.ACTIVE;
    }

    public static bool isActivated(Bundle bundle) {
        if ((bundle.getState() & Bundle.STARTING) !is 0)
            return WorkbenchPlugin.getDefault().isStarting(bundle);
        return bundle !is null && (bundle.getState() & (Bundle.ACTIVE | Bundle.STOPPING)) !is 0;
    }

    // TODO: needs a better name
    public static bool isReady(Bundle bundle) {
        return bundle !is null && isReady(bundle.getState());
    }

    public static bool isReady(int bundleState) {
        return (bundleState & (Bundle.RESOLVED | Bundle.STARTING | Bundle.ACTIVE | Bundle.STOPPING)) !is 0;
    }

    public static bool isActive(String bundleId) {
        return isActive(Platform.getBundle(bundleId));
    }

    public static bool isActivated(String bundleId) {
        return isActivated(Platform.getBundle(bundleId));
    }

    public static bool isReady(String bundleId) {
        return isReady(Platform.getBundle(bundleId));
    }

    public static URL find(Bundle bundle, String path) {
        if (bundle is null) {
            return null;
        }
        return Platform.find(bundle, new Path(path));
    }

    public static URL find(String bundleId, String path) {
        return find(Platform.getBundle(bundleId), path);
    }

    public static void log(String bundleId, Throwable exception) {
        Bundle bundle = Platform.getBundle(bundleId);
        if (bundle is null) {
            return;
        }
        IStatus status = new Status(IStatus.ERROR, bundleId, IStatus.ERROR,
                exception.getMessage() is null ? "" : exception.getMessage(), //$NON-NLS-1$
                exception);
        Platform.getLog(bundle).log(status);
    }
}
