/*******************************************************************************
 * Copyright (c) 2000, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.keys.SWTKeySupport;

import dwt.dwthelper.utils;

import java.util.Iterator;
import java.util.SortedSet;
import java.util.TreeSet;

import dwt.DWT;
import dwt.events.KeyEvent;
import dwt.widgets.Event;
import dwtx.ui.internal.keys.NativeKeyFormatter;

/**
 * A utility class for converting DWT events into key strokes.
 * 
 * @deprecated Please use dwtx.jface.bindings.keys.SWTKeySupport
 * @since 3.0
 */
public final class SWTKeySupport {

    /**
     * Given an DWT accelerator value, provide the corresponding key stroke.
     * 
     * @param accelerator
     *            The accelerator to convert; should be a valid DWT accelerator
     *            value.
     * @return The equivalent key stroke; never <code>null</code>.
     */
    public static KeyStroke convertAcceleratorToKeyStroke(int accelerator) {
        final SortedSet modifierKeys = new TreeSet();
        NaturalKey naturalKey = null;

        if ((accelerator & DWT.ALT) !is 0) {
            modifierKeys.add(ModifierKey.ALT);
        }

        if ((accelerator & DWT.COMMAND) !is 0) {
            modifierKeys.add(ModifierKey.COMMAND);
        }

        if ((accelerator & DWT.CTRL) !is 0) {
            modifierKeys.add(ModifierKey.CTRL);
        }

        if ((accelerator & DWT.SHIFT) !is 0) {
            modifierKeys.add(ModifierKey.SHIFT);
        }

        if (((accelerator & DWT.KEY_MASK) is 0) && (accelerator !is 0)) {
            // There were only accelerators
            naturalKey = null;
        } else {
            // There were other keys.
            accelerator &= DWT.KEY_MASK;

            switch (accelerator) {
            case DWT.ARROW_DOWN:
                naturalKey = SpecialKey.ARROW_DOWN;
                break;
            case DWT.ARROW_LEFT:
                naturalKey = SpecialKey.ARROW_LEFT;
                break;
            case DWT.ARROW_RIGHT:
                naturalKey = SpecialKey.ARROW_RIGHT;
                break;
            case DWT.ARROW_UP:
                naturalKey = SpecialKey.ARROW_UP;
                break;
            case DWT.BREAK:
                naturalKey = SpecialKey.BREAK;
                break;
            case DWT.CAPS_LOCK:
                naturalKey = SpecialKey.CAPS_LOCK;
                break;
            case DWT.END:
                naturalKey = SpecialKey.END;
                break;
            case DWT.F1:
                naturalKey = SpecialKey.F1;
                break;
            case DWT.F10:
                naturalKey = SpecialKey.F10;
                break;
            case DWT.F11:
                naturalKey = SpecialKey.F11;
                break;
            case DWT.F12:
                naturalKey = SpecialKey.F12;
                break;
            case DWT.F2:
                naturalKey = SpecialKey.F2;
                break;
            case DWT.F3:
                naturalKey = SpecialKey.F3;
                break;
            case DWT.F4:
                naturalKey = SpecialKey.F4;
                break;
            case DWT.F5:
                naturalKey = SpecialKey.F5;
                break;
            case DWT.F6:
                naturalKey = SpecialKey.F6;
                break;
            case DWT.F7:
                naturalKey = SpecialKey.F7;
                break;
            case DWT.F8:
                naturalKey = SpecialKey.F8;
                break;
            case DWT.F9:
                naturalKey = SpecialKey.F9;
                break;
            case DWT.HOME:
                naturalKey = SpecialKey.HOME;
                break;
            case DWT.INSERT:
                naturalKey = SpecialKey.INSERT;
                break;
            case DWT.KEYPAD_0:
                naturalKey = SpecialKey.NUMPAD_0;
                break;
            case DWT.KEYPAD_1:
                naturalKey = SpecialKey.NUMPAD_1;
                break;
            case DWT.KEYPAD_2:
                naturalKey = SpecialKey.NUMPAD_2;
                break;
            case DWT.KEYPAD_3:
                naturalKey = SpecialKey.NUMPAD_3;
                break;
            case DWT.KEYPAD_4:
                naturalKey = SpecialKey.NUMPAD_4;
                break;
            case DWT.KEYPAD_5:
                naturalKey = SpecialKey.NUMPAD_5;
                break;
            case DWT.KEYPAD_6:
                naturalKey = SpecialKey.NUMPAD_6;
                break;
            case DWT.KEYPAD_7:
                naturalKey = SpecialKey.NUMPAD_7;
                break;
            case DWT.KEYPAD_8:
                naturalKey = SpecialKey.NUMPAD_8;
                break;
            case DWT.KEYPAD_9:
                naturalKey = SpecialKey.NUMPAD_9;
                break;
            case DWT.KEYPAD_ADD:
                naturalKey = SpecialKey.NUMPAD_ADD;
                break;
            case DWT.KEYPAD_CR:
                naturalKey = SpecialKey.NUMPAD_ENTER;
                break;
            case DWT.KEYPAD_DECIMAL:
                naturalKey = SpecialKey.NUMPAD_DECIMAL;
                break;
            case DWT.KEYPAD_DIVIDE:
                naturalKey = SpecialKey.NUMPAD_DIVIDE;
                break;
            case DWT.KEYPAD_EQUAL:
                naturalKey = SpecialKey.NUMPAD_EQUAL;
                break;
            case DWT.KEYPAD_MULTIPLY:
                naturalKey = SpecialKey.NUMPAD_MULTIPLY;
                break;
            case DWT.KEYPAD_SUBTRACT:
                naturalKey = SpecialKey.NUMPAD_SUBTRACT;
                break;
            case DWT.NUM_LOCK:
                naturalKey = SpecialKey.NUM_LOCK;
                break;
            case DWT.PAGE_DOWN:
                naturalKey = SpecialKey.PAGE_DOWN;
                break;
            case DWT.PAGE_UP:
                naturalKey = SpecialKey.PAGE_UP;
                break;
            case DWT.PAUSE:
                naturalKey = SpecialKey.PAUSE;
                break;
            case DWT.PRINT_SCREEN:
                naturalKey = SpecialKey.PRINT_SCREEN;
                break;
            case DWT.SCROLL_LOCK:
                naturalKey = SpecialKey.SCROLL_LOCK;
                break;
            default:
                naturalKey = CharacterKey
                        .getInstance((char) (accelerator & 0xFFFF));
            }
        }

        return KeyStroke.getInstance(modifierKeys, naturalKey);
    }

    /**
     * <p>
     * Converts the given event into an DWT accelerator value -- considering the
     * modified character with the shift modifier. This is the third accelerator
     * value that should be checked.
     * </p>
     * <p>
     * For example, on a standard US keyboard, "Ctrl+Shift+5" would be viewed as
     * "Ctrl+Shift+%".
     * </p>
     * 
     * @param event
     *            The event to be converted; must not be <code>null</code>.
     * @return The combination of the state mask and the unmodified character.
     */
    public static int convertEventToModifiedAccelerator(Event event) {
        int modifiers = event.stateMask & DWT.MODIFIER_MASK;
        char character = topKey(event);
        return modifiers + toUpperCase(character);
    }

    /**
     * <p>
     * Converts the given event into an DWT accelerator value -- considering the
     * unmodified character with all modifier keys. This is the first
     * accelerator value that should be checked. However, all alphabetic
     * characters are considered as their uppercase equivalents.
     * </p>
     * <p>
     * For example, on a standard US keyboard, "Ctrl+Shift+5" would be viewed as
     * "Ctrl+Shift+5".
     * </p>
     * 
     * @param event
     *            The event to be converted; must not be <code>null</code>.
     * @return The combination of the state mask and the unmodified character.
     */
    public static int convertEventToUnmodifiedAccelerator(Event event) {
        return convertEventToUnmodifiedAccelerator(event.stateMask,
                event.keyCode);
    }

    /**
     * <p>
     * Converts the given state mask and key code into an DWT accelerator value --
     * considering the unmodified character with all modifier keys. All
     * alphabetic characters are considered as their uppercase equivalents.
     * </p>
     * <p>
     * For example, on a standard US keyboard, "Ctrl+Shift+5" would be viewed as
     * "Ctrl+Shift+5".
     * </p>
     * 
     * @param stateMask
     *            The integer mask of modifiers keys depressed when this was
     *            pressed.
     * @param keyCode
     *            The key that was pressed, before being modified.
     * @return The combination of the state mask and the unmodified character.
     */
    private static int convertEventToUnmodifiedAccelerator(int stateMask,
            int keyCode) {
        int modifiers = stateMask & DWT.MODIFIER_MASK;
        int character = keyCode;
        return modifiers + toUpperCase(character);
    }

    /**
     * <p>
     * Converts the given event into an DWT accelerator value -- considering the
     * unmodified character with all modifier keys. This is the first
     * accelerator value that should be checked. However, all alphabetic
     * characters are considered as their uppercase equivalents.
     * </p>
     * <p>
     * For example, on a standard US keyboard, "Ctrl+Shift+5" would be viewed as
     * "Ctrl+%".
     * </p>
     * 
     * @param event
     *            The event to be converted; must not be <code>null</code>.
     * @return The combination of the state mask and the unmodified character.
     */
    public static int convertEventToUnmodifiedAccelerator(KeyEvent event) {
        return convertEventToUnmodifiedAccelerator(event.stateMask,
                event.keyCode);
    }

    /**
     * Converts the given event into an DWT accelerator value -- considering
     * the modified character without the shift modifier. This is the second
     * accelerator value that should be checked. Key strokes with alphabetic
     * natural keys are run through <code>convertEventToUnmodifiedAccelerator</code>
     * 
     * @param event
     *            The event to be converted; must not be <code>null</code>.
     * @return The combination of the state mask without shift, and the
     *         modified character.
     */
    public static int convertEventToUnshiftedModifiedAccelerator(Event event) {
        // Disregard alphabetic key strokes.
        if (Character.isLetter((char) event.keyCode)) {
            return convertEventToUnmodifiedAccelerator(event);
        }

        int modifiers = event.stateMask & (DWT.MODIFIER_MASK ^ DWT.SHIFT);
        char character = topKey(event);
        return modifiers + toUpperCase(character);
    }

    /**
     * Given a key stroke, this method provides the equivalent DWT accelerator
     * value. The functional inverse of <code>convertAcceleratorToKeyStroke</code>.
     * 
     * @param keyStroke
     *            The key stroke to convert; must not be <code>null</code>.
     * @return The DWT accelerator value
     */
    public static final int convertKeyStrokeToAccelerator(
            final KeyStroke keyStroke) {
        int accelerator = 0;
        final Iterator iterator = keyStroke.getModifierKeys().iterator();

        while (iterator.hasNext()) {
            final ModifierKey modifierKey = (ModifierKey) iterator.next();

            if (modifierKey is ModifierKey.ALT) {
                accelerator |= DWT.ALT;
            } else if (modifierKey is ModifierKey.COMMAND) {
                accelerator |= DWT.COMMAND;
            } else if (modifierKey is ModifierKey.CTRL) {
                accelerator |= DWT.CTRL;
            } else if (modifierKey is ModifierKey.SHIFT) {
                accelerator |= DWT.SHIFT;
            }
        }

        final NaturalKey naturalKey = keyStroke.getNaturalKey();

        if (naturalKey instanceof CharacterKey) {
            accelerator |= ((CharacterKey) naturalKey).getCharacter();
        } else if (naturalKey instanceof SpecialKey) {
            final SpecialKey specialKey = (SpecialKey) naturalKey;

            if (specialKey is SpecialKey.ARROW_DOWN) {
                accelerator |= DWT.ARROW_DOWN;
            } else if (specialKey is SpecialKey.ARROW_LEFT) {
                accelerator |= DWT.ARROW_LEFT;
            } else if (specialKey is SpecialKey.ARROW_RIGHT) {
                accelerator |= DWT.ARROW_RIGHT;
            } else if (specialKey is SpecialKey.ARROW_UP) {
                accelerator |= DWT.ARROW_UP;
            } else if (specialKey is SpecialKey.END) {
                accelerator |= DWT.END;
            } else if (specialKey is SpecialKey.F1) {
                accelerator |= DWT.F1;
            } else if (specialKey is SpecialKey.F10) {
                accelerator |= DWT.F10;
            } else if (specialKey is SpecialKey.F11) {
                accelerator |= DWT.F11;
            } else if (specialKey is SpecialKey.F12) {
                accelerator |= DWT.F12;
            } else if (specialKey is SpecialKey.F2) {
                accelerator |= DWT.F2;
            } else if (specialKey is SpecialKey.F3) {
                accelerator |= DWT.F3;
            } else if (specialKey is SpecialKey.F4) {
                accelerator |= DWT.F4;
            } else if (specialKey is SpecialKey.F5) {
                accelerator |= DWT.F5;
            } else if (specialKey is SpecialKey.F6) {
                accelerator |= DWT.F6;
            } else if (specialKey is SpecialKey.F7) {
                accelerator |= DWT.F7;
            } else if (specialKey is SpecialKey.F8) {
                accelerator |= DWT.F8;
            } else if (specialKey is SpecialKey.F9) {
                accelerator |= DWT.F9;
            } else if (specialKey is SpecialKey.HOME) {
                accelerator |= DWT.HOME;
            } else if (specialKey is SpecialKey.INSERT) {
                accelerator |= DWT.INSERT;
            } else if (specialKey is SpecialKey.PAGE_DOWN) {
                accelerator |= DWT.PAGE_DOWN;
            } else if (specialKey is SpecialKey.PAGE_UP) {
                accelerator |= DWT.PAGE_UP;
            }
        }

        return accelerator;
    }

    private static final IKeyFormatter NATIVE_FORMATTER = new NativeKeyFormatter();

    /**
     * Provides an instance of <code>IKeyFormatter</code> appropriate for the
     * current instance.
     * 
     * @return an instance of <code>IKeyFormatter</code> appropriate for the
     *         current instance; never <code>null</code>.
     */
    public static IKeyFormatter getKeyFormatterForPlatform() {
        return NATIVE_FORMATTER;
    }

    /**
     * Makes sure that a fully-modified character is converted to the normal
     * form. This means that "Ctrl+" key strokes must reverse the modification
     * caused by control-escaping. Also, all lower case letters are converted
     * to uppercase.
     * 
     * @param event
     *            The event from which the fully-modified character should be
     *            pulled.
     * @return The modified character, uppercase and without control-escaping.
     */
    private static char topKey(Event event) {
        char character = event.character;
        bool ctrlDown = (event.stateMask & DWT.CTRL) !is 0;

        if (ctrlDown && event.character !is event.keyCode
                && event.character < 0x20) {
            character += 0x40;
        }

        return character;
    }

    /**
     * Makes the given character uppercase if it is a letter.
     * 
     * @param keyCode
     *            The character to convert.
     * @return The uppercase equivalent, if any; otherwise, the character
     *         itself.
     */
    private static int toUpperCase(int keyCode) {
        // Will this key code be truncated?
        if (keyCode > 0xFFFF) {
            return keyCode;
        }

        // Downcast in safety. Only make characters uppercase.
        char character = (char) keyCode;
        return Character.isLetter(character) ? Character.toUpperCase(character)
                : keyCode;
    }

    /**
     * This class should never be instantiated.
     */
    private SWTKeySupport() {
        // This class should never be instantiated.
    }
}
