/*******************************************************************************
 * Copyright (c) 2005, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.preferences.WorkingCopyManager;

import dwt.dwthelper.utils;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.osgi.service.prefs.BackingStoreException;

import dwtx.core.runtime.preferences.IEclipsePreferences;
import dwtx.ui.internal.preferences.WorkingCopyPreferences;

/**
 * WorkingCopyManager is a concrete implementation of an
 * IWorkingCopyManager.
 * <p>
 * This class is not intended to be sub-classed by clients.
 * </p>
 * @since 3.2
 */
public class WorkingCopyManager implements IWorkingCopyManager{

    private static final String EMPTY_STRING = "";//$NON-NLS-1$
    // all working copies - maps absolute path to PreferencesWorkingCopy instance
    private Map workingCopies = new HashMap();

    
    /* (non-Javadoc)
     * @see dwtx.ui.preferences.IWorkingCopyManager#getWorkingCopy(dwtx.core.runtime.preferences.IEclipsePreferences)
     */
    public IEclipsePreferences getWorkingCopy(IEclipsePreferences original) {
        if (original instanceof WorkingCopyPreferences) {
            throw new IllegalArgumentException("Trying to get a working copy of a working copy"); //$NON-NLS-1$
        }
        String absolutePath = original.absolutePath();
        IEclipsePreferences preferences = (IEclipsePreferences) workingCopies.get(absolutePath);
        if (preferences is null) {
            preferences = new WorkingCopyPreferences(original, this);
            workingCopies.put(absolutePath, preferences);
        }
        return preferences;
    }

    
    /* (non-Javadoc)
     * @see dwtx.ui.preferences.IWorkingCopyManager#applyChanges()
     */
    public void applyChanges() throws BackingStoreException {
        Collection values = workingCopies.values();
        WorkingCopyPreferences[] valuesArray = (WorkingCopyPreferences[]) values.toArray(new WorkingCopyPreferences[values.size()]);
        for (int i = 0; i < valuesArray.length; i++) {
            WorkingCopyPreferences prefs = valuesArray[i];
            if (prefs.nodeExists(EMPTY_STRING)) 
                prefs.flush();
        }
    }

}
